
	LIST P=PIC18F452

			; **********************
			; MIDI System Controller
			; Revision 3_5
			; December 2006
			; **********************


#INCLUDE <C:\Program Files\MICROCHIP\MPASM Suite\p18f452.inc>

; conditional compilation
; =======================

UNIT	EQU	000H
DEBUG	EQU	080H	; debug mode
RS232	EQU	040H	; enable rs232 output of midi (38400 baud required)
ICD	EQU	020H

; development
; ===========

; MPLAB IDE 7.00
; MPLAB ICD2
; MPLAB MASM


; configuration
; =============

;Program Configuration Register 1H
		__CONFIG    _CONFIG1H, _OSCS_OFF_1H & _HSPLL_OSC_1H

;Program Configuration Register 2L
		__CONFIG    _CONFIG2L, _BOR_ON_2L & _BORV_27_2L & _PWRT_ON_2L

;Program Configuration Register 3H
		__CONFIG    _CONFIG3H, _CCP2MX_ON_3H

;Program Configuration Register 6L
		__CONFIG    _CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L & _WRT2_OFF_6L & _WRT3_OFF_6L 

;Program Configuration Register 6H
		__CONFIG    _CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H & _WRTD_OFF_6H

;Program Configuration Register 7L
		__CONFIG    _CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L & _EBTR2_OFF_7L & _EBTR3_OFF_7L

;Program Configuration Register 7H
		__CONFIG    _CONFIG7H, _EBTRB_OFF_7H


.IF (UNIT & DEBUG)

;Program Configuration Register 2H
		__CONFIG    _CONFIG2H, _WDT_OFF_2H & _WDTPS_128_2H


;Program Configuration Register 5L
		__CONFIG    _CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L & _CP2_OFF_5L & _CP3_OFF_5L 

;Program Configuration Register 5H
		__CONFIG    _CONFIG5H, _CPB_OFF_5H & _CPD_OFF_5H

.ELSE

;Program Configuration Register 2H
		__CONFIG    _CONFIG2H, _WDT_ON_2H & _WDTPS_128_2H


;Program Configuration Register 5L
		__CONFIG    _CONFIG5L, _CP0_ON_5L & _CP1_ON_5L & _CP2_ON_5L & _CP3_ON_5L 

;Program Configuration Register 5H
		__CONFIG    _CONFIG5H, _CPB_ON_5H & _CPD_OFF_5H


.ENDIF

.IF (UNIT & ICD)

;Program Configuration Register 4L
		__CONFIG    _CONFIG4L, _STVR_ON_4L & _LVP_OFF_4L & _DEBUG_ON_4L

.ELSE

;Program Configuration Register 4L
		__CONFIG    _CONFIG4L, _STVR_ON_4L & _LVP_OFF_4L & _DEBUG_OFF_4L

.ENDIF

; hardware
; ========

; 10 MHz xtal with PLL enabled (40 MHz effective)
; i2c operates at 400KHz
; keypad i2c addresses 10h,12h,14h
; function key  i2c address 16h


; firmware
; ========

; fsr2 used exclusively in timer interrupt (high priority)
; 2 level interrupts


; revision 1.1 - REPORTED AS 2.1
; ============

; analogue buffers re-located to group calibration/analogue setup
; checksums applied to calibration/analogue set-up
; velocity tables held in ram and loaded at power up
; checksum individually applied to each velocity table
; upload/download of velocity tables
; event options for hot key disabled until supported
; default string generated at power up
;	if link-back on rs232 will default everything

; revision 1.2 - reported as 2.2
; ============

; control of LCD backlight

; revision 1.3 - reported as 2.3
; ============

; function keys now report both up and down (not programme keys)
; detect both play and util down to put into idle state
; wake up on play button only
; in idle all interrupts serviced but data discarded
; lcd blanked with back light off
; reported as AXiS 64

; revision 1_5 - reported as 2.5
; ============

; revision 1_4 dumped
;	16 key buffer to hold current key down status
;	for use if transpose / channel change while key down

; on transpose / channel change - simply send all notes off
; on programme change key - force play screen with cursor -> program


; revision 3_0 
; ============

; revsion level 2 skipped to keep sync reported and actual

; usb connection via CP2102 at 57600 baud
; all comms now 7 data with even parity
; Baud rate selection removed from utility setup
; utility screens re-assigned

;	80 	backlight
;	81 	Hotkey
;	82	Noise Margin
;	83	Calibrate
;	84	Default

; support for programme update via serial port / usb
; all code filled to 64 byte boundaries with nops

; revision 3_1
; ============

; program key change - play screen with cursor in previous position

; mkey3_1
; outer key down - send note on regardless of inner switch
; key release on out switch to attampt to stop sustain

; revision 3_2
; ============

; no changes to control firmware - just upgrade revision to 3.2

; mkey3.2
; new status table to include stat3 - outer up, inner down
;			      stat4 - outer down, inner up
; sweep included in scan - if stat3 and inner key up then send key off
;			   and revert to stat0

; revision 3_3
; ============

; mfunc3_1
; analogue channels for unused and breath control
; no longer sent by function key processor

; mctrl3_3
; breath control removed from midi and utility screens (hardware_tab)
; new default to put 12 bits noise margin to both foot controls

; utility screens re-assigned
;	80 	backlight
;	81 	Hotkey
;	82	Noise Margin
;	83	Calibrate
;	84	Firmware and serial number
;	85	Default

; revision 3_4
; ============

; foot controls swapped to match rear-panel markings

; mkey3.3 as part of release
; counts set to zero on key release and return to status zero
; count also reset on outer switch closure only

; cross-channel interference in calibration removed
; pitch bend calibration fixed
; first move flag introduced to force load of max and min on first
;	movement of control in calibration

; revision 3_5
; ============







#DEFINE	MIDI_OUT	PORTA,2		; MIDI
#DEFINE	MIDI_IN		PORTA,1
#DEFINE	LED_DEBUG	PORTA,4

					; interrupt request
#DEFINE	KEY_INT0	PORTB,0		; function key
#DEFINE	KEY_INT3	PORTB,1		; key 3
#DEFINE	KEY_INT2	PORTB,2		; key 2
#DEFINE	KEY_INT1	PORTB,3		; key 1

#DEFINE	LED_YEL		PORTB,4
#DEFINE	LED_GRN		PORTB,5

#DEFINE	FIN		PORTC,0		; ffram
#DEFINE	CS1		PORTC,1
#DEFINE	FOUT		PORTC,2
#DEFINE	FCLK		PORTD,0
#DEFINE	CS0		PORTD,1

#DEFINE	BACKLIGHT	PORTD,2		; LCD backlight (active high)

#DEFINE	E		PORTE,0		; display control
#DEFINE	RW		PORTE,1
#DEFINE	RS		PORTE,2

; Constants
; =========

K1ADD		EQU	10H			; i2c addresses
K2ADD		EQU	12H
K3ADD		EQU	14H
FKADD		EQU	16H

RPT_SHORT	EQU	005H		; repeat interval for = / - keys
RPT_LONG	EQU	020H

					; fram commands

F_WREN		EQU	006H		; write enable latch
F_WRDI		EQU	004H		; write disable
F_RDSR		EQU	005H		; read status register
F_WRSR		EQU	001H		; write status register
F_READ		EQU	003H		; read memory
F_WRITE		EQU	002H		; write memory


; Register Map
; ============

; BANK 0
; ======

MBFRIN		EQU	00H		; midi buffer pointers/size
MBFROUT		EQU	01H
MBFRSIZ		EQU	02H
MDATA		EQU	03H		; midi output data byte
MRTIME		EQU	04H		; midi real time output byte
					; output next regardless of buffer state
MCNT		EQU	05H		; midi output bit count
					; 1-8 for data (lsb first)
MSTAT		EQU	06H		; current status of midi out
ASCNT		EQU	07H		; active sensing count reg - inc on 4 msec

I2CADD		EQU	08H		; i2c write address

MSEC4		EQU	09H		; inc up to 4 msec
COUNT		EQU	0AH		; dec on 4 msec basis to zero
					; timeout flag set
DDATA		EQU	0BH		; display data
CSRPOS		EQU	0CH		; lcd cursor position
CSRSAV		EQU	0DH		; lcd cursor return position

DEBUG_REG	EQU	0FH

KEY_ID		EQU	10H		; fkey id
RPTKEY		EQU	11H		; hold status of up/down keys - repeatable
RPT_TIME	EQU	12H		; multiples of 50 msec to key repeat
BFR_PTR		EQU	13H		; point to analogue buffer in setup

MAX		EQU	14H		; max and min buffers
MIN		EQU	15H

MIDI_STATUS	EQU	18H		; save midi status byte
SAV_STATUS	EQU	19H
TEMP_CHAN	EQU	1AH		; temp channel used in hotkey strings
TEMP_STAT	EQU	1BH

ADDH		EQU	1CH		; fram address
ADDL		EQU	1DH
DBYTE		EQU	1EH		; fram data byte
BITPTR		EQU	1FH		; bit pointer for transfer

F0SAVL		EQU	30H		; save fsr0 in velocoity table load
F0SAVH		EQU	31H

SIPTR		EQU	32H		; serial input pointer
RXDATA		EQU	33H		; last rec'd character
TXCSUM		EQU	34H		; transmit checksum

SUBSEC		EQU	35H		; inc on 50msec basis to count of 20
LCD_TIMEOUT	EQU	36H		; remaining time in seconds

PROGRAM		EQU	38H
MIDI_BANK	EQU	39H
GROUP		EQU	3AH
MODE		EQU	3BH
MODE_SAV	EQU	3CH		; save mode in play screen
				
					; non-volatile - save in fram

CHANNEL		EQU	40H		; channel 0 - 15
TRANSPOSE	EQU	41H		; transpose -24 TO +25 decimal
VELTBL		EQU	42H		; velocity table
FLAG_STAT	EQU	43H		; running status, active sending etc
HOT_KEY		EQU	44H
BAUDRATE	EQU	45H
LCD_TIME	EQU	46H		; lcd backlight
					; free data

CSUM0		EQU	4FH		; comp sum of data 40h - 4eh inc

PARITY		EQU	50H		; used to check even parity
PCNT		EQU	51H
PVAL		EQU	52H

TPARITY		EQU	53H		; used to genertate parity
TPCNT		EQU	54H
TPVAL		EQU	55H
TXDATA		EQU	56H

DFLT_CNT	EQU	57H		; count key strokes in default routine

TPTRH		EQU	59H
TPTRL		EQU	5AH

CRCH		EQU	5BH		; flash crc
CRCL		EQU	5CH
DLY1		EQU	5DH		; non interrupt delay regs
DLY2		EQU	5EH

MIDI_STRING	EQU	60H		; 16 byte midi string to load to mbfr

WK0		EQU	70H		; wk regs
WK1		EQU	71H
WK2		EQU	72H
WK3		EQU	73H

MSB		EQU	74H		; 2 byte register for data conversion
LSB		EQU	75H

WSAV		EQU	78H		; save during low priority interrupt
F1LSAV		EQU	79H
F1HSAV		EQU	7AH
STATSAV		EQU	7BH

FLAG0		EQU	7CH		; general purpose flag registers
FLAG1		EQU	7DH
FLAG2		EQU	7EH
FLAG3		EQU	7FH


; define flags

					; flag_stat held in fram

#DEFINE	RUN_STAT_FLG	FLAG_STAT,0	; set for running status
#DEFINE	ACTIVE_FLG	FLAG_STAT,1	; set for active sensing

					; all other flags volatile

#DEFINE	TMR0_FLG	FLAG0,0		; set during timer0 interrupt
#DEFINE	MDATA_FLG	FLAG0,1		; set during transmission of midi byte
#DEFINE	RTIME_FLG	FLAG0,2		; set if real time data
#DEFINE	I2CERR_FLG	FLAG0,3		; set for i2c error
#DEFINE	TIMEOUT_FLG	FLAG0,4		; set on timeout of count register
#DEFINE	TMR1_FLG	FLAG0,5		; set during timer1 interrupt
#DEFINE	MIDI_INH_FLG	FLAG0,6		; set to inhibit output during buffer load

#DEFINE	BGND_FLG	FLAG1,0		; set to update background in dsp_update
#DEFINE	ONE_FLG		FLAG1,1		; set to display from 1 for channel etc.
#DEFINE	RPT_FLG		FLAG1,2		; set if repeat key down
#DEFINE	NEG_FLG		FLAG1,3		; set if negative
#DEFINE	CAL_FLG		FLAG1,4		; set during calibration
#DEFINE	HIGHMEM_FLG	FLAG1,5		; set for high memory select
#DEFINE	BFR_FULL_FLG	FLAG1,6		; set if rs232 input biffer full

#DEFINE	PLAY_FLG	FLAG2,0		; set if play key down
#DEFINE IDLE_FLG	FLAG2,1		; set in idle mode
#DEFINE	SHUTDOWN_FLG	FLAG2,2		; set if shutdown screen active
#DEFINE	RESTORE_FLG	FLAG2,3		; set to restore settings
#DEFINE DEFAULT_FLG	FLAG2,4		; set to enable default from util
#DEFINE	FIRST_MOVE_FLG	FLAG2,5		; set for first move of control in calibration


#DEFINE	DEBUG_FLG	FLAG3,7		; used for debug


; BANK 1
; ======
; MIDI output buffer

MBFR		EQU	100H


; BANK 2 LOWER
; ============
; ANALOGUE Buffers

ASSIGN_BFR	EQU	200H		; assigned controller
INVERT_BFR	EQU	210H		; invert control
NOISE_BFR	EQU	220H		; noise margin
SCALE_BFR	EQU	230H		; calibration scale factor * 256
OFFSET_BFR	EQU	240H		; calibration offset (atod)
ATOD_BFR	EQU	250H		; atod input
OUTPUT_BFR	EQU	260H		; scaled analogue output
SERNO		EQU	278H		; serial number fom eeprom
ACSUM		EQU	24FH		; place csum at top of offset buffer





; BANKS 2 UPPER, BANKS 3 & 4
; ==========================
; Velocity tables

VTBL1		EQU	280H
VTBL2		EQU	300H
VTBL3		EQU	380H
VTBL4		EQU	400H
VTBL5		EQU	480H

; BANK 5
; ======
; RS232 input buffer

IPBFR		EQU	500H

OPBFR		EQU	580H		; used to xfer packet in pgm mode



			; ******
			; Macros
			; ******

DSTR	MACRO STRINGPTR
					; String display
					; tblptru assumed zero
	MOVLW	HIGH STRINGPTR
	MOVWF	TBLPTRH,A
	MOVLW	LOW STRINGPTR
	MOVWF	TBLPTRL,A
	ENDM

APTR	MACRO BUFFER
					; point to analogue buffer
					; tablat holds channel no.
	LFSR	FSR0,BUFFER
	MOVF	TABLAT,W,A
	IORWF	FSR0L,F,A
	ENDM


; Code
; ====
			; **************
			; Power On Reset
			; **************

	ORG	0000H

PON_RESET:


	GOTO	INIT			; jump to initialisation routine
	NOP				; nops to pack hex file
	NOP


	ORG	0008H

INT_HIGH:

	CALL	TIMER0_INT		; can only be timer0 interrupt
	RETFIE	FAST			; restore w, bsr, status
	NOP				; pack hex file
	NOP
	NOP
	NOP
	NOP
	ORG	0018H

	GOTO	INT_LOW

	NOP				; pack hex file
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	ORG	0040H

ERASE_VECTOR:

INT_LOW:
					; low priority interrupts use FSR1
	MOVWF	WSAV,A			; save w, fsr1, status
	MOVFF	STATUS,STATSAV
	MOVFF	FSR1L,F1LSAV
	MOVFF	FSR1H,F1HSAV

	BTFSC	PIR1,RCIF,A
	CALL	SERIAL_INT

	BTFSC	PIR1,TMR1IF,A
	CALL	TIMER1_INT


	MOVFF	F1LSAV,FSR1L		; restore w, fsr1, status
	MOVFF	F1HSAV,FSR1H
	MOVFF	STATSAV,STATUS
	MOVF	WSAV,W,A

	RETFIE

INIT:
	
	CALL	CLEAR_RAM		; clear ram 000h thro' 5ffh
	
	BSF	RCON,IPEN,A		; 2 level priority on interrupts
	CLRF	EECON1,A		; ensure access to eeprom

					; set port i/o
					; port a
	MOVLW	00EH
	MOVWF	ADCON1,A		; all digital except an0
	MOVLW	014H
	MOVWF	LATA,A
	MOVLW	B'00101011'		; midi output, debug led output
	MOVWF	TRISA,A			; all input
					; port b
	MOVLW	0FFH
	MOVWF	LATB,A
	MOVLW	B'11001111'		; LED outputs - rest input
	MOVWF	TRISB,A			; 
					
					; port c
	MOVLW	0FFH
	MOVWF	LATC,A
	MOVLW	B'10111001'		; TXD , FOUT, CS1 as output
	MOVWF	TRISC,A
					; port d
	MOVLW	B'00001111'		; lcd data low, fram lines high
	MOVWF	LATD,A			; backlight high (on)
	MOVLW	B'00001000'		; LCD data and fram clk, cs0, backligh
	MOVWF	TRISD,A			; all outputs
					; port e
	CLRF	LATE,A			; all output
	CLRF	TRISE,A
					; init serial i/o
	CALL	INIT_UART		; will set baud rate to 57600
					
					; init i2c as master
INIT0:
	MOVLW	080H			; slew rate disabled
	MOVWF	SSPSTAT,A

	MOVLW	018H			; 400 khz
	MOVWF	SSPADD,A

	CLRF	SSPCON2,A
	MOVLW	B'00101000'		; sspen, master mode fosc/4*(sspadd+1)
	MOVWF	SSPCON1,A

					; init timer 0

	MOVLW	B'01000000'		; 8 bit, internal clk, prescale 1:2
	MOVWF	T0CON,A
	BSF	T0CON,TMR0ON,A		; start timer 0

					; init timer 1

	MOVLW	B'10110000'		; 16 bit, internal clk, 8:1 prescale
	MOVWF	T1CON,A
	BSF	T1CON,TMR1ON,A		; start timer 1

					; enable interrupts

	BSF	INTCON,TMR0IE,A		; timer 0
					; external interrupts not enabled
					; simply check flags in intcon/intcon3
	CLRF	INTCON2,A		; ext int on falling edge
					; weak pull-up on port b
	BSF	INTCON2,TMR0IP,A	; set timer0 to high priority
	CLRF	INTCON3,A

	CLRF	IPR1,A			; ensure all other interrupts low priority
	CLRF	IPR2,A

	BSF	INTCON,GIEH,A		; enable high priority interrupts
	BSF	INTCON,GIEL,A		; enable low priority interrupts
	
	BSF	PIE1,RCIE,A		; serial rec interrupt
	BSF	PIE1,TMR1IE,A		; timer1 interrupt

	CLRF	TBLPTRU,A		; assume all maps and messages
					; in range 0000h to ffffh



			; ************
			; Main Routine
			; ************

MAIN:

	CALL	DSP_INIT		; initialise lcd display

	DSTR	M_ID			; intro screen
	CALL	DSP_STRING
	CALL	DELAY_2SEC

	CALL	CLEAR_WPROT		; ensure no write protect in fram
	CALL	RESTORE_SETTINGS	; get settings from fram
					; default if necessary
	CALL	CSR_ON
	CLRF	MODE_SAV,A		; point to programmme
	CALL	DSP_PLAY_SCREEN		; paint play screen and update
					; program, bank and group
	BCF	LED_GRN,A		; power led on

	CALL	SET_LCD			; set backlight timeout

	CALL	READ_EEPROM		; get serial number from eeprom

MAIN1:

	BTFSC	SSPCON1,SSPOV,A		; test overflow of i2c
	GOTO	I2C_ERROR1

	BTFSC	SSPCON1,WCOL,A		; test write collision
	GOTO	I2C_ERROR2

	CLRWDT

	BTFSC	BFR_FULL_FLG		; test data in serial buffer
	CALL	ACTION_SERIAL

	BSF	LED_DEBUG,A		; test standard key interrupts
	MOVLW	K1ADD
	BTFSS	KEY_INT1,A
	CALL	ACTION_STDKEY

	BSF	LED_DEBUG,A
	MOVLW	K2ADD
	BTFSS	KEY_INT2,A
	CALL	ACTION_STDKEY

	BSF	LED_DEBUG,A
	MOVLW	K3ADD
	BTFSS	KEY_INT3,A
	CALL	ACTION_STDKEY

	BSF	LED_DEBUG,A
	MOVLW	FKADD
	BTFSS	KEY_INT0,A
	CALL	ACTION_FKEY

	BTFSC	RESTORE_FLG,A
	GOTO	INIT			; go re-initialise processor

	BTFSC	IDLE_FLG,A		; no repeats in idle mode
	GOTO	MAIN1

	BTFSS	RPT_FLG,A		; test action required on repeat keys
	GOTO	MAIN1

	MOVF	RPT_TIME,W,A
	BTFSC	STATUS,Z,A
	CALL	ACTION_REPEAT

	GOTO	MAIN1


			; ******************
			; I2C Error Routines
			; ******************


I2C_ERROR1:				; overlow on receive

	MOVF	SSPBUF,W,A		; reset BF in sspstat
	BCF	SSPCON1,SSPOV,A
	GOTO	INIT0

I2C_ERROR2:				; collision detect on transmit

	BCF	SSPCON1,WCOL,A
	GOTO	INIT0



			; **************************
			; Restore / Default settings
			; **************************

RESTORE_SETTINGS:			; get settings from fram

	BCF	HIGHMEM_FLG,A		; use lower memory
	
	LFSR	FSR0,020H		; test default level message
	CALL	READ@FSR0
	
	DSTR	M_DFLT_LEVEL
	LFSR	FSR0,020H
	MOVLW	010H
	MOVWF	WK2,A			; length of string

REST1:
	
	TBLRD*+				; read from pgm memory
	MOVF	TABLAT,W,A
	XORWF	POSTINC0,W,A		; xor internal memory
	BTFSS	STATUS,Z,A
	GOTO	REST2			; fail - clear all memory and default all settings
	DECFSZ	WK2,F,A
	GOTO	REST1

	GOTO	RESTORE_SETUP		; OK - go load set-up etc.

REST2:					; default all
	CALL	CLEAR_FRAM		; clear all fram

	DSTR	M_DFLT_LEVEL		; copy string to internal ram
	LFSR	FSR0,020H
	MOVLW	010H
	MOVWF	WK2,A
	CALL	GET_STRING	
	LFSR	FSR0,020H		; write default level string at 020H
	CALL	WRITE@FSR0
					; fall thro to test set-up etc

RESTORE_SETUP:

	LFSR	FSR0,CHANNEL		; get channel ~ baudrate
	CALL	READ@FSR0		; 16 bytes to channel

	LFSR	FSR0,CHANNEL		; verify checksum
	MOVLW	00FH
	MOVWF	WK2,A
	CALL	GET_CSUM		; return csum in w reg, fsr0 -> csum

	XORWF	POSTINC0,W,A
	BTFSS	STATUS,Z,A
	CALL	DEFAULT_SETUP

RESTORE_ANALOGUE:

	LFSR	FSR0,ASSIGN_BFR		; copy all analogue buffers from fram
	CALL	READ@FSR0
	LFSR	FSR0,INVERT_BFR
	CALL	READ@FSR0
	LFSR	FSR0,NOISE_BFR
	CALL	READ@FSR0
	LFSR	FSR0,SCALE_BFR
	CALL	READ@FSR0
	LFSR	FSR0,OFFSET_BFR
	CALL	READ@FSR0
	
	LFSR	FSR0,ASSIGN_BFR
	MOVLW	04FH
	MOVWF	WK2,A
	CALL	GET_CSUM

	XORWF	POSTINC0,W,A
	BTFSS	STATUS,Z
	CALL	DEFAULT_ANALOGUE

RESTORE_VELOCITY:

	LFSR	FSR0,VTBL1
	DSTR	DFLT_VTBL1
	MOVLW	'1'
	MOVWF	WK3,A			; display table number
	CALL	RESTORE_VEL1

	LFSR	FSR0,VTBL2
	DSTR	DFLT_VTBL2
	INCF	WK3,F,A			; inc table number
	CALL	RESTORE_VEL1

	LFSR	FSR0,VTBL3
	DSTR	DFLT_VTBL3
	INCF	WK3,F,A
	CALL	RESTORE_VEL1

	LFSR	FSR0,VTBL4
	DSTR	DFLT_VTBL4
	INCF	WK3,F,A
	CALL	RESTORE_VEL1

	LFSR	FSR0,VTBL5
	DSTR	DFLT_VTBL5
	INCF	WK3,F,A
	CALL	RESTORE_VEL1
	RETURN

RESTORE_VEL1:
	
	MOVFF	FSR0L,F0SAVL		; save fsr0
	MOVFF	FSR0H,F0SAVH

	CALL	READ128@FSR0		; read table from fram
	MOVFF	F0SAVL,FSR0L
	MOVFF	F0SAVH,FSR0H
	MOVLW	07FH
	MOVWF	WK2,A	
	CALL	GET_CSUM

	XORWF	POSTINC0,W,A
	BTFSS	STATUS,Z,A
	CALL	DEFAULT_VTBL
	RETURN
	
					; default routines


DEFAULT_SETUP:				; channel ~ baudrate + free all zero

	LFSR	FSR0,CHANNEL
	MOVLW	00FH
	MOVWF	WK2,A

DFLT_SETUP1:
	CLRF	POSTINC0,A		; set all to 00h
	DECFSZ	WK2,F,A
	GOTO	DFLT_SETUP1

	CALL	DSP_HOME		; display default message
	DSTR	M_DEFAULT
	CALL	DSP_STRING	
	MOVLW	040H			; line 2
	CALL	DSP_CSR_ADD
	DSTR	M_SETUP			; setup message
	CALL	DSP_STRING
	CALL	DELAY_2SEC

WRITE_SETUP:				; get csum and write setup to fram

	LFSR	FSR0,CHANNEL
	MOVLW	00FH
	MOVWF	WK2,A
	CALL	GET_CSUM
	MOVWF	INDF0,A
	
	LFSR	FSR0,CHANNEL
	CALL	WRITE@FSR0		; place in fram
	RETURN

DEFAULT_ANALOGUE:			; default all five analogue buffers

	
	DSTR	DFLT_ASS_MAP		; point to default assign map
	LFSR	FSR0,ASSIGN_BFR
	MOVLW	010H
	MOVWF	WK2,A
	CALL	GET_STRING

	DSTR	DFLT_NULL_MAP		; default invert buffer
	LFSR	FSR0,INVERT_BFR
	MOVLW	010H
	MOVWF	WK2,A
	CALL	GET_STRING

	DSTR	DFLT_NOISE_MAP		; default noise to 6 bits
	LFSR	FSR0,NOISE_BFR		; for all but foot controls rev3.3
	MOVLW	010H
	MOVWF	WK2,A
	CALL	GET_STRING

	DSTR	DFLT_SCL_MAP		; default scale to 0100H per channel
	LFSR	FSR0,SCALE_BFR
	MOVLW	010H
	MOVWF	WK2,A
	CALL	GET_STRING

	DSTR	DFLT_NULL_MAP		; default offset to zero
	LFSR	FSR0,OFFSET_BFR
	MOVLW	010H
	MOVWF	WK2,A
	CALL	GET_STRING

	CALL	DSP_HOME		; display default message
	DSTR	M_DEFAULT
	CALL	DSP_STRING	
	MOVLW	040H			; line 2
	CALL	DSP_CSR_ADD
	DSTR	M_ANALOGUE		; setup message
	CALL	DSP_STRING
	CALL	DELAY_2SEC

WRITE_ANALOGUE:				; get csum and write analogue to fram

	LFSR	FSR0,ASSIGN_BFR		; get and place csum
	MOVLW	04FH
	MOVWF	WK2,A
	CALL	GET_CSUM
	MOVWF	INDF0,A

	LFSR	FSR0,ASSIGN_BFR		; write default values to fram
	CALL	WRITE@FSR0	
	LFSR	FSR0,INVERT_BFR
	CALL	WRITE@FSR0
	LFSR	FSR0,NOISE_BFR
	CALL	WRITE@FSR0
	LFSR	FSR0,SCALE_BFR
	CALL	WRITE@FSR0
	LFSR	FSR0,OFFSET_BFR
	CALL	WRITE@FSR0
	RETURN


DEFAULT_VTBL:				; f0savl, f0savh hold fsr0
					; table pointer already set
					; wk3 holds table number (ascii)
	MOVFF	F0SAVL,FSR0L		; set fsr0
	MOVFF	F0SAVH,FSR0H		
	
	MOVLW	080H			; 128 bytes
	MOVWF	WK2,A
	CALL	GET_STRING		; get table from pgm mem

	CALL	DSP_HOME		; display default mesage
	DSTR	M_DEFAULT
	CALL	DSP_STRING
	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_VTBL
	CALL	DSP_STRING

	MOVLW	056H			; position cursor for table number
	CALL	DSP_CSR_ADD
	MOVF	WK3,W,A
	CALL	DSP_WRITE_DATA
	CALL	DELAY_2SEC

WRITE_VTBL:

	MOVFF	F0SAVL,FSR0L		; set fsr0
	MOVFF	F0SAVH,FSR0H		
	MOVLW	07FH
	MOVWF	WK2,A
	CALL	GET_CSUM
	MOVWF	INDF0,A

	MOVFF	F0SAVL,FSR0L		; set fsr0
	MOVFF	F0SAVH,FSR0H		
	CALL	WRITE128@FSR0
	RETURN
	

			; ***********
			; Default All
			; ***********

DEFAULT_ALL:

	CALL	CSR_OFF

	CALL	DEFAULT_SETUP		; default setup and analogue
	CALL	DEFAULT_ANALOGUE

	LFSR	FSR0,VTBL1		; default each velocity table in turn
	DSTR	DFLT_VTBL1
	MOVLW	'1'
	MOVWF	WK3,A
	CALL	ACT_DFLT2

	LFSR	FSR0,VTBL2
	DSTR	DFLT_VTBL2
	CALL	ACT_DFLT2

	LFSR	FSR0,VTBL3		
	DSTR	DFLT_VTBL3
	CALL	ACT_DFLT2

	LFSR	FSR0,VTBL4		
	DSTR	DFLT_VTBL4
	CALL	ACT_DFLT2

	LFSR	FSR0,VTBL5		
	DSTR	DFLT_VTBL5
	CALL	ACT_DFLT2

	CALL	CSR_ON
	CLRF	MODE_SAV,A		; point to programmme
	CALL	DSP_PLAY_SCREEN		; paint play screen and update

	RETURN


ACT_DFLT2:

	MOVFF	FSR0L,F0SAVL		; save fsr0
	MOVFF	FSR0H,F0SAVH
	CALL	DEFAULT_VTBL
	INCF	WK3,F,A			; increment displayed table
	RETURN


			; ******************
			; Backlight Routines
			; ******************

SET_LCD:

	BTFSC	LCD_TIME,2,A		; test 4 = off
	GOTO	SET_OFF
	
	BSF	BACKLIGHT		; backlight on
	DSTR	LCD_MAP
	MOVF	LCD_TIME,W,A
	ADDWF	TBLPTRL,F,A
	MOVLW	000H
	ADDWFC	TBLPTRH,F,A

	TBLRD*

	MOVFF	TABLAT,LCD_TIMEOUT
	RETURN
	
SET_OFF:

	BCF	BACKLIGHT,A
	RETURN



			; **********************
			; Data transfer routines
			; **********************

GET_STRING:		; tblptr -> string in pgm memory
			; fsr0 -> internal memory
			; wk2 = length of data
	TBLRD*+
	MOVFF	TABLAT,POSTINC0		; xfer data
	DECFSZ	WK2,F,A
	GOTO	GET_STRING
	RETURN

GET_CSUM:		; fsr0 -> data
			; wk2 holds byte count prior to csum
			; return w = csum
			; fsr0 -> csum location
	MOVLW	000H

GET_CSUM1:
	ADDWF	POSTINC0,W,A
	DECFSZ	WK2,F,A
	GOTO	GET_CSUM1

	XORLW	0FFH			; complement csum
	RETURN				




			; *********************
			; Serial port interrupt
			; *********************

SERIAL_INT:				; use fsr1 - already saved prior to call		
					; to serial interrupt
	MOVFF	RCREG,RXDATA		; get data and store in rxdata

	CALL	PARITY_CHECK
	BTFSS	STATUS,Z,A
	RETURN


	MOVLW	07FH			; mask parity bit
	ANDWF	RXDATA,F,A
	MOVF	SIPTR,W,A
	BNZ	SERIAL10

	MOVLW	001H			; test valid header
	XORWF	RXDATA,W,A
	BTFSS	STATUS,Z,A		; match - place header in buffer
	RETURN

SERIAL10:				; place data in buffer

	BTFSC	BFR_FULL_FLG,A
	RETURN				; ignore data if buffer already full

	LFSR	FSR1,IPBFR
	MOVFF	SIPTR,FSR1L		; set low pointer	
	MOVFF	RXDATA,INDF1		; data to buffer

	INCF	SIPTR,F,A		; inc pointer
	MOVLW	003H
	CPFSGT	SIPTR,A			; skip if siptr > 3
	RETURN				; header, length, id only

	LFSR	FSR1,IPBFR+1		; get packet length
	MOVF	INDF1,W,A
	XORWF	SIPTR,W,A
	BTFSC	STATUS,Z,A		; end of packet
	BSF	BFR_FULL_FLG,A		; say buffer full
	RETURN



			; ********************
			; Serial Data Routines
			; ********************



INIT_UART:				; rev 3_0 fixed 57600 baud

	MOVLW	02AH
	MOVWF	SPBRG,A
	MOVLW	024H
	MOVWF	TXSTA,A
	MOVLW	090H			; spen, cren
	MOVWF	RCSTA,A
	RETURN


ACTION_SERIAL:

					; nb no ack/nak of data in normal mode

	DECF	SIPTR,W,A		; get byte count for csum verification
	MOVWF	WK2,A

	LFSR	FSR0,IPBFR		; verify checksum - simple addition header
					; to last byte of data
	MOVLW	000H			; init csum
ACT_SER1:

	ADDWF	POSTINC0,W,A
	DECFSZ	WK2,F,A
	GOTO	ACT_SER1

	ANDLW	07FH			; 7 data only

	XORWF	INDF0,W,A		; fsr0 -> csum
	BTFSS	STATUS,Z,A
	GOTO	ACT_NAK			; csum fail
	
					; valid packet

	MOVFF	IPBFR+2,WK0		; get packet id
					; save id
	BTFSS	WK0,6,A			; test address 4 or 5
	GOTO	ACT_NAK			; otherwise ignore

	BTFSC	WK0,4,A			; test velocity table
	GOTO	ACT_VELOCITY

	MOVF	WK0,W,A
	XORLW	048H			; test enq
	BNZ	ACT_SER1A

	MOVLW	04EH			; say normal mode - control processor
	CALL	SEND_PKT
	GOTO	ACT_SERX

ACT_SER1A:

	MOVF	WK0,W,A
	XORLW	04BH
	BNZ	ACT_SERX

	CALL	DSP_HOME		; place pgm message on lcd
	DSTR	M_PGM_MODE
	CALL	DSP_STRING

	CALL	COPY_NORM_VECTOR
	CLRWDT
	CALL	WRITE_BOOT_VECTOR	; write boot vector at start of flash

	BCF	RCON,IPEN,A		; for icd - ensure no unused interrupts
	BCF	PIE1,TMR1IE,A
	BCF	INTCON,TMR0IE,A

	RESET

ACT_NAK:				; nak unidentified packets

	MOVLW	04DH
	CALL	SEND_PKT
	GOTO	ACT_SERX


ACT_VELOCITY:
					; velocity table routines

	MOVF	WK0,W,A			; get packet id
	ANDLW	007H			; mask request bit
	MOVWF	WK1,A			; save table id in wk1

	MOVLW	005H
	CPFSLT	WK1,A
	GOTO	ACT_NAK 		; invlaid table id

	MOVLW	080H			; table size
	MULWF	WK1,A

	LFSR	FSR0,VTBL1		; FSR0 -> start of table
	MOVF	PRODL,W,A
	ADDWF	FSR0L,F,A

	MOVF	PRODH,W,A
	ADDWFC	FSR0H,F,A

	BTFSC	WK0,3,A			; test up/download
	GOTO	ACT_SER10

					; UPLOAD
					; fsr0 -> start of table
	LFSR	FSR1,IPBFR+1		; verify length
	MOVF	POSTINC1,W,A		; FSR1 -> id
	XORLW	068H
	BTFSS	STATUS,Z,A
	GOTO	ACT_SERX		; ignore if not true length

	MOVFF	FSR0L,F0SAVL		; save fsr0
	MOVFF	FSR0H,F0SAVH


	INCF	FSR1L,F,A		; fsr1 -> start of data
	MOVLW	064H			; length of data
	MOVWF	WK2,A
	CLRF	TXCSUM,A		; generate csum during load

ACT_SER2:

	MOVF	POSTINC1,W,A		; transfer data
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A		; update csum
	DECFSZ	WK2,F,A
	GOTO	ACT_SER2

	MOVLW	01BH			; fill remaining 27 bytes with 01h
	MOVWF	WK2,A
	MOVLW	001H
ACT_SER3:
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A
	DECFSZ	WK2,F,A
	GOTO	ACT_SER3

	MOVLW	0FFH			; place csum
	XORWF	TXCSUM,W,A
	MOVWF	INDF0,A

	CALL	WRITE_VTBL		; save in fram
	GOTO	ACT_SERX

ACT_SER10:				; REQUEST DOWNLOAD
					; fsr0 -> start of table

	CLRF	TXCSUM,A		; reset checksum
	MOVLW	001H			; header
	CALL	XMIT
	MOVLW	068H			; packet length = 104
	CALL	XMIT

	MOVF	WK1,W,A			; get table number
	CALL	XMIT

	MOVLW	064H			; 100 bytes data
	MOVWF	WK2,A

ACT_SER11:

	MOVF	POSTINC0,W,A		; get data from table
	CALL	XMIT
	DECFSZ	WK2,F,A
	GOTO	ACT_SER11

	MOVF	TXCSUM,W,A
	CALL	XMIT			; fall thro'
	
ACT_SERX:

	CLRF	SIPTR,A			; reset pointer
	BCF	BFR_FULL_FLG,A		; re-enable input
	RETURN

			; *************
			; fram routines
			; *************


CLEAR_WPROT:				; clear write protect on both rams

	BCF	HIGHMEM_FLG,A
	CALL	CLRP1
	BSF	HIGHMEM_FLG,A
					; fall thro to clear ram 2
CLRP1:

	CALL	SELECT_RAM		; chip select ram according highmem_flg

	MOVLW	F_WREN			; enable writes
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE
	BSF	CS0,A
	BSF	CS1,A

	CALL	SELECT_RAM

	MOVLW	F_WRSR
	CALL	WRITE_BYTE
	CLRF	DBYTE,A			; clear all protect
	CALL	WRITE_BYTE
	BSF	CS0,A
	BSF	CS1,A

	RETURN

CLEAR_FRAM:

	DSTR	DFLT_NULL_MAP		; ensure nulls to output buffer
	LFSR	FSR0,OUTPUT_BFR
	MOVLW	010H
	MOVWF	WK2,A
	CALL	GET_STRING

	BCF	HIGHMEM_FLG,A		; low memory
	CLRF	ADDH,A
	CLRF	ADDL,A
	CALL	SELECT_RAM
	CALL	CLR_FRAM1

	BSF	HIGHMEM_FLG,A
	CLRF	ADDH,A
	CLRF	ADDL,A
	CALL	SELECT_RAM
	CALL	CLR_FRAM1
	BCF	HIGHMEM_FLG,A
	RETURN

CLR_FRAM1:

	CLRWDT

	MOVLW	010H			; 16 bytes
	MOVWF	WK2,A			; byte count in wk2

	LFSR	FSR0,OUTPUT_BFR		; already cleared to zeros
	CALL	WRITE_RAM

	MOVLW	010H
	ADDWF	ADDL,F,A
	MOVLW	000H
	ADDWFC	ADDH,F,A

	BTFSS	ADDH,5,A		; test end of fram
	GOTO	CLR_FRAM1
	RETURN

WRITE128@FSR0:				; used for velocity tables

	MOVFF	FSR0H,ADDH		; internal address=fram address
	MOVFF	FSR0L,ADDL

	MOVLW	080H			; 128 bytes
	MOVWF	WK2,A			; byte count in wk2
	GOTO	WRITE_RAM


WRITE@FSR0:

	MOVFF	FSR0H,ADDH		; internal address=fram address
	MOVFF	FSR0L,ADDL

	MOVLW	010H			; 16 bytes
	MOVWF	WK2,A			; byte count in wk2

WRITE_RAM:				; transfer 16 bytes @ FSR0
					; transfer always on 16 byte boundary

	CALL	SELECT_RAM		; chip select ram

	MOVLW	F_WREN			; write enable
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE
	BSF	CS0,A
	BSF	CS1,A

	CALL	SELECT_RAM		; select ram again
	
	MOVLW	F_WRITE 		; write command
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE

	MOVF	ADDH,W,A		; high address byte
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE

	MOVF	ADDL,W,A		; low address byte
	ANDLW	0F0H			; ensure 16 byte boundary
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE

WRAM1:
	MOVFF	POSTINC0,DBYTE		; place data in dbyte
	CALL	WRITE_BYTE
	DECFSZ	WK2,F,A			; test 16 bytes written
	GOTO	WRAM1

	BSF	CS0,A			; deselect rams
	BSF	CS1,A
	RETURN

READ128@FSR0:				; used for velocity tables

	MOVFF	FSR0H,ADDH		; internal address=fram address
	MOVFF	FSR0L,ADDL

	MOVLW	080H
	MOVWF	WK2,A			; byte count
	GOTO	READ_RAM

READ@FSR0:

	MOVFF	FSR0H,ADDH		; internal address=fram address
	MOVFF	FSR0L,ADDL

	MOVLW	010H
	MOVWF	WK2,A			; byte count

READ_RAM:				; transfer 16 bytes @ fsr0
					
	CALL	SELECT_RAM

	MOVLW	F_READ
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE

	MOVF	ADDH,W,A
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE

	MOVF	ADDL,W,A
	MOVWF	DBYTE,A
	CALL	WRITE_BYTE
	
RRAM1:
	CALL	READ_BYTE
	MOVFF	DBYTE,POSTINC0
	DECFSZ	WK2,F,A
	GOTO	RRAM1

	BSF	CS0,A			; deselect rams
	BSF	CS1,A
	RETURN


SELECT_RAM:

	BCF	FCLK,A			; ensure clock low
	BTFSS	HIGHMEM_FLG,A
	BCF	CS0,A
	BTFSC	HIGHMEM_FLG,A
	BCF	CS1,A
	RETURN

					; nb data rise and fall times 1uSec
WRITE_BYTE:				; put dbyte to bus msb first

	MOVLW	008H			; 8 bits
	MOVWF	BITPTR,A

WB1:
	BTFSC	DBYTE,7,A		; test msb
	GOTO	WB_HIGH

WB_LOW:

	BCF	FOUT,A			; low bit
	GOTO	WB2	

WB_HIGH:

	BSF	FOUT,A			; high bit
WB2:
	CALL	FRAM_DELAY
	BSF	FCLK,A
	NOP
	NOP
	NOP
	NOP
	BCF	FCLK,A
	CALL	FRAM_DELAY
	RLNCF	DBYTE,F,A		; get next bit
	DECFSZ	BITPTR,F,A
	GOTO	WB1
	RETURN

	
READ_BYTE:			; get single byte from fram

	MOVLW	008H
	MOVWF	BITPTR,A
	CLRF	DBYTE,A
READBYTE1:
	CALL	FRAM_DELAY
	BCF	STATUS,C,A
	BTFSC	FIN,A			; sample input from fram
	BSF	STATUS,C,A		; c = status of current bit

	RLCF	DBYTE,F,A		; rotate c to lsb

	BSF	FCLK,A			; clock
	NOP
	NOP
	NOP
	NOP
	BCF	FCLK,A
	CALL	FRAM_DELAY

	DECFSZ	BITPTR,F,A
	GOTO	READBYTE1
	RETURN


FRAM_DELAY:
	MOVLW	005H
	MOVWF	WK0,A
FD_LOOP:

	DECFSZ	WK0,F,A
	GOTO	FD_LOOP
	RETURN


			; **********************
			; standard key interrupt
			; **********************

ACTION_STDKEY:

	BCF	LED_DEBUG,A		; debug led on

	MOVWF	I2CADD,A		; enter w reg holds i2caddress
	CALL	READ_I2C

	MOVF	WK0,W,A			; verify csum
	XORWF	WK1,W,A
	CPFSEQ	WK2,A
	RETURN				; bad csum

ACTION_TEST:

	BTFSC	IDLE_FLG,A		; test idle mode
	RETURN

	LFSR	FSR1,MIDI_STRING	; point to midi string

	MOVF	CHANNEL,W,A
	IORLW	090H			; say key press
	CALL	PUT_MIDI_STATUS

	MOVF	WK0,W,A			; get key id
	ADDWF	TRANSPOSE,W,A		; transpose note if required
	MOVWF	POSTINC1,A

	MOVF	WK1,W,A			; test up or down
	BTFSS	STATUS,Z,A
	GOTO	ACTION_STDKEY_DOWN

	CLRF	POSTINC1,A		; say released
	CALL	PUT_LENGTH
	RETURN


ACTION_STDKEY_DOWN:


; rev 1.1 velocity table now held in ram

	LFSR	FSR0,VTBL1		; fsro -> vel tables
	
	MOVLW	080H			; bytes per table
	MULWF	VELTBL,A

	MOVF	PRODL,W,A		; get start of active table
	ADDWF	FSR0L,F,A
	MOVF	PRODH,W,A
	ADDWFC	FSR0H,F,A

					; FSR0L will be 00h or 80h
	MOVF	WK1,W,A			; get offset into table	
	ANDLW	07FH			; should not be necessary as 0<vel<128
	ADDWF	FSR0L,F,A
	MOVFF	INDF0,POSTINC1
	CALL	PUT_LENGTH
	RETURN


			; **********************
			; function key interrupt
			; **********************

ACTION_IDLE:

	MOVLW	004H	; test play
	CPFSEQ	KEY_ID,A
	RETURN				; ignore if not play

	BCF	IDLE_FLG,A
	BSF	RESTORE_FLG,A		; say re-initialise processor
	RETURN




ACTION_REPEAT:		; repeat timeout

	MOVLW	002H			; say down key
	BTFSS	RPTKEY,0,A
	MOVLW	003H
	MOVWF	KEY_ID,A		; load key_id

	MOVLW	RPT_SHORT		; load short delay
	MOVWF	RPT_TIME,A

	CALL	SET_LCD			; set lcd timeout

	GOTO	ACT_FKEY1

ACTION_FKEY:

	BCF	LED_DEBUG,A

	MOVLW	FKADD			; get key data from fkey processor
	MOVWF	I2CADD,A
	CALL	READ_I2C

	MOVF	WK0,W,A			; verify csum
	XORWF	WK1,W,A
	CPFSEQ	WK2,A
	RETURN				; bad csum

	MOVFF	WK0,KEY_ID		; save key id

	BTFSC	IDLE_FLG,A		; test idle mode
	GOTO	ACTION_IDLE

	BTFSC	KEY_ID,4,A		; test analogue input
	GOTO	ANALOGUE_INPUT

	CALL	SET_LCD			; set lcd backlight

	BTFSC	KEY_ID,3,A		; test programme key
	GOTO	ACTION_PGM_KEY		; programme keys reported down only

	MOVLW	004H			; test play key
	CPFSEQ	KEY_ID,A
	GOTO	ACT_FKEY2

	BCF	PLAY_FLG,A		; say key released
	MOVF	WK1,W,A			; test direction of play key
	BTFSS	STATUS,Z,A
	BSF	PLAY_FLG

ACT_FKEY2:				; test direction

	BTFSC	KEY_ID,2,A		; skip if tab, + -	
	GOTO	ACT_FKEY3
	BTFSC	KEY_ID,1,A		; skip to act_fkey3 if tab
	GOTO	ACT_FKEY4		; allow up and down for + -

ACT_FKEY3:				; discard if key release

	MOVF	WK1,W,A
	BTFSC	STATUS,Z,A
	RETURN				; ignore key release


ACT_FKEY4:				; routine from rev 1_2 and ealier

	MOVLW	007H			; test hot key
	CPFSLT	KEY_ID,A
	GOTO	ACTION_HOTKEY

	BTFSC	KEY_ID,2,A
	GOTO	ACT_FKEY1
	BTFSS	KEY_ID,1,A		; test + / -
	GOTO	ACT_FKEY1

	MOVF	WK1,W,A			; test direction
	BTFSS	STATUS,Z
	GOTO	ACT_FKEY0

	CLRF	RPTKEY,A		; key released - clear rptkey and exit
	BCF	RPT_FLG,A
	RETURN

ACT_FKEY0:

	MOVLW	001H			; bit 0 - down
	BTFSC	KEY_ID,0,A
	MOVLW	002H			; bit 1 - up
	MOVWF	RPTKEY,A
	BSF	RPT_FLG,A
	MOVLW	RPT_LONG
	MOVWF	RPT_TIME,A

ACT_FKEY1:
	BTFSC	SHUTDOWN_FLG,A
	GOTO	ACTION_SHUTDOWN

	BTFSC	MODE,7,A		; test screen mode
	GOTO	ACTION_UTIL
	BTFSC	MODE,6,A
	GOTO	ACTION_MIDI


; ---------------------- PLAY SCREEN --------------------------------------	
			
ACTION_PLAY:			
;==========
	MOVLW	004H			; test play, midi, util
	CPFSLT	KEY_ID,A
	GOTO	MODE_CHANGE_PLAY

	BTFSC	MODE,1,A		; test function
	GOTO	ACTION_GROUP		; 00 - programme
	BTFSC	MODE,0,A		; 01 - bank
	GOTO	ACTION_BANK		; 10 - group
	
					; back tab, tab, down, up
					; key_id 00, 01, 02, 03

ACTION_PROGRAM:
;=============

	BTFSS	KEY_ID,1,A
	GOTO	ACT_PGM_TAB

	BTFSS	KEY_ID,0,A		; either inc or dec program
	DECF	PROGRAM,F,A
	BTFSC	KEY_ID,0,A
	INCF	PROGRAM,F,A
	MOVLW	07FH
	ANDWF	PROGRAM,F,A

	CALL	PUT_PROGRAM_CHANGE	; output new programme to midi
	CALL	DSP_PROGRAM
	CALL	RESTORE_CSR
	RETURN

ACT_PGM_TAB:				; only fwd tab valid

	BTFSS	KEY_ID,0,A
	RETURN
	INCF	MODE,F,A
	MOVFF	MODE,MODE_SAV		; save play mode
	CALL	SET_PLAY_CSR
	RETURN



ACTION_BANK:
;==========

	BTFSS	KEY_ID,1,A
	GOTO	ACT_BANK_TAB

	BTFSS	KEY_ID,0,A		; either inc or dec bank
	DECF	MIDI_BANK,F,A
	BTFSC	KEY_ID,0,A
	INCF	MIDI_BANK,F,A
	MOVLW	07FH
	ANDWF	MIDI_BANK,F,A
	
	CALL	PUT_BANK_SEL
	CALL	DSP_BANK
	CALL	RESTORE_CSR
	RETURN

ACT_BANK_TAB:

	BTFSC	KEY_ID,0,A		; inc/dec function in mode
	INCF	MODE,F,A
	BTFSS	KEY_ID,0,A
	DECF	MODE,F,A


	MOVFF	MODE,MODE_SAV		; save
	CALL	SET_PLAY_CSR		; place cursor
	RETURN

ACTION_GROUP:
;===========

	BTFSS	KEY_ID,1,A
	GOTO	ACT_GROUP_TAB

	BTFSS	KEY_ID,0,A		; either inc or dec bank
	DECF	GROUP,F,A
	BTFSC	KEY_ID,0,A
	INCF	GROUP,F,A
	MOVLW	00FH			; only 16 groups
	ANDWF	GROUP,F,A
	CALL	DSP_GROUP
	CALL	RESTORE_CSR
	RETURN

ACT_GROUP_TAB:

	BTFSC	KEY_ID,0,A		; back tab only
	RETURN
	DECF	MODE,F,A
	MOVFF	MODE,MODE_SAV		; save play mode
	CALL	SET_PLAY_CSR
	RETURN

MODE_CHANGE_PLAY:			; key_id 4,5,6
;===============

	BTFSC	KEY_ID,0,A
	CALL	DSP_MIDI_SCREEN
	BTFSS	KEY_ID,1,A
	RETURN				; test play key still down

	BTFSC	PLAY_FLG,A
	GOTO	POWER_OFF
	CALL	DSP_UTIL_SCREEN
	RETURN


POWER_OFF:

	CALL	DSP_HOME
	DSTR	M_SHUTDOWN		; shutdown screen
	CALL	DSP_STRING

	BSF	SHUTDOWN_FLG,A
	RETURN				; ignore play



; ---------------------- MIDI SCREEN ---------------------------------------

			
ACTION_MIDI:			
;==========

	MOVLW	004H			; test play, midi, util
	CPFSLT	KEY_ID,A
	GOTO	MODE_CHANGE_MIDI

	MOVLW	040H			; midi transpose
	CPFSEQ	MODE,A
	GOTO	ACT_MIDI_1

ACTION_TRANSPOSE:			; only + / - keys
;===============

	CALL	PUT_ALL_NOTES_OFF	; rev 1_5

	BTFSC	KEY_ID,0,A
	GOTO	ACT_TRAN_INC

ACT_TRAN_DEC:

	DECF	TRANSPOSE,F,A
	MOVLW	0E7H			; test less than -24
	CPFSEQ	TRANSPOSE,A
	GOTO	ACT_TRAN_RET
	INCF	TRANSPOSE,F,A

ACT_TRAN_RET:

	CALL	DSP_TRANSPOSE
	RETURN

ACT_TRAN_INC:

	INCF	TRANSPOSE,F,A
	MOVLW	01AH			; test over 25
	CPFSEQ	TRANSPOSE,A
	GOTO	ACT_TRAN_RET
	DECF	TRANSPOSE,F,A
	GOTO	ACT_TRAN_RET


ACT_MIDI_1:

	MOVLW	041H			; midi channel
	CPFSEQ	MODE,A
	GOTO	ACT_MIDI_2

ACTION_CHANNEL:				; only + / - keys
;=============

	CALL	PUT_ALL_NOTES_OFF	; rev 1_5

	BTFSC	KEY_ID,0,A
	INCF	CHANNEL,F,A
	BTFSS	KEY_ID,0,A
	DECF	CHANNEL,F,A
	MOVLW	00FH			; range 0 - 15
	ANDWF	CHANNEL,F,A

	CALL	DSP_CHANNEL
	RETURN

ACT_MIDI_2:

	MOVLW	042H
	CPFSEQ	MODE,A
	GOTO	ACT_MIDI_3

ACTION_ASSIGN:				; assign controllers to hardware
;============

	LFSR	FSR0,ASSIGN_BFR
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	BTFSS	KEY_ID,1,A		; test tab
	GOTO	ACT_ASSTAB

	BTFSC	KEY_ID,0,A
	GOTO	ACT_ASS_INC

	DECF	INDF0,F,A
	MOVF	INDF0,W,A
	BNZ	ACT_ASS0
	MOVLW	081H			; pitchbend
	GOTO	ACT_ASS1		; go place in buffer

ACT_ASS0:

	BTFSS	INDF0,5,A		; test dec out of range (3fh, 7fh)
	GOTO	ACT_ASS3

	MOVLW	05FH			; test 7fh
	BTFSS	INDF0,6,A
	MOVLW	01FH

ACT_ASS1:
	MOVWF	INDF0,A
ACT_ASS3:
	CALL	DSP_CONTROLLER
	RETURN

ACT_ASS_INC:

	INCF	INDF0,F,A
	MOVF	INDF0,W,A
	XORLW	082H
	BNZ	ACT_ASS2

	MOVLW	001H			; modulation
	GOTO	ACT_ASS1

ACT_ASS2:

	BTFSS	INDF0,5,A		; test inc out of range 20, 60, 82h
	GOTO	ACT_ASS3

	MOVLW	040H			; test 20 / 60 and set 40, 80h
	BTFSC	INDF0,6,A
	MOVLW	080H
	GOTO	ACT_ASS1


ACT_ASSTAB:

	CALL	HARDWARE_TAB
	CALL	DSP_HARDWARE
	CALL	DSP_CONTROLLER
	RETURN


ACT_MIDI_3:

	MOVLW	043H
	CPFSEQ	MODE,A
	GOTO	ACT_MIDI_4


ACTION_INVERT:
;============

	LFSR	FSR0,INVERT_BFR
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	BTFSS	KEY_ID,1,A		; test tab
	GOTO	ACT_INVTAB
					; must be + / -
	BTG	INDF0,0,A
	CALL	DSP_INVERT
	RETURN

ACT_INVTAB:

	CALL	HARDWARE_TAB
	CALL	DSP_HARDWARE
	CALL	DSP_INVERT
	RETURN


ACT_MIDI_4:

	MOVLW	044H
	CPFSEQ	MODE,A
	GOTO	ACT_MIDI_5


ACTION_VELOCITY:
;==============

	BTFSS	KEY_ID,1,A		; onlt + / - keys active
	RETURN

	BTFSC	KEY_ID,0,A
	GOTO	ACT_VELINC

	DECF	VELTBL,F,A
	BTFSS	VELTBL,7,A		; test underflow
	GOTO	ACT_VEL_RET

	MOVLW	004H
	MOVWF	VELTBL,A
	GOTO	ACT_VEL_RET

ACT_VELINC:

	INCF	VELTBL,F,A
	MOVF	VELTBL,W,A		; test overflow
	XORLW	005H
	BTFSC	STATUS,Z,A
	CLRF	VELTBL,A
					; fall thro to update display
ACT_VEL_RET:

	CALL	DSP_VELOCITY
	RETURN


ACT_MIDI_5:

	MOVLW	045H
	CPFSEQ	MODE,A
	GOTO	ACT_MIDI_6

ACTION_RUNSTAT:
;=============

	BTFSS	KEY_ID,1,A		; only + / -
	RETURN

	BTG	RUN_STAT_FLG,A

	CLRF	WK0,A
	BTFSC	RUN_STAT_FLG,A
	INCF	WK0,F,A
	CALL	DSP_ON_OFF
	RETURN
	
ACT_MIDI_6:

	MOVLW	046H
	CPFSEQ	MODE,A
	GOTO	ACT_MIDI_7

ACTION_ACTIVE_SENSE:
;==================

	BTFSS	KEY_ID,1,A		; only + / -
	RETURN

	BTG	ACTIVE_FLG,A

	CLRF	WK0,A
	BTFSC	ACTIVE_FLG,A
	INCF	WK0,F,A
	CALL	DSP_ON_OFF
	RETURN



ACT_MIDI_7:				; should not reach here
					; include for expansion of midi function
	RETURN


MODE_CHANGE_MIDI:			; key_id 4,5,6
;===============

	BTFSC	KEY_ID,1,A
	GOTO	MIDI2UTIL		; util key
	BTFSS	KEY_ID,0,A
	GOTO	MIDI2PLAY		; play key
	
	INCF	MODE,F,A		; midi key
	MOVLW	047H			; MIDI modes 40 - 46h
	CPFSEQ	MODE,A
	GOTO	DSP_MIDI_SUBSCREEN	; return from routine exits this sub
					; fall thro to play screen

MIDI2PLAY:
	CALL	SAVE_MIDI
	CALL	DSP_PLAY_SCREEN
	RETURN


MIDI2UTIL:
	CALL	SAVE_MIDI
	CALL	DSP_UTIL_SCREEN
	RETURN

SAVE_MIDI:

	CALL	WRITE_SETUP		; update csum and write to fram
	CALL	WRITE_ANALOGUE

	RETURN	


; ---------------------- UTILITY SCREEN -------------------------------------
			
ACTION_UTIL:				; utility screen
;==========

	MOVLW	004H			; test play, midi, util
	CPFSLT	KEY_ID,A
	GOTO	MODE_CHANGE_UTIL


	MOVLW	080H			; lcd backlight
	CPFSEQ	MODE,A
	GOTO	ACT_UTIL_1

ACTION_LCDTIME:
; ============

	BTFSS	KEY_ID,1,A		; test tab keys
	RETURN

					; + / - keys only
	BTFSS	KEY_ID,0,A
	GOTO	ACT_LCD1
					; +
	INCF	LCD_TIME,F,A
	MOVLW	005H
	CPFSLT	LCD_TIME,A
	CLRF	LCD_TIME,A
	GOTO	ACT_LCD2

ACT_LCD1:				; -

	DECF	LCD_TIME,F,A
	MOVLW	004H			; pre-load off condition
	BTFSC	LCD_TIME,7,A
	MOVWF	LCD_TIME,A

ACT_LCD2:

	CALL	DSP_LCDTIME
	RETURN

ACT_UTIL_1:

	MOVLW	081H			; hotkey
	CPFSEQ	MODE,A
	GOTO	ACT_UTIL_2
	

ACTION_UTIL_HOTKEY:
;=================

	BTFSS	KEY_ID,1,A		; test tab
	RETURN				; no action for tab keys

	BTFSC	KEY_ID,0,A
	GOTO	ACT_HOT_INC

					; range 0 - 4 / 80h - 0ffh

	DECF	HOT_KEY,F,A
	MOVF	HOT_KEY,W,A
;	XORLW	07FH
	XORLW	0FFH			; rev 1.1 no events
	BTFSS	STATUS,Z,A
	GOTO	ACT_HOT_RET

	MOVLW	004H			; panic
	MOVWF	HOT_KEY,A
	

ACT_HOT_RET:

	CALL	DSP_HOT_KEY
	RETURN

ACT_HOT_INC:

	INCF	HOT_KEY,F,A
	MOVF	HOT_KEY,W,A
	XORLW	005H
	BTFSS	STATUS,Z,A
	GOTO	ACT_HOT_RET
;	MOVLW	080H
;	MOVWF	HOT_KEY,A
	CLRF	HOT_KEY,A		; rev 1.1
	GOTO	ACT_HOT_RET


ACT_UTIL_2:

	MOVLW	082H			; noise margin
	CPFSEQ	MODE,A
	GOTO	ACT_UTIL_3

ACTION_UTIL_NOISE:
;================

	LFSR	FSR0,NOISE_BFR
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A		; fsr0 -> buffer location

	BTFSS	KEY_ID,1,A		; test tab
	GOTO	ACT_NOISETAB

	BTFSS	KEY_ID,0,A		; test inc / dec count
	DECF	INDF0,F,A
	BTFSC	KEY_ID,0,A
	INCF	INDF0,F,A

	MOVLW	007H
	ANDWF	INDF0,F,A		; mask to 3 bits

	CALL	DSP_NOISE
	RETURN

ACT_NOISETAB:

	CALL	HARDWARE_TAB
	CALL	DSP_HARDWARE
	CALL	DSP_NOISE
	RETURN

ACT_UTIL_3:

	MOVLW	083H			; calibration
	CPFSEQ	MODE,A
	GOTO	ACT_UTIL_4

ACTION_CALIBRATE:
;===============

	BTFSS	KEY_ID,1,A		; test tab
	GOTO	ACT_CALTAB
					; + / - keys

	MOVF	MAX,W,A			; test max=min
	XORWF	MIN,W,A
	BTFSC	STATUS,Z,A
	RETURN				; do nothing if max=min

	CALL	CALIBRATE		; use displayed max min to 
					; calibrate control
	MOVLW	040H			; clear max min to ack
	CALL	DSP_CSR_ADD
	DSTR	M_CALIBRATE
	CALL	DSP_STRING
	RETURN


ACT_CALTAB:

	CALL	HARDWARE_TAB
	CALL	DSP_HARDWARE

	LFSR	FSR0,ATOD_BFR		; get channel 0 atod
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	MOVF	INDF0,W,A
	MOVWF	MAX,A
	MOVWF	MIN,A
	CALL	DSP_MAX_MIN

	BSF	FIRST_MOVE_FLG,A	; say next movement is first
	RETURN


ACT_UTIL_4:				; display firmware and serial no.

	MOVLW	084H
	CPFSEQ	MODE,A	
	GOTO	ACT_UTIL_5
	
ACTION_SERNO:
; ============

	RETURN				; no key action for serial no display
	


ACT_UTIL_5:

	MOVLW	085H			; default
	CPFSEQ	MODE,A
	GOTO	ACT_UTIL_6

ACTION_DEFAULT:
; ============
	
	MOVF	KEY_ID,W,A
	XORLW	003H			; test + key
	BNZ	UTIL2PLAY		; exit if not + key
	
	INCF	DFLT_CNT,F,A
	MOVF	DFLT_CNT,W,A
	XORLW	058H			; test end of bottom line
	BZ	ACT_UTIL_DFLT		; go default all
	CALL	DSP_UTIL_SUBSCREEN
	RETURN
	
ACT_UTIL_DFLT:
		
	CALL	DEFAULT_ALL
	RETURN
	

ACT_UTIL_6:

	RETURN

MODE_CHANGE_UTIL:

					; 80 thro 845then exit

	BCF	CAL_FLG,A		; say not calibration
	BTFSC	KEY_ID,0,A
	GOTO	UTIL2MIDI		; MIDI
	BTFSS	KEY_ID,1,A
	GOTO	UTIL2PLAY		; play
	
	INCF	MODE,F,A		; util

	MOVLW	040H			; use as csr address for bottom line
	MOVWF	DFLT_CNT,A


	MOVF	MODE,W,A
	XORLW	083H
	BTFSC	STATUS,Z,A		; test switch to calibration screen
	BSF	FIRST_MOVE_FLG,A
	
	MOVLW	086H			; 80 - 85 valid
	CPFSEQ	MODE,A
	GOTO	DSP_UTIL_SUBSCREEN	; return from routine exits this sub
					; fall thro' to play screen	

UTIL2PLAY:
	CALL	SAVE_UTIL
	CALL	DSP_PLAY_SCREEN
	RETURN


UTIL2MIDI:
	CALL	SAVE_UTIL
	CALL	DSP_MIDI_SCREEN
	RETURN

SAVE_UTIL:
	
	CALL	WRITE_SETUP		; update csum and write to fram
	CALL	WRITE_ANALOGUE

	RETURN	


; ----------------------- SHUTDOWN ----------------------------------------

ACTION_SHUTDOWN:

	MOVLW	002H		; test - key
	CPFSEQ	KEY_ID,A
	GOTO	ACT_SD1

	BCF	SHUTDOWN_FLG,A
	CALL	DSP_PLAY_SCREEN
	RETURN

ACT_SD1:
	MOVLW	003H		; test + key
	CPFSEQ	KEY_ID,A
	RETURN

	BCF	SHUTDOWN_FLG,A
	BSF	IDLE_FLG,A
	BCF	BACKLIGHT,A

	CALL	DSP_CLEAR
	CALL	CSR_OFF

	CLRF	RPTKEY,A		;  - clear rptkey
	BCF	RPT_FLG,A

	RETURN


; ---------------------- ALL SCREENS ----------------------------------------
			

ACTION_PGM_KEY:
					; play, midi and util screens

	MOVFF	GROUP,PROGRAM		; set new program
	RLNCF	PROGRAM,F,A
	RLNCF	PROGRAM,F,A
	RLNCF	PROGRAM,F,A

	MOVF	KEY_ID,W,A
	ANDLW	007H			; mask 3 lsb
	IORWF	PROGRAM,F,A
		
	CALL	PUT_PROGRAM_CHANGE

					; rev 3_1
					; leave cursor at previous position

					; rev 1_5	
	;CLRF	MODE_SAV,A		; force play screen
					; mode_sav loaded to mode
					; in dsp_play_screen
					; cursor -> program
	CALL	DSP_PLAY_SCREEN
	RETURN

	
ACTION_HOTKEY:

	MOVF	HOT_KEY,W,A
	BTFSC	STATUS,Z,A
	RETURN				; 0 - do nothing

	XORLW	001H
	BTFSS	STATUS,Z,A
	GOTO	HOTKEY1

	CALL	PUT_ALL_NOTES_OFF
	RETURN

HOTKEY1:
	MOVF	HOT_KEY,W,A
	XORLW	002H
	BTFSS	STATUS,Z,A
	GOTO	HOTKEY2
	
	CALL	PUT_RESET_ALL
	RETURN

HOTKEY2:
	MOVF	HOT_KEY,W,A		; clear - all notes off
	XORLW	003H			;       + reset all controllers
	BTFSS	STATUS,Z,A
	GOTO	HOTKEY3
	CALL	PUT_ALL_NOTES_OFF
	CALL	PUT_RESET_ALL

	RETURN
HOTKEY3:
	MOVF	HOT_KEY,W,A
	XORLW	004H
	BTFSS	STATUS,Z,A
	GOTO	HOTKEY4
	
	CALL	PUT_PANIC
HOTKEY4:				; no further functions supported
	RETURN				; at present


			
;----------------------- GENERAL ROUTINES -------------------------------------

HARDWARE_TAB:				; used to set bfr_ptr in 
					; assign, invert, noise routines


	BTFSC	KEY_ID,0,A
	GOTO	TAB_INC

	DECF	BFR_PTR,F,A		; dec pointer to assign buffer
	MOVLW	005H			; rev 3.3 - no breath control
	BTFSC	BFR_PTR,7,A		; test underflow
	MOVWF	BFR_PTR,A
	RETURN

TAB_INC:

	INCF	BFR_PTR,F,A		; inc pointer to assign buffer
	MOVF	BFR_PTR,W,A
	XORLW	006H			; rev 3.3 - no breath control
	BTFSC	STATUS,Z,A		; test fsr0=7
	CLRF	BFR_PTR,A
	RETURN


PUT_CSUM:

	MOVLW	00FH
	MOVWF	WK2,A
	LFSR	FSR0,CHANNEL
	MOVLW	000H

PUT_CSUM1:

	ADDWF	POSTINC0,W,A
	DECFSZ	WK2,F,A
	GOTO	PUT_CSUM1
	MOVWF	INDF0,A			; place csum
	COMF	INDF0,F,A		; 1s complement
	RETURN




			; ***********
			; Calibration
			; ***********

CALIBRATE:

	LFSR	FSR0,OFFSET_BFR
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	MOVF	MIN,W,A			; min = offset
	MOVWF	INDF0,A
					; w reg still holds min
	SUBWF	MAX,W,A			; w = max - min
	MOVWF	WK2,A			; save atod range

	CLRF	WK0,A			; wk1 msb of 2 byte count
	CLRF	WK1,A

	CLRF	MSB,A			; clear result registers
	CLRF	LSB,A
CAL1:
	MOVF	WK1,W,A			; get high count
	ADDWF	WK2,W,A			; add scale - keep in w reg
	BTFSC	STATUS,C,A		; test overflow
	GOTO	CAL2
	
	MOVWF	WK1,A
	INCF	MSB,F,A
	GOTO	CAL1			; loop to increment upper byte	

CAL2:					; lower byte loop
		
	MOVF	WK0,W,A
	ADDWF	WK2,W,A
	MOVWF	WK0,A

	MOVLW	000H
	ADDWFC	WK1,F,A
	BTFSC	STATUS,C,A
	GOTO	CAL3

	INCF	LSB,F,A
	GOTO	CAL2

CAL3:					; save to scale buffer

	LFSR	FSR0,SCALE_BFR
	RLNCF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	MOVFF	MSB,POSTINC0
	MOVFF	LSB,INDF0

	MOVF	BFR_PTR,W,A		; test pitchbend wheel
	BTFSS	STATUS,Z,A
	RETURN

					; assume returned to centre position

	CLRF	WK0,A			; get atod mid point
	CLRF	WK1,A
	MOVF	MIN,W,A
	CALL	CAL4			; add to wk regs
	MOVF	MAX,W,A
	CALL	CAL4			; add to wk regs

	BCF	STATUS,C,A
	RRCF	WK1,F,A			; wk0 = atod mid point
	RRCF	WK0,F,A
		
	LFSR	FSR0,ATOD_BFR		; nb pitchbend wheel bfr_ptr=0
	MOVF	INDF0,W,A
	SUBWF	WK0,W,A			; w = mean - atod @ centre
					
	LFSR	FSR0,OFFSET_BFR
	SUBWF	INDF0,F,A
	RETURN



CAL4:	ADDWF	WK0,F,A
	MOVLW	000H
	ADDWFC	WK1,F,A
	RETURN



			; **************
			; Analogue Input
			; **************

ANALOGUE_INPUT:

	MOVF	WK0,W,A			; test unused input
	XORLW	012H
	BTFSC	STATUS,Z,A
	RETURN

	DSTR	ATODMAP			; convert id -> bfr_ptr
	MOVF	WK0,W,A
	ANDLW	007H
	ADDWF	TBLPTRL,F,A
	MOVLW	000H
	ADDWFC	TBLPTRH,F,A
	TBLRD*				; tablat -> analoge channel 00h - 06h

	APTR	ATOD_BFR		; fsr0 -> atod buffer with tablat offset
	MOVFF	WK1,INDF0		; place data in buffer

	BTFSC	CAL_FLG,A
	GOTO	TEST_MAX_MIN		; simply display max min for 
					; channel address by bfr_ptr
					; in calibration mode
	APTR	OFFSET_BFR		; remove offset
	MOVF	INDF0,W,A		; offset to w reg
	SUBWF	WK1,F,A			; wk1 = atod - offset
	BTFSS	STATUS,C,A		; test negative result
	CLRF	WK1,A			; set zero if underflow

	LFSR	FSR0,SCALE_BFR		; get scale factor
	RLNCF	TABLAT,W,A		; scale buffer 2 bytes per channel
	IORWF	FSR0L,F,A		; fsr0 -> high scale
	
	MOVF	POSTINC0,W,A
	MULWF	WK1,A			; prodh should be zero
	MOVF	PRODH,W,A
	BNZ	AIN0			; overflow - force ffh as result
	MOVFF	PRODL,MSB

	MOVF	INDF0,W,A		; lsb of scale
	MULWF	WK1,A

	MOVFF	PRODL,LSB
	MOVF	PRODH,W,A
	ADDWF	MSB,F,A			; scaled input in msb, lsb
	BTFSC	STATUS,C,A
	GOTO	AIN0			; overflow

	BTFSS	LSB,7,A			; test round up required
	GOTO	AIN1

	MOVLW	001H
	ADDWF	MSB,F,A
	BTFSS	STATUS,C,A
	GOTO	AIN1

AIN0:					; overflow
	SETF	MSB,A
AIN1:					; msb holds 8 bit scaled input
	APTR	INVERT_BFR		; test need to invert
	MOVF	INDF0,W,A
	BZ	AIN2			; no inversion required
	
	COMF	MSB,F,A			; Complement msb

AIN2:					; apply noise margin

	APTR	NOISE_BFR
	MOVFF	INDF0,WK2		; wk2 00h - 07h (6 - 13 bits)
	MOVF	INDF0,W,A
	BZ	AIN8			; 6 bits - leave as 8 bit value

	XORLW	007H			; test 13 bits
	BTFSC	STATUS,Z,A
	GOTO	AIN5			; switch 00h or ffh only

	MOVLW	0FEH			; mask for 7 bits
	MOVWF	WK0,A
	MOVLW	001H			; round up bit for 7 bits
	MOVWF	WK1,A

AIN3:
	DCFSNZ	WK2,F,A
	GOTO	AIN4			; get apply noise filter
	BCF	STATUS,C,A		; clear carry
	RLCF	WK0,F,A			; rotate mask
	RLNCF	WK1,F,A			; rotate round up bit
	GOTO	AIN3

AIN4:	

	MOVF	MSB,W,A
	ANDWF	WK0,F,A			; apply mask - result in wk0
	ANDWF	WK1,W,A			; test round up
	BTFSC	STATUS,Z,A
	GOTO	AIN4A
					; round up required
	RLNCF	WK1,W,A			; set amount to add
					; 2 for 7 bits, 4 for 8 bits etc.
	ADDWF	WK0,F,A
	BTFSC	STATUS,C,A
	SETF	WK0,A			; set 0ffh if overflow
					; fall thro' to reload msb
AIN4A:					; no round up
	MOVFF	WK0,MSB
	GOTO	AIN8

AIN5:					; 13 bits - test only bit 7

	MOVLW	000H			; only 00h or ffh
	BTFSC	MSB,7,A
	MOVLW	0FFH
	MOVWF	MSB,A
	 
AIN8:					; here after noise margin applied
					; data in msb
	APTR	OUTPUT_BFR
	MOVF	INDF0,W,A		; test same as previous data
	XORWF	MSB,W,A
	BTFSC	STATUS,Z,A
	RETURN				; ignore if same as previous
	MOVFF	MSB,INDF0		; save data to buffer

	CALL	PUT_CONTROLLER		; put data to midi out buffer
	RETURN


TEST_MAX_MIN:				; BFR_PTR -> calibration channel

	LFSR	FSR0,ATOD_BFR
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A		; fsr0 -> atod buffer for cal channel

	MOVF	INDF0,W,A		; get atod
	MOVWF	WK1,A			; wk1 still holds 8 bit value
	
	BTFSS	FIRST_MOVE_FLG,A	; load max and min with atod if first
	CPFSGT	MAX,A			; time control moved
	MOVWF	MAX,A			; update max if atod <= max

	BTFSS	FIRST_MOVE_FLG,A	; 
	CPFSLT	MIN,A
	MOVWF	MIN,A
	CALL	DSP_MAX_MIN
	BCF	FIRST_MOVE_FLG,A
	RETURN


			; *******************
			; MIDI output strings
			; *******************
			; use fsr1 as fsr0 point to analogue buffers
PUT_PROGRAM_CHANGE:

	LFSR	FSR1,MIDI_STRING	; point to midi output string
	MOVF	CHANNEL,W,A		; get channel
	IORLW	0C0H			; programme change
	CALL	PUT_MIDI_STATUS

	MOVFF	PROGRAM,POSTINC1	; new programme

PUT_LENGTH:

	MOVF	FSR1L,W,A
	ANDLW	00FH
	MOVWF	WK2,A
	CALL	PUT_STRING
	RETURN

PUT_BANK_SEL:

	LFSR	FSR1,MIDI_STRING	; point to midi string

	MOVF	CHANNEL,W,A		; get channel
	IORLW	0B0H			; programme change
	CALL	PUT_MIDI_STATUS
	CLRF	POSTINC1,A		; 00h - msb
	CLRF	POSTINC1,A		; 00h
	
	MOVF	CHANNEL,W,A		; get channel
	IORLW	0B0H			; programme change
	CALL	PUT_MIDI_STATUS
	MOVLW	020H			; lsb
	MOVWF	POSTINC1,A
	MOVFF	MIDI_BANK,POSTINC1	; bank id

	GOTO	PUT_LENGTH

PUT_CONTROLLER:				; tablat -> control channel
					; msb holds data
	LFSR	FSR1,MIDI_STRING

	APTR	ASSIGN_BFR		; get controller data
	BTFSC	INDF0,7,A		; test pitchbend, aftertouch
	GOTO	PUT_CON1

					; all other controllers
	MOVF	CHANNEL,W,A
	IORLW	0B0H
	CALL	PUT_MIDI_STATUS

	MOVFF	INDF0,POSTINC1		; controller id

	BCF	STATUS,C,A
	RRCF	MSB,W,A			; only 7 msb transmitted
	MOVWF	POSTINC1,A
	GOTO	PUT_LENGTH


PUT_CON1:

	BTFSC	INDF0,0,A
	GOTO	PUT_PBEND

PUT_AFTER:				; aftertouch

	LFSR	FSR1,MIDI_STRING

	MOVF	CHANNEL,W,A
	IORLW	0D0H
	CALL	PUT_MIDI_STATUS

	BCF	STATUS,C,A
	RRCF	MSB,W,A			; only 7 msb transmitted
	MOVWF	POSTINC1,A
	GOTO	PUT_LENGTH
	
PUT_PBEND:				; pitchbend

	LFSR	FSR1,MIDI_STRING

	MOVF	CHANNEL,W,A
	IORLW	0E0H
	CALL	PUT_MIDI_STATUS

	MOVF	MSB,W,A			; test ffh
	XORLW	0FFH
	BTFSS	STATUS,Z,A
	GOTO	PUT_PBEND1

	MOVLW	07FH			; ffh transmitted as 7fh, 7fh
	MOVWF	POSTINC1,A
	GOTO	PUT_PBEND2

PUT_PBEND1:

	MOVLW	000H			; set lsb
	BTFSC	MSB,0,A
	MOVLW	040H
	MOVWF	POSTINC1,A

PUT_PBEND2:

	BCF	STATUS,C,A
	RRCF	MSB,W,A			; msb - 7 bits only		
	MOVWF	POSTINC1,A
	GOTO	PUT_LENGTH


PUT_ALL_NOTES_OFF:			; Bcccc 7bh 00h to all channels


	CLRF	TEMP_CHAN,A		; temp channel
	MOVLW	07BH
	MOVWF	TEMP_STAT,A

PUT_ALL_CHANNELS:

	LFSR	FSR1,MIDI_STRING

	MOVF	TEMP_CHAN,W,A
	IORLW	0B0H
	CALL	PUT_MIDI_STATUS

	MOVFF	TEMP_STAT,POSTINC1
	CLRF	POSTINC1,A		; OFF
	CALL	PUT_LENGTH

	INCF	TEMP_CHAN,F,A
	BTFSS	TEMP_CHAN,4,A
	GOTO	PUT_ALL_CHANNELS
	RETURN

PUT_RESET_ALL:

	CLRF	TEMP_CHAN,A
	MOVLW	079H
	MOVWF	TEMP_STAT,A
	GOTO	PUT_ALL_CHANNELS

PUT_PANIC:

	BSF	MIDI_INH_FLG,A		; stop any further output from midi buffer

PP1:
	BTFSC	MDATA_FLG,A		; test character being transmitted
	GOTO	PP1

	CLRF	MBFRIN,A		; reset buffer
	CLRF	MBFROUT,A
	CLRF	MBFRSIZ,A
	
	BCF	MIDI_INH_FLG,A

	MOVLW	0FCH			; stop current sequence
	MOVWF	MRTIME,A		; real time data
	BSF	RTIME_FLG,A

	CLRF	TEMP_CHAN,A
	MOVLW	07BH			; all notes off
	MOVWF	TEMP_STAT,A
	CALL	PUT_ALL_CHANNELS
	CALL	MIDI_DELAY

	CLRF	TEMP_CHAN,A
	MOVLW	079H			; reset all controllers
	MOVWF	TEMP_STAT,A
	CALL	PUT_ALL_CHANNELS
	CALL	MIDI_DELAY

	CLRF	TEMP_CHAN,A
	MOVLW	078H			; all sound off
	MOVWF	TEMP_STAT,A
	CALL	PUT_ALL_CHANNELS
	CALL	MIDI_DELAY

	CLRF	TEMP_CHAN,A
PP2:
	LFSR	FSR1,MIDI_STRING	; point to midi string

	MOVF	TEMP_CHAN,W,A
	IORLW	0E0H			; centre pitch bend
	CALL	PUT_MIDI_STATUS		; Ecccc 00 40
	CLRF	POSTINC1,A
	MOVLW	040H
	MOVWF	POSTINC1,A

	MOVF	TEMP_CHAN,W,A
	IORLW	0B0H			; mod wheel to zero
	CALL	PUT_MIDI_STATUS		; MSB
	MOVLW	001H			; 
	MOVWF	POSTINC1,A		; Bcccc 01 00 [Bcccc] 21 00
	CLRF	POSTINC1,A

	MOVF	TEMP_CHAN,W,A
	IORLW	0B0H
	CALL	PUT_MIDI_STATUS		; LSB
	MOVLW	021H
	MOVWF	POSTINC1,A
	CLRF	POSTINC1,A

	MOVF	TEMP_CHAN,W,A
	IORLW	0B0H			; sustain off
	CALL	PUT_MIDI_STATUS		; 
	MOVLW	040H			; 
	MOVWF	POSTINC1,A		; Bcccc 40 00 
	CLRF	POSTINC1,A

	MOVF	TEMP_CHAN,W,A
	IORLW	0B0H			; volume on
	CALL	PUT_MIDI_STATUS		; 
	MOVLW	007H			; 
	MOVWF	POSTINC1,A		; Bcccc 07 7F 
	MOVLW	07FH
	MOVWF	POSTINC1,A
	
	CALL	PUT_LENGTH
	CALL	MIDI_DELAY

	LFSR	FSR1,MIDI_STRING	; ensure no overflow of midi_string

	MOVF	TEMP_CHAN,W,A
	IORLW	0D0H			; aftertouch off
	CALL	PUT_MIDI_STATUS		; Dcccc 00
	CLRF	POSTINC1,A
	CALL	PUT_LENGTH

	INCF	TEMP_CHAN,F,A
	BTFSS	TEMP_CHAN,4,A
	GOTO	PP2

	CLRF	TEMP_CHAN,A		; turn all notes off individually
PP3:
	CLRF	WK0,A			; temp wk0 = note

PP4:
	LFSR	FSR1,MIDI_STRING	; start of midi string

PP5:

	MOVF	TEMP_CHAN,W,A
	IORLW	090H			; 9ccc note(00 - 7f) 00
	CALL	PUT_MIDI_STATUS
	MOVFF	WK0,POSTINC1		
	INCF	WK0,F,A
	CLRF	POSTINC1,A

	MOVF	WK0,W,A			; test 4 notes to midi_string
	ANDLW	003H
	BTFSS	STATUS,Z,A
	GOTO	PP5

	CALL	PUT_LENGTH
	MOVF	WK0,W,A
	ANDLW	00FH
	BTFSS	STATUS,Z,A		; test 16 notes (48 bytes max)
	GOTO	PP4
	CALL	MIDI_DELAY

	BTFSS	WK0,7,A			; test last note sent
	GOTO	PP4

	INCF	TEMP_CHAN,F,A
	BTFSS	TEMP_CHAN,4,A
	GOTO	PP3

	RETURN


PUT_MIDI_STATUS:			; put status byte to midi_string

	MOVWF	SAV_STATUS,A		; temp save status byte
	BTFSC	RUN_STAT_FLG,A
	GOTO	PUT_RUNSTAT
	
	MOVWF	POSTINC1,A		; no running status
	RETURN

PUT_RUNSTAT:

	XORWF	MIDI_STATUS,W,A		; test same status byte
	BTFSC	STATUS,Z,A
	RETURN				; no need to place status
	MOVFF	SAV_STATUS,POSTINC1	; put data to buffer
	MOVFF	SAV_STATUS,MIDI_STATUS	; save new status byte
	RETURN



			; data in midi_string
			; length of data in wk2
PUT_STRING:	

	BSF	MIDI_INH_FLG,A		; inhibit any new midi out while loading

	LFSR	FSR0,MBFR		; point to buffer entry point
	MOVF	MBFRIN,W,A
	ADDWF	FSR0L,F,A		; fsr0 -> data entry point

	LFSR	FSR1,MIDI_STRING

PUT_STRING1:

	MOVFF	POSTINC1,INDF0		; copy data - no inc midi pointer

	INCF	MBFRIN,F,A
	MOVFF	MBFRIN,FSR0L		; ensure stay in page
					; fsr0h not incremented
	INCF	MBFRSIZ,F,A
	DECFSZ	WK2,F,A
	GOTO	PUT_STRING1

	BCF	MIDI_INH_FLG,A		; re-enable output
	RETURN




			; *****************
			; Timer 0 Interrupt
			; *****************

			; every 32 uSec
			; source - internal clock/2
			; FSR2 used exclusively in timer interrupt

TIMER0_INT:

	BCF	INTCON,TMR0IF,A		; reset interrupt flag

	MOVLW	065H		
	MOVWF	TMR0L,A

	BSF	TMR0_FLG,A		; say timer 0 interrupt occurred

					; handle midi out
	BTFSC	MDATA_FLG,A
	GOTO	TIMER0_14

TIMER0_10:				; test new data ready to xmit

	BTFSC	MIDI_INH_FLG,A		; test data being loaded to buffer
	GOTO	TIMER0_16
	
	BTFSS	RTIME_FLG,A		; test real time data to load
	GOTO	TIMER0_12


	BCF	RTIME_FLG,A		; clear real time flag
	MOVFF	MRTIME,MDATA		; load data

TIMER0_11:

	BSF	MIDI_OUT,A		; start bit *
	BSF	MDATA_FLG,A		; say in send data
	CLRF	ASCNT,A			; clear active sensing count
	BCF	LED_YEL,A		; yellow led on
	
.IF (UNIT & RS232)
	MOVFF	MDATA,TXREG
.ENDIF

	GOTO	TIMER0_16

TIMER0_12:

	MOVF	MBFRSIZ,W,A
	BZ	TIMER0_16		; exit if no data


	BSF	MIDI_OUT,A		; start bit *
	BSF	MDATA_FLG,A		; say in send data
	CLRF	ASCNT,A			; clear active sensing count
	BCF	LED_YEL,A

	LFSR	FSR2,0100H		; point to midi output buffer
	MOVFF	MBFROUT,FSR2L		; transfer low pointer
	MOVFF	INDF2,MDATA		; data @ mbfout to mdata

	INCF	MBFROUT,F,A		; inc output pointer
	DECF	MBFRSIZ,F,A		; dec size count

.IF (UNIT & RS232)
	MOVFF	MDATA,TXREG
.ENDIF


	GOTO	TIMER0_16		


TIMER0_14:				; in process of sending midi byte

	BTFSC	MCNT,3,A		; test last bit 8th bit
	GOTO	TIMER0_13		; go send stop bit

	INCF	MCNT,F,A		; inc count

	RRCF	MDATA,F,A		; rr data thro carry
	BC	TIMER0_15		; go set midi_out high
	NOP
	BSF	MIDI_OUT,A		; midi_out high *
	GOTO	TIMER0_16


TIMER0_13:				; send stop bit
	
	BCF	MDATA_FLG,A		; clear data flag and counter
	CLRF	MCNT,A			; fall thro to send stop bit
	BSF	LED_YEL,A			; yellow led off
TIMER0_15:				; set midi_out high
	BCF	MIDI_OUT,A		; *
	GOTO	TIMER0_16

TIMER0_16:				; generate 4msec based timer

	DECFSZ	MSEC4,F,A
	RETURN
					; 4 msec interval
	MOVLW	07DH			; reload with 125
	MOVWF	MSEC4,A

	INCF	ASCNT,F,A		; inc active sensing count
	BTFSC	ASCNT,6,A
	CALL	ACTIVE_SENSE

	MOVF	COUNT,W,A		; test count register for timeout
	BTFSC	STATUS,Z,A
	RETURN

	DCFSNZ	COUNT,F,A
	BSF	TIMEOUT_FLG,A
	RETURN	


ACTIVE_SENSE:				; ascnt reached 256msec

	CLRF	ASCNT,A
	CLRF	MIDI_STATUS,A		; cancel running status status byte
	BTFSS	ACTIVE_FLG,A
	RETURN

	MOVLW	0FEH			; active sense command
	MOVWF	MRTIME,A
	BSF	RTIME_FLG,A
	RETURN				; send data next timer0 interrupt



			; *****************
			; Timer 1 interrupt
			; *****************


TIMER1_INT:		; 50 msec interval

	BCF	PIR1,TMR1IF,A		; clear int flag

	MOVLW	00BH			; reload timer
	MOVWF	TMR1H,A
	MOVLW	0DCH
	MOVWF	TMR1L,A
	BSF	TMR1_FLG,A

	MOVF	RPT_TIME,W,A
	BTFSS	STATUS,Z,A
	DECF	RPT_TIME,F,A

	INCF	SUBSEC,F,A
	MOVLW	014H
	CPFSEQ	SUBSEC,A
	RETURN

					; 1 second elapsed
	CLRF	SUBSEC,A		; reset subsec
	BTFSC	LCD_TIME,2,A		; test lcd backlight off
	RETURN

	MOVF	LCD_TIME,W,A
	BTFSC	STATUS,Z,A		; test backlight always on
	RETURN

	DECF	LCD_TIMEOUT,F,A
	MOVF	LCD_TIMEOUT,W,A
	BTFSS	STATUS,Z,A
	RETURN				; not yet timed out
	BCF	BACKLIGHT,A
	RETURN


			; **************
			; Delay Routines
			; **************

MIDI_DELAY:				; 20 msec to allow about 50 characters
					; to clear buffer
	MOVLW	005H
	MOVWF	COUNT,A
	CLRWDT
	GOTO	DELAY



DELAY_2SEC:

	CALL	DELAY_1SEC		; call and then fall thro'

DELAY_1SEC:

	MOVLW	0FAH
					; fall thro'
	
DELAY:
	MOVWF	COUNT,A			; enter w regs holds delay * 4 msec
					; 4 msec interval
	MOVLW	07DH			; reload with 125
	MOVWF	MSEC4,A

	BCF	TIMEOUT_FLG,A
	BCF	TMR0_FLG,A

DELAY1:
	BTFSS	TMR0_FLG,A
	GOTO	DELAY2
	BCF	TMR0_FLG,A	
	CLRWDT				; reset watchdog at 32 usec


DELAY2:
	BTFSS	TIMEOUT_FLG,A
	GOTO	DELAY1
	RETURN





			; ************
			; i2c Routines
			; ************
	

WRITE_I2C:		; write wk0 @ i2cadd


	CALL	TEST_I2C_IDLE		; test bus idle
	BTFSS	STATUS,Z,A
	GOTO	WRITE_I2C		; loop until idle
					; wdog will reset all if error !!

	BSF	I2CERR_FLG,A		; preset error
	BSF	SSPCON2,SEN,A		; start bit

WRITE_I2C_START:

	BTFSC	SSPCON2,SEN,A		; wait on start bit to be cleared by hardware
	GOTO	WRITE_I2C_START	

	MOVF	I2CADD,W,A		; write address
	CALL	PUT_I2C
	BTFSC	SSPCON2,ACKSTAT,A	; test ack negative
	GOTO	WRITE_I2C_STOP

	MOVLW	002H			; internal address %% TEST EEPROM
	CALL	PUT_I2C
	BTFSC	SSPCON2,ACKSTAT,A
	GOTO	WRITE_I2C_STOP

	MOVF	WK0,W,A			; data
	CALL	PUT_I2C
	BTFSC	SSPCON2,ACKSTAT,A
	GOTO	WRITE_I2C_STOP

	BCF	I2CERR_FLG,A		; say transfer ok

WRITE_I2C_STOP:

	BSF	SSPCON2,PEN,A

WRITE_I2C_STOP1:

	BTFSC	SSPCON2,PEN,A
	GOTO	WRITE_I2C_STOP1
	RETURN


PUT_I2C:				; put w reg to i2c bus

	MOVWF	SSPBUF,A
PUT1:
	BTFSC	SSPSTAT,R_W,A		; wait until byte sent
	GOTO	PUT1			; R/W cleared after ack bit rec'd
	RETURN
	


TEST_I2C_IDLE:				; return zero if idle

	BTFSC	SSPCON1,WCOL,A		; test write collision
	BCF	SSPCON1,WCOL,A		; clear if so

	BTFSC	SSPSTAT,R_W,A
	RETLW	0FFH			; say busy if r/w set

	MOVF	SSPCON2,W,A
	ANDLW	01FH			; mask acken, rcen,pen,rsen,sen
	RETURN



READ_I2C:				; read wk0,wk1 @ i2cadd
					; read csum to wk2

	CALL	TEST_I2C_IDLE		; test bus idle
	BTFSS	STATUS,Z,A
	GOTO	READ_I2C		; loop until idle
					; wdog will reset all if error !!

	BSF	I2CERR_FLG,A		; preset error bit
	BSF	SSPCON2,SEN,A		; start bit

READ_I2C_START:

	BTFSC	SSPCON2,SEN,A		; wait on start bit to be cleared by hardware
	GOTO	READ_I2C_START	

	INCF	I2CADD,W,A		; read address = write address+1
	CALL	PUT_I2C
	BTFSC	SSPCON2,ACKSTAT,A	; test ack negative
	GOTO	READ_I2C_STOP
					; assume bus idle after ack from slave

	CALL	GET_I2C			; get first byte
	BCF	SSPCON2,ACKDT,A		; ack first data byte
	CALL	GET_ACK

	MOVF	SSPBUF,W,A
	MOVWF	WK0,A

					; get 2nd byte
	CALL	GET_I2C			; 
	BCF	SSPCON2,ACKDT,A		; ack 2nd data byte
	CALL	GET_ACK

	MOVF	SSPBUF,W,A
	MOVWF	WK1,A

					; get 3rd byte (csum)
	CALL	GET_I2C			; 
	BSF	SSPCON2,ACKDT,A		; no ack for final byte
	CALL	GET_ACK

	MOVF	SSPBUF,W,A
	MOVWF	WK2,A

	BCF	I2CERR_FLG,A		; say transfer ok

READ_I2C_STOP:

	BSF	SSPCON2,PEN,A

READ_I2C_STOP1:

	BTFSC	SSPCON2,PEN,A
	GOTO	READ_I2C_STOP1
	RETURN



GET_I2C:				; get data byte from i2c bus

	BSF	SSPCON2,RCEN,A		; initiate data transfer

GET_I2C_DATA:

	BTFSS	SSPSTAT,BF,A		; test buffer full
	GOTO	GET_I2C_DATA
	RETURN


GET_ACK:
	BSF	SSPCON2,ACKEN,A
GET_ACK1:

	BTFSC	SSPCON2,ACKEN,A
	GOTO	GET_ACK1
	RETURN


			; *******************
			; Arithmetic Routines
			; *******************


TO_BCD:					; convert data @ fsr0 to bcd
					; 3 hex nibbles only
	CLRF	WK0,A			; fsr0 -> high byte (msb first)
	CLRF	WK1,A

	MOVF	POSTINC0,W,A
	ANDLW	00FH
	MOVWF	WK2,A

	BTFSC	STATUS,Z,A
	GOTO	TO_BCD_SINGLE

TBCD1:

	MOVLW	056H
	ADDWF	WK0,W,A			; keep result in w
	DAW				; decimal adjust
	MOVWF	WK0,A

	MOVLW	002H
	ADDWFC	WK1,W,A
	DAW
	MOVWF	WK1,A

	DECFSZ	WK2,F,A
	GOTO	TBCD1
	GOTO	TBCD4			; bypass clr wk0,wk1


TO_BCD_SINGLE:				; single byte conversion

	CLRF	WK0,A
	CLRF	WK1,A
TBCD4:

	SWAPF	INDF0,W,A
	ANDLW	00FH
	MOVWF	WK2,A

	BTFSC	STATUS,Z,A
	GOTO	TBCD3

TBCD2:

	MOVLW	016H
	ADDWF	WK0,W,A
	DAW
	MOVWF	WK0,A

	MOVLW	000H
	ADDWFC	WK1,W,A
	DAW
	MOVWF	WK1,A
	
	DECFSZ	WK2,F,A
	GOTO	TBCD2

TBCD3:

	MOVF	INDF0,W,A
	ANDLW	00FH
	ADDLW	000H			; DA single nibble
	DAW
	
	CALL	TBCD5
	BTFSS	ONE_FLG,A		; test need to add 1 prior to display
	RETURN

	BCF	ONE_FLG,A
	MOVLW	001H			; fall thro to increment

TBCD5:					; add w to wk0,wk1 and decimal adjust

	ADDWF	WK0,W,A
	DAW
	MOVWF	WK0,A

	MOVLW	000H
	ADDWFC	WK1,W,A
	DAW
	MOVWF	WK1,A
	RETURN


			; ********************
			; LCD Display Routines
			; ********************

			; 4 bit interface usinh 4 msb of port d
			; port e used for control lines

; Low level routines
; ==================


DSP_INIT:

	CLRF	LATE,A			; ensure e, rw, rs all low

	MOVLW	005H			; 20 msec delay
	CALL	DELAY

	MOVLW	00FH
	ANDWF	LATD,F,A		; leave midi and leds alone
	MOVLW	030H
	IORWF	LATD,F,A
	CALL	DSP_STROBE

	MOVLW	002H			; 8 msec delay
	CALL	DELAY

	CALL	DSP_STROBE		; 

	MOVLW	001H			; 4 msec delay
	CALL	DELAY

	CALL	DSP_STROBE


	MOVLW	00FH
	ANDWF	LATD,F,A		; leave midi and leds alone
	MOVLW	020H			; set 4 bit interface
	IORWF	LATD,F,A
	CALL	DSP_STROBE
	CALL	DSP_BUSY

					; now talk to display on 4 bit bus

	MOVLW	028H			; 2 lines
	CALL	DSP_WRITE_CTRL
	CALL	DSP_BUSY

	MOVLW	00CH			; display on, no cursor, no blink
	CALL	DSP_WRITE_CTRL
	CALL	DSP_BUSY

	MOVLW	006H			; increment, no shift
	CALL	DSP_WRITE_CTRL
	CALL	DSP_BUSY

DSP_CLEAR:

	MOVLW	001H			; clear display
	CALL	DSP_WRITE_CTRL
	CALL	DSP_BUSY

	MOVLW	002H
	CALL	DELAY	

DSP_HOME:				; cursor addressing

	MOVLW	000H

DSP_CSR_ADD:	

	MOVWF	CSRPOS,A		; save cursor position
	IORLW	080H			; set msb for cursor address
	CALL	DSP_WRITE_CTRL
	CALL	DSP_BUSY
	RETURN


DSP_WRITE_CTRL:				; write control data to display

	MOVWF	DDATA,A			; save data
	CLRF	LATE,A			; rs r/w both 0

DSP_WRITE_BYTE:				; write data or control to display

	CALL	DSP_WRITE_NIBBLE	; ms nibble in w to display
	SWAPF	DDATA,F,A		; ls nibble
	CALL	DSP_WRITE_NIBBLE
	GOTO	DSP_BUSY

DSP_WRITE_NIBBLE:

	MOVLW	00FH
	;ANDWF	LATD,F,A		; leave midi and leds alone
	ANDWF	PORTD,F,A

	MOVF	DDATA,W,A
	ANDLW	0F0H			; mask top nibble
	IORWF	LATD,F,A
	CALL	DSP_STROBE
	RETURN

DSP_WRITE_DATA:				; display data in wreg

	MOVWF	DDATA,A			; save data
	CLRF	LATE,A
	BSF	RS,A			; set rs if data
	INCF	CSRPOS,F,A		; keep track of cursor position
	GOTO	DSP_WRITE_BYTE


DSP_BUSY:				; time delay

	MOVLW	0FAH
	MOVWF	DDATA,A

DSP_BUSY1:

	NOP
	NOP
	DECFSZ	DDATA,F,A
	GOTO	DSP_BUSY1

	RETURN


DSP_STROBE:				; pulse e high

	BSF	E,A
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	BCF	E,A
	RETURN


CSR_ON:	
	MOVLW	00DH			; display and blink
	CALL	DSP_WRITE_CTRL
	GOTO	DSP_BUSY

CSR_OFF:

	MOVLW	00CH
	CALL	DSP_WRITE_CTRL
	GOTO	DSP_BUSY

RESTORE_CSR:

	MOVF	CSRSAV,W,A
	CALL	DSP_CSR_ADD
	RETURN


; Mid level display routines
; ==========================

DSP_BCD:				; BCD count held on wk0,wk1
					; wk2 reg holds mask digits to display
					; eg 00000111 to display to 3 digits
	SWAPF	WK2,F,A			; move mask to top nibble

	SWAPF	WK1,W,A
	CALL	DSP_BCD1

	MOVF	WK1,W,A
	CALL	DSP_BCD1

	SWAPF	WK0,W,A
	CALL	DSP_BCD1

	MOVF	WK0,W,A
					; fall thro
DSP_BCD1:

	ANDLW	00FH
	IORLW	030H
	RLCF	WK2,F,A
	BTFSC	STATUS,C,A
	CALL	DSP_WRITE_DATA
	RETURN



DSP_STRING:


	TBLRD*+	
	MOVF	TABLAT,W,A
	BZ	DSRET
	
	CALL	DSP_WRITE_DATA

	MOVF	CSRPOS,W,A		; test end of top line
	XORLW	018H
	BTFSS	STATUS,Z,A
	GOTO	DSP_STRING

	MOVLW	040H
	CALL	DSP_CSR_ADD
	GOTO	DSP_STRING


DSRET:
	RETURN

			; ***************
			; Display screens
			; ***************

DSP_PLAY_SCREEN:
;==============
	MOVFF	MODE_SAV,MODE		; retore play mode
	CALL	DSP_CLEAR		; clear
	DSTR	M_PLAY			; background
	CALL	DSP_STRING
	CALL	DSP_PROGRAM		; update values
	CALL	DSP_BANK
	CALL	DSP_GROUP
	CALL	CSR_ON
	CALL	SET_PLAY_CSR
	RETURN

DSP_MIDI_SCREEN:
;==============

	MOVLW	040H			; set midi mode
	MOVWF	MODE,A
	CALL	DSP_CLEAR		; clear
	DSTR	M_MIDI			; background
	CALL	DSP_STRING

DSP_MIDI_SUBSCREEN:
;=================
	MOVLW	040H			; midi transpose
	CPFSEQ	MODE,A
	GOTO	DMS1

	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_TRANSPOSE		; transpose message
	CALL	DSP_STRING
	MOVLW	049H
	MOVWF	CSRSAV,A
	CALL	CSR_OFF			; cursor off in transpose
	CALL	DSP_TRANSPOSE		; display current value
	CALL	RESTORE_CSR
	RETURN

DMS1:

	MOVLW	041H			; midi channel
	CPFSEQ	MODE,A
	GOTO	DMS2

	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_CHANNEL		; channel message
	CALL	DSP_STRING
	MOVLW	050h
	MOVWF	CSRSAV,A
	CALL	CSR_OFF			; cursor off in channel
	CALL	DSP_CHANNEL		; display current value
	CALL	RESTORE_CSR
	RETURN

DMS2:

	MOVLW	042H			; assign control
	CPFSEQ	MODE,A
	GOTO	DMS3
		
	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_ASSIGN
	CALL	DSP_STRING
	CALL	CSR_OFF
	CLRF	BFR_PTR,A

	CALL	DSP_HARDWARE
	CALL	DSP_CONTROLLER
	RETURN

DMS3:

	MOVLW	043H			; invert control
	CPFSEQ	MODE,A
	GOTO	DMS4
		
	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_INVERT
	CALL	DSP_STRING
	CALL	CSR_OFF
	CLRF	BFR_PTR,A

	CALL	DSP_HARDWARE
	CALL	DSP_INVERT
	RETURN

DMS4:

	MOVLW	044H			; velocity table
	CPFSEQ	MODE,A
	GOTO	DMS5
		
	MOVLW	00AH			; overwrite hardware display
	CALL	DSP_CSR_ADD
	DSTR	M_CONTROLLER		; use blank string in contoller table
	CALL	DSP_STRING

	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_VELOCITY
	CALL	DSP_STRING
	CALL	CSR_OFF
	
	CALL	DSP_VELOCITY
	RETURN
	
DMS5:

	MOVLW	045H			; running status
	CPFSEQ	MODE,A
	GOTO	DMS6

	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_RUNNING_STAT
	CALL	DSP_STRING

	CLRF	WK0,A			; wk0 used to hold status
	BTFSC	RUN_STAT_FLG,A
	INCF	WK0,F,A
	CALL	DSP_ON_OFF
	RETURN

DMS6:
	MOVLW	046H			; running status
	CPFSEQ	MODE,A
	GOTO	DMS7

	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_ACTIVE_SENSE
	CALL	DSP_STRING

	CLRF	WK0,A			; wk0 used to hold status
	BTFSC	ACTIVE_FLG,A
	INCF	WK0,F,A
	CALL	DSP_ON_OFF
	RETURN
DMS7:

	RETURN


DSP_UTIL_SCREEN:
;==============
	MOVLW	080H			; start with lcd backlight
	MOVWF	MODE,A			; set utility mode
	CALL	DSP_CLEAR		; clear
	DSTR	M_UTIL			; background
	CALL	DSP_STRING
	
					; fall thro
DSP_UTIL_SUBSCREEN:
;=================
	
	
	MOVLW	080H
	CPFSEQ	MODE,A
	GOTO	DUS0

	CALL	CSR_OFF
	MOVLW	040H			; lcd backlight
	CALL	DSP_CSR_ADD
	DSTR	M_BACKLIGHT
	CALL	DSP_STRING	
	CALL	DSP_LCDTIME
	RETURN

DUS0:

	MOVLW	081H
	CPFSEQ	MODE,A
	GOTO	DUS1
					; hot key assignment
	MOVLW	040H			; address cursor
	CALL	DSP_CSR_ADD
	DSTR	M_HOT_SCRN
	CALL	DSP_STRING
	CALL	DSP_HOT_KEY
	RETURN

DUS1:
	MOVLW	082H
	CPFSEQ	MODE,A
	GOTO	DUS3

	MOVLW	040H			; noise margin
	CALL	DSP_CSR_ADD
	DSTR	M_NOISE
	CALL	DSP_STRING

	CLRF	BFR_PTR,A			; point to 1st hardware
	CALL	DSP_HARDWARE
	CALL	DSP_NOISE
	RETURN

DUS3:
	MOVLW	083H
	CPFSEQ	MODE,A
	GOTO	DUS4

	MOVLW	040H			; calibration
	CALL	DSP_CSR_ADD
	DSTR	M_CALIBRATE
	CALL	DSP_STRING
	BSF	CAL_FLG,A		; say calibration mode

	LFSR	FSR0,ATOD_BFR		; get channel 0 atod
	MOVF	INDF0,W,A
	MOVWF	MAX,A
	MOVWF	MIN,A
	CLRF	BFR_PTR,A		; point to 1st hardware
	CALL	DSP_HARDWARE
	CALL	DSP_MAX_MIN		; max min values updated in analogue input
	RETURN				

DUS4:

	MOVLW	084H			; serial no and firmware
	CPFSEQ	MODE,A
	GOTO	DUS5


	MOVLW	00AH			; allow for space at head of string
	CALL	DSP_CSR_ADD
	DSTR	M_REVISION		; firmware revision to top line
	CALL	DSP_STRING
	
	MOVLW	040H			; line 2
	CALL	DSP_CSR_ADD
	
	DSTR	M_SERNO			; serial no to line 2
	CALL	DSP_STRING

	LFSR	FSR0,SERNO		; point to 4 digit bcd number
	MOVFF	POSTINC0,WK1
	MOVFF	POSTINC0,WK0
	MOVLW	00FH			; display all 4 digits
	MOVWF	WK2,A
	CALL	DSP_BCD
	
	MOVLW	'-'			; hyphen after 4 bytes
	CALL	DSP_WRITE_DATA
	
	MOVFF	POSTINC0,WK1
	MOVFF	POSTINC0,WK0
	MOVLW	00FH			; display all 4 digits
	MOVWF	WK2,A
	CALL	DSP_BCD
	
	MOVLW	' '
	CALL	DSP_WRITE_DATA		; over-write any trailing data
	RETURN				; from calibration screen
		


DUS5:

	MOVLW	085H			; default count display		
	CPFSEQ	MODE,A
	GOTO	DUS6


	MOVLW	00BH
	CALL	DSP_CSR_ADD

	DSTR	M_PLUS
	BTFSS	DFLT_CNT,1,A		; test which message to display
	GOTO	DUS5A
	
	DSTR	M_MINUS
	
DUS5A:

	CALL	DSP_STRING		; display + / - message


	MOVLW	040H
	CALL	DSP_CSR_ADD
	DSTR	M_SYS_DFLT
	CALL	DSP_STRING

	CALL	CSR_ON
	MOVF	DFLT_CNT,W,A
	CALL	DSP_CSR_ADD
	
DUS6:

	RETURN



			; *********************
			; Display Data routines
			; *********************			


DSP_PROGRAM:

	MOVLW	014H			; dispay program
	CALL	DSP_CSR_ADD		
	BSF	ONE_FLG,A		; start from 1
	LFSR	FSR0,PROGRAM
	CALL	TO_BCD_SINGLE
	MOVLW	007H			; 3 digit display
	MOVWF	WK2,A
	CALL	DSP_BCD
	RETURN

DSP_BANK:

	MOVLW	046H			; bank
	CALL	DSP_CSR_ADD
	BSF	ONE_FLG,A		; start from 1
	LFSR	FSR0,MIDI_BANK
	CALL	TO_BCD_SINGLE
	MOVLW	007H			; 3 digits
	MOVWF	WK2,A
	CALL	DSP_BCD
	RETURN

DSP_GROUP:

	MOVLW	051H			; group
	CALL	DSP_CSR_ADD
	DSTR	M_GROUP			; set tbl pointer
	MOVF	GROUP,W,A
	MULLW	00AH
	CALL	DSP_TABLE
	RETURN


DSP_TABLE:				; enter prodh/l set, tblptr -> start of table

	MOVF	PRODL,W,A
	ADDWF	TBLPTRL,F,A
	MOVF	PRODH,W,A
	ADDWFC	TBLPTRH,F,A
	CALL	DSP_STRING
	RETURN

SET_PLAY_CSR:				; position cursor in play screen

	MOVLW	012H			; program
	BTFSC	MODE,0,A
	MOVLW	044H			; bank
	BTFSC	MODE,1,A
	MOVLW	050H			; group

	MOVWF	CSRSAV,A
	CALL	RESTORE_CSR
	RETURN
	

DSP_TRANSPOSE:

	MOVLW	04BH			; transpose
	CALL	DSP_CSR_ADD
	
	MOVFF	TRANSPOSE,WK3		; temp copy
	BCF	NEG_FLG,A		; assume positive
	BTFSS	TRANSPOSE,7,A		; test negative
	GOTO	DSP_TRANS1
	
	BSF	NEG_FLG,A
	COMF	WK3,F,A			; 2s complement if negative
	INCF	WK3,F,A		

DSP_TRANS1:
					; display polarity
	MOVF	TRANSPOSE,W,A
	BNZ	DSP_TRANS2
	MOVLW	' '			; no sign if zero
	GOTO	DSP_TRANS3

DSP_TRANS2:
	MOVLW	'+'
	BTFSC	NEG_FLG,A
	MOVLW	'-'

DSP_TRANS3:

	CALL	DSP_WRITE_DATA

	LFSR	FSR0,WK3
	CALL	TO_BCD_SINGLE
	MOVLW	003H
	MOVWF	WK2,A
	CALL	DSP_BCD
	RETURN

DSP_CHANNEL:

	MOVLW	052H
	CALL	DSP_CSR_ADD

	BSF	ONE_FLG,A		; display 1 - 16
	LFSR	FSR0,CHANNEL
	CALL	TO_BCD_SINGLE
	MOVLW	003H
	MOVWF	WK2,A
	CALL	DSP_BCD
	RETURN


DSP_HARDWARE:				; FSR0 -> harware in assign buffer

	MOVLW	00BH
	CALL	DSP_CSR_ADD

	DSTR	M_HARDWARE
	MOVF	BFR_PTR,W,A
	MULLW	00EH
	CALL	DSP_TABLE

	RETURN

DSP_CONTROLLER:				; 000H - 01FH / 040H - 05FH / 080H - 081H

	MOVLW	04BH
	CALL	DSP_CSR_ADD
	
	LFSR	FSR0,ASSIGN_BFR
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	DSTR	M_CONTROLLER
	MOVFF	INDF0,WK0		; get value from assign buffer
	BTFSC	WK0,6,A			; test 40 - 5fh
	BSF	WK0,5,A			; swap to 20 - 3fh
	BCF	WK0,6,A
	BTFSC	WK0,7,A			; test 80-81h
	BSF	WK0,6,A			; swap to 40 - 41h
	BCF	WK0,7,A

	MOVF	WK0,W,A
	MULLW	010H

	CALL	DSP_TABLE
	RETURN


DSP_INVERT:

	MOVLW	04BH
	CALL	DSP_CSR_ADD

	LFSR	FSR0,INVERT_BFR		; buffer holds 0 or 1
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	DSTR	M_NORMAL
	MOVF	INDF0,W,A
	MULLW	008H
	CALL	DSP_TABLE		
	RETURN
	
DSP_VELOCITY:

	MOVLW	050H			; velocity table selection
	CALL	DSP_CSR_ADD
	BSF	ONE_FLG,A		; start from 1
	LFSR	FSR0,VELTBL
	CALL	TO_BCD_SINGLE
	MOVLW	001H			; 1 digit
	MOVWF	WK2,A
	CALL	DSP_BCD
	RETURN


DSP_ON_OFF:
					; enter wk0 0=off, 1 = on
	MOVLW	050H			; dsiplay on/off for
	CALL	DSP_CSR_ADD		; for active sensing / running status
	DSTR	M_OFF
	MOVF	WK0,W,A
	MULLW	006H
	CALL	DSP_TABLE
	RETURN


DSP_HOT_KEY:				; hot key function

	MOVLW	049H
	CALL	DSP_CSR_ADD

	BTFSC	HOT_KEY,7,A		; test event
	GOTO	DSP_HOT_EVENT

	DSTR	M_HOT_KEY
	MOVF	HOT_KEY,W,A
	MULLW	012H
	CALL	DSP_TABLE
	RETURN


DSP_HOT_EVENT:

	DSTR	M_EVENT
	CALL	DSP_STRING	

	MOVF	HOT_KEY,W,A
	ANDLW	07FH			; mask top bit
	MOVWF	WK3,A

	MOVLW	051H
	CALL	DSP_CSR_ADD
	BSF	ONE_FLG,A		; count from 1
	LFSR	FSR0,WK3
	CALL	TO_BCD_SINGLE
	MOVLW	007H			; 3 digits
	MOVWF	WK2,A
	CALL	DSP_BCD
	RETURN

DSP_BAUDRATE:				; display baud rate

	MOVLW	04BH
	CALL	DSP_CSR_ADD

	DSTR	M_BAUD			; point to actual values
	MOVF	BAUDRATE,W,A
	MULLW	008H
	CALL	DSP_TABLE
	RETURN

DSP_LCDTIME:				; display lcd backlight setting

	MOVLW	04FH
	CALL	DSP_CSR_ADD

	DSTR	M_LCDTIME		; get vakue from table
	MOVF	LCD_TIME,W,A
	MULLW	00AH
	CALL	DSP_TABLE
	RETURN

DSP_NOISE:

	MOVLW	04EH
	CALL	DSP_CSR_ADD

	LFSR	FSR0,NOISE_BFR		; buffer holds 0-7 for 6 - 13 bits
	MOVF	BFR_PTR,W,A
	IORWF	FSR0L,F,A

	DSTR	M_BITS
	MOVF	INDF0,W,A
	MULLW	00AH
	CALL	DSP_TABLE		
	RETURN

DSP_MAX_MIN:				; data already in max, min registers

	MOVLW	04BH			; min
	CALL	DSP_CSR_ADD
	MOVFF	MIN,LSB
	CALL	DSP_MM1
	
	MOVLW	053H			; max
	CALL	DSP_CSR_ADD
	MOVFF	MAX,LSB
					; fall thro'
DSP_MM1:				; multiply by 4 to restore 10 bits
					; and display as decimal
	CLRF	MSB,A
	BCF	STATUS,C,A
	RLCF	LSB,F,A
	RLCF	MSB,F,A
	BCF	STATUS,C,A
	RLCF	LSB,F,A
	RLCF	MSB,F,A

	LFSR	FSR0,MSB
	CALL	TO_BCD
	MOVLW	0FH			; 4 digits
	MOVWF	WK2,A
	CALL	DSP_BCD
	RETURN


			; ***************
			; EEPROM routines
			; ***************

READ_EEPROM:				; get 4 bytes from eeprom
	
	LFSR	FSR0,SERNO		; point to serial no buffer

	MOVLW	004H
	MOVWF	WK0,A
	
	BCF	EECON1,EEPGD,A		; point to eeprom
	BCF	EECON1,CFGS,A		; not configuration

	CLRF	EEADR,A			; load at start of eeprom

READ_EEPROM1:
	
	BSF	EECON1,RD,A
	MOVFF	EEDATA,POSTINC0
	
	INCF	EEADR,F,A			; inc eeprom address
	DECFSZ	WK0,F,A
	GOTO	READ_EEPROM1
	
	RETURN				; return if ok





					; pack to 64 byte boundary
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	
	NOP
	NOP
	NOP
	NOP
	NOP
	


			; *****************
			; Maps and Messages
			; *****************

	ORG	4000H

			; nb. keep to even number of characters
;		   "------------------------"	24 characters
M_ID:		DW " C-THRU MUSIC  AXiS 64  "
M_REVISION:	DW " Revision 3.5 ",00H

M_PGM_MODE:	DW " C-THRU MUSIC AXiS 64   "
		DW " Programme Mode         ",00H

M_SHUTDOWN:	DW " SHUTDOWN ?             "
		DW " + / - to confirm       ",00H



M_PLAY		DW "PLAY       Program:     "
		DW "Bank:      Group: ",00H
M_MIDI:		DW "MIDI",00H
M_UTIL:		DW "UTILITY ",00H
M_GROUP:	DW "001-008 ",00H
		DW "009-016 ",00H
		DW "017-024 ",00H
		DW "025-032 ",00H
		DW "033-040 ",00H
		DW "041-048 ",00H
		DW "049-056 ",00H
		DW "057-064 ",00H
		DW "065-072 ",00H
		DW "073-080 ",00H
		DW "081-088 ",00H
		DW "089-096 ",00H
		DW "097-104 ",00H
		DW "105-112 ",00H
		DW "113-120 ",00H
		DW "121-128 ",00H

M_TRANSPOSE:	DW "Transpose:              ",00H
M_CHANNEL:	DW "Transmit Channel:       ",00H  
M_ASSIGN:	DW "Assign >                ",00H
M_INVERT:	DW "Invert >                ",00H
M_VELOCITY:	DW "Velocity Table:         ",00H
M_RUNNING_STAT:	DW "Running Status:         ",00H
M_ACTIVE_SENSE:	DW "Active Sensing:         ",00H
M_HOT_SCRN:	DW "Hot Key:                ",00H
M_BAUDRATE: 	DW "Baud Rate:              ",00H
M_NOISE:	DW "Noise Margin:           ",00H
M_CALIBRATE:	DW "Calibrate: ---- <> ---- ",00H
M_BACKLIGHT:	DW "LCD Backlight:          ",00H
M_DEFAULT:	DW "Default Settings Loaded ",00H
M_SETUP:	DW "    -> System Set-Up    ",00H
M_ANALOGUE:	DW "    -> Analogue Set-Up  ",00H
M_VTBL:		DW "    -> Velocity Table   ",00H
M_SYS_DFLT:	DW "Default all settings ?  ",00H
M_DFLT_LEVEL:	DW "DEFAULT LEVEL 0000      ",00H
M_COUNT:	DW "COUNT         ",00H
M_SERNO:	DW "Serial Number ",00H
M_PLUS:		DW "+ to Default",00H
M_MINUS:	DW "- to Cancel ",00H

M_HARDWARE:	DW "PBend Wheel ",00H
		DW "Mod Wheel   ",00H
		DW "Knob Ctrl 1 ",00H
		DW "Knob Ctrl 2 ",00H
		DW "Foot Ctrl 1 ",00H
		DW "Foot Ctrl 2 ",00H
		DW "Breath Ctrl ",00H
M_CONTROLLER:				; 000H - 01FH
		DW "              ",00H
		DW "Modulation    ",00H
		DW "Breath Ctrl   ",00H
		DW "CC_03         ",00H
		DW "Foot Pedal    ",00H
		DW "Portamento    ",00H
		DW "Data Entry    ",00H
		DW "Volume        ",00H
		DW "Balance       ",00H
		DW "CC_09         ",00H
		DW "Pan           ",00H
		DW "Expression    ",00H
		DW "Effect Ctrl 1 ",00H
		DW "Effect Ctrl 2 ",00H
		DW "CC_14         ",00H
		DW "CC_15         ",00H
		DW "General #1    ",00H
		DW "General #2    ",00H
		DW "General #3    ",00H
		DW "General #4    ",00H
		DW "CC_20         ",00H
		DW "CC_21         ",00H
		DW "CC_22         ",00H
		DW "CC_23         ",00H
		DW "CC_24         ",00H
		DW "CC_25         ",00H
		DW "CC_26         ",00H
		DW "CC_27         ",00H
		DW "CC_28         ",00H
		DW "CC_29         ",00H
		DW "CC_30         ",00H
		DW "CC_31         ",00H
					; 040H - 05FH
		DW "Sustain       ",00H
		DW "Portamento    ",00H
		DW "Sostenuto     ",00H
		DW "Soft Pedal    ",00H
		DW "Legato        ",00H
		DW "Hold 2        ",00H
		DW "Sound Ctrl 1  ",00H
		DW "Sound Ctrl 2  ",00H
		DW "Sound Ctrl 3  ",00H
		DW "Sound Ctrl 4  ",00H
		DW "Sound Ctrl 5  ",00H
		DW "Sound Ctrl 6  ",00H
		DW "Sound Ctrl 7  ",00H
		DW "Sound Ctrl 8  ",00H
		DW "Sound Ctrl 9  ",00H
		DW "Sound Ctrl 10 ",00H
		DW "General #5    ",00H
		DW "General #6    ",00H
		DW "General #7    ",00H
		DW "General #8    ",00H
		DW "Portamento    ",00H
		DW "CC_85         ",00H
		DW "CC_86         ",00H
		DW "CC_87         ",00H
		DW "CC_88         ",00H
		DW "CC_89         ",00H
		DW "CC_90         ",00H
		DW "Ext.Eff.Depth ",00H
		DW "Tremelo Depth ",00H
		DW "Chorus Depth  ",00H
		DW "Celeste Depth ",00H
		DW "Phaser Depth  ",00H
					; 080H - 081H
		DW "Aftertouch    ",00H
		DW "Pitchbend     ",00H

M_NORMAL:	DW "Normal",00H
		DW "Invert",00H
M_OFF:		DW "OFF ",00H
		DW "ON  ",00H
M_HOT_KEY:	DW "(Off)           ",00H
		DW "All Notes Off   ",00H
		DW "Reset All Ctrls ",00H
		DW "Clear           ",00H
		DW "Panic!          ",00H
M_EVENT:	DW "Event #         ",00H
M_BAUD:		DW "9,600 ",00H
		DW "19,200",00H
		DW "38,400",00H
		DW "57,600",00H
M_BITS:		DW "6 Bits  ",00H
		DW "7 Bits  ",00H
		DW "8 Bits  ",00H
		DW "9 Bits  ",00H
		DW "10 Bits ",00H
		DW "11 Bits ",00H
		DW "12 Bits ",00H
		DW "13 Bits ",00H
M_LCDTIME:	DW "On      ",00H
		DW "15 Sec. ",00H
		DW "30 Sec. ",00H
		DW "1 Min.  ",00H
		DW "Off     ",00H

DFLT_ASS_MAP:	DB 081H,001H,007H,080H,004H,040H,002H,000H
		DB 000H,000H,000H,000H,000H,000H,000H,000H
		; default assignment of controllers
DFLT_NULL_MAP:	DB 000H,000H,000H,000H,000H,000H,000H,000H
		DB 000H,000H,000H,000H,000H,000H,000H,000H
		; default null map for invert,offset
DFLT_NOISE_MAP:	DB 000H,000H,000H,000H,006H,006H,000H,000H
		DB 000H,000H,000H,000H,000H,000H,000H,000H
		; default map for noise rev 3.3
DFLT_SCL_MAP:	DB 001H,000H,001H,000H,001H,000H,001H,000H
		DB 001H,000H,001H,000H,001H,000H,001H,000H
		; default scale map
UART_MAP:	DB 040H,020H,081H,024H,040H,024H,02AH,024H
		; SPBRG, TXSTA for each baud rate
BITMAP:		DB 001H,002H,004H,008H,010H,020H,040H,080H
;ATODMAP:	DB 004H,005H,007H,001H,000H,002H,006H,003H
ATODMAP:	DB 005H,004H,007H,001H,000H,002H,006H,003H
LCD_MAP:	DB 00FH,00FH,01EH,03CH 


		; may need packing data at this point




		; place velocity tables at start of page (256 bytes)

	ORG 5000H

DFLT_VTBL1:               
VELTBL1:        DB 07FH,07FH,07FH,07FH,07DH,07AH,076H,072H
                DB 06EH,06AH,066H,062H,05EH,05AH,056H,052H
                DB 04EH,04AH,046H,042H,03EH,03CH,038H,034H
                DB 030H,02CH,028H,024H,020H,01FH,01EH,01DH
                DB 01CH,01BH,01AH,019H,018H,017H,016H,015H
                DB 014H,013H,012H,011H,010H,00FH,00EH,00DH
                DB 00CH,00BH,00AH,009H,008H,007H,006H,005H
                DB 004H,003H,002H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,000H

DFLT_VTBL2:
VELTBL2:        DB 07FH,07FH,07FH,07DH,078H,073H,06EH,069H
                DB 064H,05FH,05AH,055H,050H,04BH,046H,041H
                DB 03CH,037H,032H,02DH,028H,023H,01EH,019H
                DB 014H,013H,012H,011H,010H,00FH,00EH,00DH
                DB 00CH,00BH,00AH,009H,008H,007H,006H,005H
                DB 004H,003H,002H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,000H

DFLT_VTBL3:
VELTBL3:        DB 07FH,07FH,07FH,07FH,07FH,07DH,078H,073H
                DB 06EH,069H,064H,05FH,05AH,055H,050H,04BH
                DB 046H,041H,03CH,037H,032H,02DH,028H,023H
                DB 01EH,01DH,01CH,01BH,01AH,019H,018H,017H
                DB 016H,015H,014H,013H,012H,011H,010H,00FH
                DB 00EH,00DH,00CH,00BH,00AH,009H,008H,007H
                DB 006H,005H,004H,003H,002H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,000H

DFLT_VTBL4:
VELTBL4:        DB 064H,064H,064H,064H,064H,062H,060H,05EH
                DB 05CH,05AH,058H,054H,050H,04BH,046H,041H
                DB 03CH,037H,032H,02DH,028H,023H,01EH,019H
                DB 014H,013H,012H,011H,010H,00FH,00EH,00DH
                DB 00CH,00BH,00AH,009H,008H,007H,006H,005H
                DB 004H,003H,002H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,001H
                DB 001H,001H,001H,001H,001H,001H,001H,000H

DFLT_VTBL5:        
VELTBL5:        DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,040H
                DB 040H,040H,040H,040H,040H,040H,040H,000H


			; **************
			; Bootstrap Code
			; **************

	ORG	7000H

ERROR_TRAP:

	GOTO	ERROR_TRAP		; trap any run-away from normal 
					; operation

BOOT_INTERRUPT:

	BTFSC	PIR1,RCIF,A
	CALL	BOOT_SERIAL_INT

	BTFSC	PIR1,TMR1IF,A		; in case interrupts still enabled
	BCF	PIR1,TMR1IF,A		; icd mode only

	BTFSC	INTCON,TMR0IF,A
	BCF	INTCON,TMR0IF,A

	RETFIE	FAST			; restore w, bsr, status



BOOT_INIT:

	CLRF	INTCON,A		; ensure no interrupts

	BCF	T0CON,TMR0ON,A		; stop timer0
	CALL	CLEAR_RAM		; clear ram 000h thro' 5ffh
	
	BCF	RCON,IPEN,A		; single level priority on interrupts
	CLRF	EECON1,A		; ensure wren disabled

					; set port i/o
					; port a
	MOVLW	00EH
	MOVWF	ADCON1,A		; all digital except an0
	MOVLW	014H
	MOVWF	LATA,A
	MOVLW	B'00101011'		; midi output, debug led output
	MOVWF	TRISA,A			; all input
					; port b
	MOVLW	0FFH
	MOVWF	LATB,A
	MOVLW	B'11001111'		; LED outputs - rest input
	MOVWF	TRISB,A			; 
					
					; port c
	MOVLW	0FFH
	MOVWF	LATC,A
	MOVLW	B'10111001'		; TXD , FOUT, CS1 as output
	MOVWF	TRISC,A
					; port d
	MOVLW	B'00001111'		; lcd data low, fram lines high
	MOVWF	LATD,A			; backlight high (on)
	MOVLW	B'00001000'		; LCD data and fram clk, cs0, backligh
	MOVWF	TRISD,A			; all outputs
					; port e
	CLRF	LATE,A			; all output
	CLRF	TRISE,A
					; init serial i/o

	MOVLW	02AH			; 57600 baud
	MOVWF	SPBRG,A
	MOVLW	024H
	MOVWF	TXSTA,A
	MOVLW	090H			; spen, cren
	MOVWF	RCSTA,A					
					; init i2c as master

BOOT_INIT0:

	MOVLW	080H			; slew rate disabled
	MOVWF	SSPSTAT,A

	MOVLW	018H			; 400 khz
	MOVWF	SSPADD,A

	CLRF	SSPCON2,A
	MOVLW	B'00101000'		; sspen, master mode fosc/4*(sspadd+1)
	MOVWF	SSPCON1,A

					; enable serial interrupt only
	CLRF	INTCON,A
	CLRF	INTCON2,A		; ext int on falling edge
					; weak pull-up on port b
	CLRF	INTCON3,A

	CLRF	PIE1,A
	CLRF	PIE2,A
	BSF	PIE1,RCIE,A		; serial rec interrupt

	
	CLRF	TBLPTRU,A		; assume all maps and messages
					; in range 0000h to ffffh
					
	BSF	INTCON,PEIE,A		; enable peripheral interrupts
	BSF	INTCON,GIE,A		; enable global interrupts


			; *****************
			; Boot Main Routine
			; *****************

BOOT_MAIN:

	LFSR	FSR0,IPBFR
	MOVLW	001H		; place other processors in boot mode
	MOVWF	POSTINC0,A
	MOVLW	04H
	MOVWF	POSTINC0,A
	MOVLW	0BH
	MOVWF	POSTINC0,A
	MOVLW	10H
	MOVWF	POSTINC0,A

	CALL	BROADCAST

	CALL	DELAY_BOOT

BOOT_MAIN0:


	BTFSC	SSPCON1,SSPOV,A		; test overflow of i2c
	GOTO	BOOT_I2C_ERROR1

	BTFSC	SSPCON1,WCOL,A		; test write collision
	GOTO	BOOT_I2C_ERROR2

	CLRWDT

	BTFSC	BFR_FULL_FLG,A
	CALL	BOOT_SERIN

	BSF	LED_DEBUG,A		; test standard key interrupts
	MOVLW	K1ADD
	BTFSS	KEY_INT1,A
	CALL	XFER_PKT

	BSF	LED_DEBUG,A
	MOVLW	K2ADD
	BTFSS	KEY_INT2,A
	CALL	XFER_PKT

	BSF	LED_DEBUG,A
	MOVLW	K3ADD
	BTFSS	KEY_INT3,A
	CALL	XFER_PKT

	BSF	LED_DEBUG,A
	MOVLW	FKADD
	BTFSS	KEY_INT0,A
	CALL	XFER_PKT

	GOTO	BOOT_MAIN0



			; ******************
			; I2C Error Routines
			; ******************


BOOT_I2C_ERROR1:			; overlow on receive

	MOVF	SSPBUF,W,A		; reset BF in sspstat
	BCF	SSPCON1,SSPOV,A
	GOTO	BOOT_INIT0

BOOT_I2C_ERROR2:			; collision detect on transmit

	BCF	SSPCON1,WCOL,A
	GOTO	BOOT_INIT0




			; ***************
			; Transfer packet
			; ***************

XFER_PKT:		; transfer packet from other processor to serial port

	MOVWF	I2CADD,A		; enter w reg holds i2caddress
	CALL	BOOT_READ_I2C

	MOVFF	OPBFR+1,WK2		; get length
	LFSR	FSR0,OPBFR
	
XFER1:
	MOVF	POSTINC0,W,A
	CALL	XMIT		
	DECFSZ	WK2,F,A
	GOTO	XFER1
	RETURN


			; *********************
			; Boot serial interrupt
			; *********************


BOOT_SERIAL_INT:
					; use fsr1 - reserved fo serial interrupt		
				
	MOVFF	RCREG,RXDATA		; get data and store in rxdata

	CALL	PARITY_CHECK
	BTFSS	STATUS,Z,A
	RETURN


	MOVLW	07FH			; mask parity bit
	ANDWF	RXDATA,F,A
	MOVF	SIPTR,W,A
	BNZ	BOOT_INT1

	MOVLW	001H			; test valid header
	XORWF	RXDATA,W,A
	BTFSS	STATUS,Z,A		; match - place header in buffer
	RETURN

BOOT_INT1:				; place data in buffer

	BTFSC	BFR_FULL_FLG,A
	RETURN				; ignore data if buffer already full

	LFSR	FSR1,IPBFR
	MOVFF	SIPTR,FSR1L		; set low pointer	
	MOVFF	RXDATA,INDF1		; data to buffer

	INCF	SIPTR,F,A		; inc pointer
	MOVLW	003H
	CPFSGT	SIPTR,A			; skip if siptr > 3
	RETURN				; header, length, id only

	LFSR	FSR1,IPBFR+1		; get packet length
	MOVF	INDF1,W,A
	XORWF	SIPTR,W,A
	BTFSC	STATUS,Z,A		; end of packet
	BSF	BFR_FULL_FLG,A		; say buffer full
	RETURN


			; ******************
			; Handle Serial Data
			; ******************

BOOT_SERIN:

	DECF	SIPTR,W,A		; get byte count for csum verification
	MOVWF	WK2,A

	LFSR	FSR0,IPBFR		; verify checksum - simple addition header
					; to last byte of data
	MOVLW	000H			; init csum
BOOT_SER1:

	ADDWF	POSTINC0,W,A
	DECFSZ	WK2,F,A
	GOTO	BOOT_SER1

	ANDLW	07FH			; 7 data only

	XORWF	INDF0,W,A		; fsr0 -> csum
	BTFSS	STATUS,Z,A
	GOTO	BOOT_NAK		; csum fail
	
					; valid packet

	MOVFF	IPBFR+2,WK0		; packet id to wk0

	BTFSS	WK0,6,A			; test control address
	GOTO	BOOT_OTHER

	MOVF	WK0,W,A
	XORLW	048H
	BNZ	BOOT_SER2		; test enq packet

	MOVLW	04FH			; say boot mode
	CALL	SEND_PKT
	GOTO	BOOT_SERX

BOOT_SER2:

	MOVF	WK0,W,A			; test exit boot
	XORLW	04AH
	BNZ	BOOT_SER3

	LFSR	FSR0,IPBFR
	MOVLW	001H		; place other processors in normal mode
	MOVWF	POSTINC0,A
	MOVLW	04H
	MOVWF	POSTINC0,A
	MOVLW	0AH
	MOVWF	POSTINC0,A
	MOVLW	0FH
	MOVWF	POSTINC0,A

	CALL	BROADCAST

	CALL	RETURN_NORM		; re-write normal vector
	RESET
	
BOOT_SER3:

	MOVF	WK0,W,A
	XORLW	040H			; test pgm data
	BNZ	BOOT_SER4

	CALL	VERIFY_HEX
	BTFSS	STATUS,Z,A
	GOTO	BOOT_NAK

	CALL	PROGRAM_UPDATE
	GOTO	BOOT_ACK

BOOT_SER4:

	MOVF	WK0,W,A
	XORLW	049H		; test crc request
	BNZ	BOOT_SERX

	CALL	SEND_CRC_PKT
	GOTO	BOOT_SERX


BOOT_ACK:

	MOVLW	04CH
	CALL	SEND_PKT
					; fall thro to clear buffer

BOOT_SERX:

	CLRF	SIPTR,A			; reset pointer
	BCF	BFR_FULL_FLG,A		; re-enable input
	RETURN


BOOT_NAK:

	MOVLW	04DH			; nak packet
	CALL	SEND_PKT
	GOTO	BOOT_SERX

BOOT_OTHER:

	MOVFF	IPBFR+2,WK0		; get packet id
	SWAPF	WK0,F,A			; 00, 01, 02, 03
	MOVLW	007H
	ANDWF	WK0,F,A
	RLNCF	WK0,F,A
	BSF	WK0,4,A			
	MOVFF	WK0,I2CADD

	CALL	BOOT_WRITE_I2C
	
	GOTO	BOOT_SERX		; simply return after passing data
					; remote processor will generate reply


			; ***************
			; Verify Hex File
			; ***************


VERIFY_HEX:


	LFSR	FSR0,IPBFR+3		; -> :
	MOVF	POSTINC0,W,A
	XORLW	03AH
	BTFSS	STATUS,Z,A
	RETURN				; return non-zero if fail

	CLRF	WK0,A	; reset csum
	
	MOVLW	015H			; line lenght in bytes - always full line
	MOVWF	WK2,A			; including csum

VERIFY_HEX1:	
	
	CALL	GET_HEX_BYTE
	ADDWF	WK0,F,A

	DECFSZ	WK2,F,A
	GOTO	VERIFY_HEX1
	MOVF	WK0,W,A			; return zero if ok

	RETURN


GET_HEX_BYTE:				; get hex byte @ fsr0

	CALL	GET_HEX_NIBBLE
	MOVWF	WK1,A
	SWAPF	WK1,F,A			; xfer to top nibble
	CALL	GET_HEX_NIBBLE
	ADDWF	WK1,W,A
	RETURN


GET_HEX_NIBBLE:				; get single nibble from hex file

	MOVLW	000H
	BTFSC	INDF0,6,A			; test a-f
	MOVLW	009H
	ADDWF	POSTINC0,W,A
	ANDLW	00FH
	RETURN

			; ****************
			; Programme Update
			; ****************

PROGRAM_UPDATE:		; parity check and csum on packet ok
		

	LFSR	FSR0,IPBFR+6
	CALL	GET_HEX_BYTE
	MOVWF	TBLPTRH,A

	CALL	GET_HEX_BYTE
	MOVWF	TBLPTRL,A

PU1:

	LFSR	FSR0,IPBFR+0CH

	LFSR	FSR2,MBFR	; output buffer

	MOVF	TBLPTRL,W,A
	ANDLW	030H
	MOVWF	FSR2L,A

	MOVLW	008H		; 8 words in total
	MOVWF	WK2,A

PU2:

	CALL	GET_HEX_BYTE
	MOVWF	LSB,A
	CALL	GET_HEX_BYTE
	MOVWF	MSB,A

	MOVFF	LSB,POSTINC2
	MOVFF	MSB,POSTINC2

	DECFSZ	WK2,F,A
	GOTO	PU2

	MOVF	TBLPTRL,W,A
	ANDLW	030H		; test 4th line
	XORLW	030H
	BTFSS	STATUS,Z,A
	RETURN			


	BCF	LED_DEBUG,A

	MOVLW	0C0H
	ANDWF	TBLPTRL,F,A	; tblptr -> start of 64 byte block

	MOVF	TBLPTRH,W,A	; test power on vector
	IORWF	TBLPTRL,W,A
	BNZ	PU3

	CALL	INIT_CRC		; init crc if first block

	DSTR	NORMAL_VECTOR

PU3:
	MOVFF	TBLPTRL,TPTRL		; save tblptr
	MOVFF	TBLPTRH,TPTRH

	CALL	WRITE_FLASH

	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH

	MOVLW	040H			; bytes to verify
	MOVWF	WK2,A
	LFSR	FSR0,MBFR

PU4:
	TBLRD*+

	MOVF	TABLAT,W,A
	XORWF	POSTINC0,W,A
	BTFSS	STATUS,Z,A
	GOTO	PU5			; fail - try write once more			
	DECFSZ	WK2,F,A
	GOTO	PU4

					; update crc after verify
	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH

	MOVLW	040H
	MOVWF	WK2,A

PU6:

	TBLRD*+

	CALL	CRC

	DECFSZ	WK2,F,A
	GOTO	PU6
	
	BSF	LED_DEBUG,A
	RETURN

PU5:
	CLRWDT
	
	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH
	GOTO	PU3			; go try again


			; ***********
			; CRC Routine
			; ***********
INIT_CRC:

	MOVLW	0FFH
	MOVWF	CRCL,A
	MOVWF	CRCH,A
	RETURN

CRC:			; enter - data in tablat
			; WK0,WK1,WK3 destroyed

	MOVFF	TABLAT,WK1

	MOVLW	008H
	MOVWF	WK3,A	; bit count


CRC1:

	MOVF	CRCL,W,A	; xor lsbits
	XORWF	WK1,W,A
	MOVWF	WK0,A		; wk0 stores result of xor

	BCF	STATUS,C,A	; clear c
	RRCF	CRCH,F,A
	RRCF	CRCL,F,A

	BTFSS	WK0,0,A		; test reult of xor
	GOTO	CRC2

	MOVLW	084H
	XORWF	CRCH,F,A
	MOVLW	008H
	XORWF	CRCL,F,A

CRC2:
	RRNCF	WK1,F,A
	DECFSZ	WK3,F,A
	GOTO	CRC1
	RETURN


			; ********************
			; Read / Write Vectors
			; ********************

COPY_NORM_VECTOR:	; copy normal vetor to normal_vector

	DSTR	PON_RESET
	CALL	READ_FLASH

	DSTR	NORMAL_VECTOR
	CALL	WRITE_FLASH

	RETURN
	
WRITE_BOOT_VECTOR:	; replace normal vectors withboot vectors

	DSTR	BOOT_VECTOR
	CALL	READ_FLASH

	DSTR	PON_RESET
	CALL	WRITE_FLASH
	RETURN

RETURN_NORM:

	DSTR	NORMAL_VECTOR
	CALL	READ_FLASH	; get normal vector

	NOP
	NOP

	DSTR	PON_RESET
	CALL	WRITE_FLASH
	RETURN



			; **************
			; Flash Routines
			; **************
			;  all flash routines based on 64 byte transfer
			; mbfr used for ram
			; fsr0 as indirect pointer

READ_FLASH:		; read 64 bytes flash @ tblptr to mbfr
			; use dstr to set tblptr

	BCF	EECON1,EEPGD,A
	BCF	EECON1,CFGS,A
	LFSR	FSR0,MBFR
	MOVLW	040H		; 64 bytes
	MOVWF	WK2,A

READ_FLASH1:

	TBLRD*+	
	MOVFF	TABLAT,POSTINC0	; xfer data
	DECFSZ	WK2,F,A
	GOTO	READ_FLASH1
	RETURN


ERASE_FLASH:

	BCF	INTCON,GIE,A	; disable all interrupts during write

	BCF	EECON1,CFGS,A
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A	; enable write
	BSF	EECON1,FREE,A	; line erase (64 bytes)

	MOVLW	055H		; timed access
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A

	BSF	EECON1,WR,A
	NOP
	BSF	INTCON,GIE,A
	RETURN



WRITE_FLASH:

	BCF	INTCON,GIE,A	; disable all interrupts during write

	BCF	EECON1,CFGS,A
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A	; enable write
	BSF	EECON1,FREE,A	; line erase (64 bytes)

	MOVLW	055H		; timed access
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A

	BSF	EECON1,WR,A
	NOP

				; write data in mbfr @ tblptr
				; write 8 blocks of 8 bytes @ tblptr

	LFSR	FSR0,MBFR	; point to ram
	MOVLW	008H
	MOVWF	WK3,A		; 8 loops of 8 bytes each

WRITE_FLASH1:

	MOVLW	008H		; byte count
	MOVWF	WK2,A

WRITE_FLASH2:

	MOVF	POSTINC0,W,A	; get data from buffer	
	MOVWF	TABLAT,A	; 8 bytes to temp registers
	TBLWT*+

	DECFSZ	WK2,F,A
	GOTO	WRITE_FLASH2

	TBLRD*-			; dec tblptr to ensure written 8 byte earlier
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A
	
	MOVLW	055H
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A
	BSF	EECON1,WR,A
	NOP
	
	TBLRD*+			; restore tablptr
	DECFSZ	WK3,F,A
	GOTO	WRITE_FLASH1
	BCF	EECON1,WREN,A	; disable further writes
	BSF	INTCON,GIE,A
	RETURN


			; ***************
			; Packet Routines
			; ***************

SEND_PKT:

	MOVWF	WK1,A
	CLRF	TXCSUM,A		; reset checksum
	MOVLW	001H			; header
	CALL	XMIT
	MOVLW	004H			; packet length = 4
	CALL	XMIT

	MOVF	WK1,W,A			; get pkt id
	CALL	XMIT

	MOVF	TXCSUM,W,A
	CALL	XMIT			; fall thro'
	RETURN

SEND_CRC_PKT:				; CRC held in msb, lsb

	CLRF	TXCSUM,A
	MOVLW	001H
	CALL	XMIT
	MOVLW	008H
	CALL	XMIT

	MOVLW	044H			; crc id
	CALL	XMIT

	MOVFF	CRCH,WK0
	CALL	XMIT_HEX
	MOVFF	CRCL,WK0
	CALL	XMIT_HEX

	MOVF	TXCSUM,W,A
	CALL	XMIT
	RETURN

			; **********************
			; Transmit Data Routines
			; **********************

			; Shared between boot and normal code


XMIT:					; xmit data in w reg to serial port

	ANDLW	07FH
	MOVWF	TXDATA,A		; save data
	MOVWF	TPARITY,A
	CALL	SET_PARITY

	MOVWF	TXREG,A
	ADDWF	TXCSUM,F,A		; update checksum
	
XMIT1:
	BTFSS	PIR1,TXIF,A		; test xmission complete
	GOTO	XMIT1
	RETURN


XMIT_HEX:				; enter data in wk0

	SWAPF	WK0,W,A
	ANDLW	00FH
	MOVWF	WK1,A

	CALL	XMIT_H1

	MOVF	WK0,W,A
	ANDLW	00FH
	MOVWF	WK1,A			; fall thro for low nibble

XMIT_H1:

	MOVLW	00AH
	CPFSLT	WK1,A
	GOTO	XMIT_H2

	MOVF	WK1,W,A			; 0-9
	IORLW	030H
	CALL	XMIT
	RETURN
XMIT_H2:				; A-F

	MOVF	WK1,W,A
	ADDLW	'A'-00AH
	CALL	XMIT
	RETURN



SET_PARITY:				; set even parity on transmitted data

	MOVLW	007H
	MOVWF	TPCNT,A
	CLRF	TPVAL,A
	MOVLW	080H

SP1:
	RRCF	TPARITY,F,A
	BTFSC	STATUS,C
	XORWF	TPVAL,F,A
	DECFSZ	TPCNT,F,A
	GOTO	SP1
	
	MOVF	TXDATA,W,A
	IORWF	TPVAL,W,A
	RETURN

PARITY_CHECK:				; parity check for recd data
					; in rxdata
	MOVFF	RXDATA,PARITY		; return zero if ok
	MOVLW	008H
	MOVWF	PCNT,A
	CLRF	PVAL,A
	MOVLW	080H			; use to xor top bit of val byte
CP1:

	RRCF	PARITY,F,A
	BTFSC	STATUS,C,A		; test carry
	XORWF	PVAL,F,A
	DECFSZ	PCNT,F,A
	GOTO	CP1

	MOVF	PVAL,W,A		; return zero if ok
	RETURN


			; *****************
			; Boot i2c Routines
			; *****************
	

BOOT_WRITE_I2C:		; write pkt in ipbfr to i2cadd
			; WK2 destroyrd

	LFSR	FSR0,IPBFR
	MOVFF	IPBFR+1,WK2		; get length of data


BOOT_WRITE_I2C1:

	CALL	BOOT_TEST_I2C_IDLE	; test bus idle
	BTFSS	STATUS,Z,A
	GOTO	BOOT_WRITE_I2C1		; loop until idle
					; wdog will reset all if error !!

	BSF	I2CERR_FLG,A		; preset error
	BSF	SSPCON2,SEN,A		; start bit

BOOT_WRITE_I2C_START:

	BTFSC	SSPCON2,SEN,A		; wait on start bit to be cleared by hardware
	GOTO	BOOT_WRITE_I2C_START	

	MOVF	I2CADD,W,A		; write address
	CALL	BOOT_PUT_I2C
	BTFSC	SSPCON2,ACKSTAT,A	; test ack negative
	GOTO	BOOT_WRITE_I2C_STOP

BOOT_WRITE_I2C_DATA:

	MOVF	POSTINC0,W,A		; get data from buffer

	CALL	BOOT_PUT_I2C
	BTFSC	SSPCON2,ACKSTAT,A
	GOTO	BOOT_WRITE_I2C_STOP

	DECFSZ	WK2,F,A
	GOTO	BOOT_WRITE_I2C_DATA

	BCF	I2CERR_FLG,A		; say transfer ok

BOOT_WRITE_I2C_STOP:

	BSF	SSPCON2,PEN,A

BOOT_WRITE_I2C_STOP1:

	BTFSC	SSPCON2,PEN,A
	GOTO	BOOT_WRITE_I2C_STOP1
	RETURN


BOOT_PUT_I2C:				; put w reg to i2c bus

	MOVWF	SSPBUF,A
BOOT_PUT1:
	BTFSC	SSPSTAT,R_W,A		; wait until byte sent
	GOTO	BOOT_PUT1		; R/W cleared after ack bit rec'd
	RETURN
	


BOOT_TEST_I2C_IDLE:				; return zero if idle

	BTFSC	SSPCON1,WCOL,A		; test write collision
	BCF	SSPCON1,WCOL,A		; clear if so

	BTFSC	SSPSTAT,R_W,A
	RETLW	0FFH			; say busy if r/w set

	MOVF	SSPCON2,W,A
	ANDLW	01FH			; mask acken, rcen,pen,rsen,sen
	RETURN



BOOT_READ_I2C:				; read  @ i2cadd
					; place data in opbfr for
					; onward transmission
	LFSR	FSR0,OPBFR

	CALL	BOOT_TEST_I2C_IDLE	; test bus idle
	BTFSS	STATUS,Z,A
	GOTO	BOOT_READ_I2C		; loop until idle
					; wdog will reset all if error !!

	BSF	I2CERR_FLG,A		; preset error bit
	BSF	SSPCON2,SEN,A		; start bit

BOOT_READ_I2C_START:

	BTFSC	SSPCON2,SEN,A		; wait on start bit to be cleared by hardware
	GOTO	BOOT_READ_I2C_START	

	INCF	I2CADD,W,A		; read address = write address+1
	CALL	BOOT_PUT_I2C
	BTFSC	SSPCON2,ACKSTAT,A	; test ack negative
	GOTO	BOOT_READ_I2C_STOP
					; assume bus idle after ack from slave

	CALL	BOOT_GET_I2C		; get first byte
	BCF	SSPCON2,ACKDT,A		; ack first data byte
	CALL	BOOT_GET_ACK

	MOVFF	SSPBUF,POSTINC0
					; get 2nd byte
	CALL	BOOT_GET_I2C			; 
	BCF	SSPCON2,ACKDT,A		; ack 2nd data byte
	CALL	BOOT_GET_ACK

	MOVFF	SSPBUF,WK0		; save pkt len
	MOVFF	WK0,POSTINC0

	DECF	WK0,F,A
	DECF	WK0,F,A			; allow for 2 bytes already recd
	DECF	WK0,F,A			; also allow for not ack on final byte

BOOT_READ_I2C_LOOP:

	CALL	BOOT_GET_I2C		; get remaining data less last byte
	BCF	SSPCON2,ACKDT,A
	CALL	BOOT_GET_ACK

	MOVFF	SSPBUF,POSTINC0
	DECFSZ	WK0,F,A
	GOTO	BOOT_READ_I2C_LOOP

					; get final byte
	CALL	BOOT_GET_I2C			; 
	BSF	SSPCON2,ACKDT,A		; no ack for final byte
	CALL	BOOT_GET_ACK

	MOVFF	SSPBUF,POSTINC0

	BCF	I2CERR_FLG,A		; say transfer ok

BOOT_READ_I2C_STOP:

	BSF	SSPCON2,PEN,A

BOOT_READ_I2C_STOP1:

	BTFSC	SSPCON2,PEN,A
	GOTO	BOOT_READ_I2C_STOP1
	RETURN



BOOT_GET_I2C:				; get data byte from i2c bus

	BSF	SSPCON2,RCEN,A		; initiate data transfer

BOOT_GET_I2C_DATA:

	BTFSS	SSPSTAT,BF,A		; test buffer full
	GOTO	BOOT_GET_I2C_DATA
	RETURN


BOOT_GET_ACK:
	BSF	SSPCON2,ACKEN,A
BOOT_GET_ACK1:

	BTFSC	SSPCON2,ACKEN,A
	GOTO	BOOT_GET_ACK1
	RETURN


			; *********
			; Broadcast
			; *********

BROADCAST:		; xmit packet in ipbfr to all other processors
			; used to enter and exit boot

	MOVLW	002H
	MOVWF	WK3,A

BCLOOP:

	MOVLW	FKADD
	CALL	BCAST
	MOVLW	K1ADD
	CALL	BCAST
	MOVLW	K2ADD
	CALL	BCAST
	MOVLW	K3ADD
	CALL	BCAST

	DECFSZ	WK3,F,A
	GOTO	BCLOOP
	RETURN

BCAST:
	MOVWF	I2CADD
	CALL	BOOT_WRITE_I2C
	CALL	DELAY_2MSEC
	RETURN


			; *********
			; Clear Ram
			; *********

			; Sharded between boot and normal code

CLEAR_RAM:

	LFSR	FSR0,0000H		; clear ram 000h thro 5ffh

CLR_LOOP:

	CLRF	POSTINC0,A
	
	MOVF	FSR0H,W,A		; high pointer to w reg
	XORLW	006H
	BTFSS	STATUS,Z,A
	GOTO	CLR_LOOP
	RETURN


			; ********************
			; Non Interrupt Delays
			; ********************

DELAY_BOOT:

	CLRF	DLY2,A
	CLRF	DLY1,A
	GOTO	DELAY_LOOP


DELAY_2MSEC:		; WK2,WK3 destroyed

	MOVLW	018H
	MOVWF	DLY2,A

	CLRF	DLY1,A

DELAY_LOOP:

	DECFSZ	DLY1,F,A
	GOTO	DELAY_LOOP

	DECFSZ	DLY2,F,A
	GOTO	DELAY_LOOP
	RETURN


	ORG	7780H

BOOT_VECTOR:

	GOTO	BOOT_INIT
	
	ORG	7788H
	GOTO	BOOT_INTERRUPT




	ORG	7800H

NORMAL_VECTOR:				; normal vector copied here
					; on entering pgm mode








	END
