
	LIST P=PIC18F452

			; ************************
			; MIDI System Function Key
			; and Analogue Controller
			; Revision 3_0
			; May 2006
			; ************************


#INCLUDE <C:\Program Files\MICROCHIP\MPASM Suite\p18f452.inc>

; conditional compilation
; =======================

UNIT	EQU	000H
DEBUG	EQU	080H	; debug mode
ICD	EQU	020H

; development
; ===========

; MPLAB IDE 7.00
; MPLAB ICD2
; MPLAB MASM

; configuration
; =============

;Program Configuration Register 1H
		__CONFIG    _CONFIG1H, _OSCS_OFF_1H & _HSPLL_OSC_1H

;Program Configuration Register 2L
		__CONFIG    _CONFIG2L, _BOR_OFF_2L & _BORV_27_2L & _PWRT_ON_2L

;Program Configuration Register 3H
		__CONFIG    _CONFIG3H, _CCP2MX_ON_3H

;Program Configuration Register 6L
		__CONFIG    _CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L & _WRT2_OFF_6L & _WRT3_OFF_6L 

;Program Configuration Register 6H
		__CONFIG    _CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H & _WRTD_OFF_6H

;Program Configuration Register 7L
		__CONFIG    _CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L & _EBTR2_OFF_7L & _EBTR3_OFF_7L

;Program Configuration Register 7H
		__CONFIG    _CONFIG7H, _EBTRB_OFF_7H


.IF (UNIT & DEBUG)

;Program Configuration Register 2H
		__CONFIG    _CONFIG2H, _WDT_OFF_2H & _WDTPS_128_2H

;Program Configuration Register 5L
		__CONFIG    _CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L & _CP2_OFF_5L & _CP3_OFF_5L 

;Program Configuration Register 5H
		__CONFIG    _CONFIG5H, _CPB_OFF_5H & _CPD_OFF_5H

.ELSE

;Program Configuration Register 2H
		__CONFIG    _CONFIG2H, _WDT_ON_2H & _WDTPS_128_2H

;Program Configuration Register 5L
		__CONFIG    _CONFIG5L, _CP0_ON_5L & _CP1_ON_5L & _CP2_ON_5L & _CP3_ON_5L 

;Program Configuration Register 5H
		__CONFIG    _CONFIG5H, _CPB_ON_5H & _CPD_OFF_5H

.ENDIF


.IF (UNIT & ICD)

;Program Configuration Register 4L
		__CONFIG    _CONFIG4L, _STVR_ON_4L & _LVP_OFF_4L & _DEBUG_ON_4L

.ELSE

;Program Configuration Register 4L
		__CONFIG    _CONFIG4L, _STVR_ON_4L & _LVP_OFF_4L & _DEBUG_OFF_4L

.ENDIF


; firmware
; ========

; packet format
; byte 0	id 		00h to 0fh for key
;				10h to 17h for analogue
; byte 1	direction	0ffh key down / 00h key released
;		value		analogue value / 4 (8bits)
; byte 2	csum		byte0 xor byte1

; FSR2 used exclusively in i2c interupt


; rev 1_1
; =======

; all non - programme keys reported in both directions
; required to detect play and midi down together

; rev 1_2
; =======

; i2c interrup - ensure no write after no ack interrupt
;	to avoid collision on bus

; rev 3_0
; =======

; support for serial line programming

; rev 3_1
; =======

; full scan of analogue channels to set old and new values
; -> stop analogue strings at power up

; anlogue channels 2 and 6 no longer reported to main processor
; stop unwanted noise from unused (chan2) and breath control (chan 6)


; hardware
; ========

; 10 MHz xtal with PLL enabled (40 MHz effective)
; function key  i2c address 16h
; i2c operates at 400KHz

#DEFINE	INT_OUT		PORTC,0
#DEFINE	DEBUG_OUT	PORTC,1

#DEFINE	LED_DEBUG	PORTA,4

; Constants
; =========

FKADD	EQU	16H			; i2c address

; Register Map
; ============

; BANK 0
; ======

					; analogue buffer

NEWBFR		EQU	00H		; 8*2 bytes for analogue buffer
OLDBFR		EQU	10H

					; key switch control

NEW1		EQU	20H		; port d
NEW2		EQU	21H		; port b (0-5), port c (6-7)

OLD1		EQU	22H		; copy of previous scan
OLD2		EQU	23H

XOR1		EQU	24H		; XOR old and new
XOR2		EQU	25H

KEY_ID		EQU	26H
KEY_DIR		EQU	27H

COUNT		EQU	28H		; dec on 4 msec basis to zero
					; timeout flag set
BITPTR		EQU	29H		; gp bit pointer
CHANPTR		EQU	2AH		; analogue channel pointer
COUTPTR		EQU	2BH		; channel output pointer
ADLY		EQU	2CH		; analogue delay register


RXPTR		EQU	50H		; i2c input pointer
RXLEN		EQU	51H		; i2c rx pkt len
TXLEN		EQU	52H		; i2c tx pkt len
TXCSUM		EQU	53H

MSB		EQU	54H
LSB		EQU	55H

CRCH		EQU	56H
CRCL		EQU	57H

TPTRH		EQU	58H
TPTRL		EQU	59H

I2C_BUFFER	EQU	60H		; i2c output buffer
I2C_DATA	EQU	61H		; 2 bytes only plus csum
I2C_CSUM	EQU	62H


WK0		EQU	70H		; wk regs
WK1		EQU	71H
WK2		EQU	72H
WK3		EQU	73H

DEBUG0		EQU	78H		; debug
DEBUG1		EQU	79H
DEBUG2		EQU	7AH
DEBUG3		EQU	7BH


FLAG0		EQU	7CH		; general purpose flag registers
FLAG1		EQU	7DH
FLAG2		EQU	7EH
FLAG3		EQU	7FH



#DEFINE	TMR0_FLG	FLAG0,0		; set during timer int 0
#DEFINE	I2CERR_FLG	FLAG0,1		; set for i2c error
#DEFINE	TIMEOUT_FLG	FLAG0,2		; set on timeout of count register
#DEFINE	I2C_RX_FLG	FLAG0,3		; set in i2c receive mode (write to slave)
#DEFINE	KEY_FLG		FLAG0,4		; set on key change
#DEFINE	ANALOG_FLG	FLAG0,5		; set on analogue change
#DEFINE	I2C_BFR_FLG	FLAG0,6		; set if i2c i/p buffer full
#DEFINE	BOOT_FLG	FLAG0,7		; set to enter/exit boot

#DEFINE	DEBUG_FLG	FLAG3,7		; used for debug

; Bank 1

MBFR		EQU	100H		; used for flash read/write


; Bank 2

IPBFR		EQU	200H		; i2c receive buffer


			; ******
			; Macros
			; ******

DSTR	MACRO STRINGPTR
					; String display
					; tblptru assumed zero
	MOVLW	HIGH STRINGPTR
	MOVWF	TBLPTRH,A
	MOVLW	LOW STRINGPTR
	MOVWF	TBLPTRL,A
	ENDM

; Code
; ====
			; **************
			; Power On Reset
			; **************

	ORG	0000H

PON_RESET:

	GOTO	INIT			; jump to initialisation routine
	NOP
	NOP

	ORG	0008H

INT_HIGH:

	BTFSC	INTCON,TMR0IF,A		; test timer0 interrupt
	CALL	TIMER0_INTERRUPT

	BTFSC	PIR1,SSPIF,A		; test i2c interrupt
	CALL	I2C_INTERRUPT

	RETFIE	FAST			; restore w, bsr, status

	NOP
	NOP
	NOP
	NOP
	NOP				; pack hex file
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP

	

	ORG	040H

ERASE_VECTOR:

INIT:
	
	CALL	CLEAR_RAM		; clear ram 000h thro' 5ffh
	
	CLRF	EECON1,A		; ensure access to flash/eeprom
	BCF	RCON,IPEN,A		; single level priority on interrupts

					; set port i/o
					; port a
	MOVLW	B'11000000'
	MOVWF	ADCON1,A		; all analogue, right justify, adcs2 set
	MOVLW	010H
	MOVWF	LATA,A			; debug led off

	MOVLW	B'00101111'		; all input except debug led
	MOVWF	TRISA,A			
					; port b
	CLRF	LATB,A
	MOVLW	0FFH			; all input
	MOVWF	TRISB,A			
					; port c
	MOVLW	003H
	MOVWF	LATC,A
	MOVLW	0FCH			; all input except portc,0
	MOVWF	TRISC,A			; scl sda configured later

					; port d
	CLRF	LATD,A
	MOVLW	0FFH			; all input
	MOVWF	TRISD,A
					; port e
	CLRF	LATE,A
	MOVLW	007H			; all analogue input
	MOVWF	TRISE,A			; portd gpio
			

INIT0:					
					; init i2c as slave
	MOVLW	FKADD			; set i2c slave adddress
	MOVWF	SSPADD,A

	MOVLW	080H			; slew rate disabled
	MOVWF	SSPSTAT,A

	MOVLW	B'00110110'		; i2c 7bit slave
	MOVWF	SSPCON1,A
	CLRF	SSPCON2,A
	
					; init timer 0

	MOVLW	B'00000000'		; 16 bit, prescale 1:2
	MOVWF	T0CON,A

	MOVLW	03CH			; set for 10 msec interrupt
	MOVWF	TMR0H,A
	MOVLW	0B0H
	MOVWF	TMR0L,A
	
	BSF	T0CON,TMR0ON,A		; start timer 0

					; enable interrupts

	BSF	INTCON,TMR0IE,A		; timer 0
	CLRF	INTCON2,A		; weak pull-up on port b
	CLRF	INTCON3,A

	BSF	PIE1,SSPIE,A		; enable i2c interrupt

	BSF	INTCON,PEIE,A		; enable peripheral interrupts
	BSF	INTCON,GIE,A		; set global interrupt enable


	CLRF	TBLPTRU,A		; assume all maps and messages
					; in range 0000h to ffffh


			; ************
			; Main Routine
			; ************

MAIN:

	MOVLW	008H			; read entire analoge to set new 
	MOVWF	WK3,A			; and old registers to stop
					; analogue stings at power up

MAIN2:
	CALL	SCAN_ANALOGUE
	DECFSZ	WK3,F,A
	GOTO	MAIN2


MAIN0:

	BTFSC	SSPCON1,SSPOV,A		; test overflow of i2c
	GOTO	I2C_ERROR1

	BTFSC	SSPCON1,WCOL,A		; test write collision
	GOTO	I2C_ERROR2

	BTFSS	I2C_BFR_FLG		; test data in i2c rec buffer
	GOTO	MAIN01

	BCF	I2C_BFR_FLG
	CALL	HANDLE_I2C
	BTFSS	BOOT_FLG,A
	GOTO	MAIN01

	BCF	BOOT_FLG,A

	CALL	COPY_NORM_VECTOR
	CLRWDT
	CALL	WRITE_BOOT_VECTOR	; write boot vector at start of flash

	BCF	RCON,IPEN,A		; for icd - ensure no unused interrupts
	BCF	INTCON,TMR0IE,A
	BCF	PIE1,SSPIE,A		; i2c interrupt

	RESET


MAIN01:
	BTFSS	TMR0_FLG,A
	GOTO	MAIN0

	BCF	TMR0_FLG,A		; 10 msec interval
	CLRWDT

	CALL	SCAN_KEYS		; scan keyboard - load to switch1 & swicth2
	BTFSS	KEY_FLG,A
	GOTO	MAIN1

	BCF	KEY_FLG,A
	
	CALL	TRANSPOSE_KEY		; re-assign new value

	BTFSC	KEY_ID,3,A		; test key programme keys
	GOTO	MAIN04
;	BTFSC	KEY_ID,2,A		; rev 1_1 all other keys report both directions
;	GOTO	MAIN04
;	BTFSS	KEY_ID,1,A		; test + / -
;	GOTO	MAIN04

					; fall thro to report both
					; down and up for + / -

MAIN03:
	MOVFF	KEY_ID,I2C_BUFFER
	MOVFF	KEY_DIR,I2C_BUFFER+1

	MOVF	KEY_ID,W,A
	XORWF	KEY_DIR,W,A
	MOVWF	I2C_BUFFER+2,A		; csum

	BCF	INT_OUT,A
	BCF	LED_DEBUG,A
	GOTO	MAIN0

MAIN04:					; here if non repeat key
					; no need to send of code
	MOVF	KEY_DIR,W,A
	BNZ	MAIN03			; key down
					; else fall thro to test analogue
	
MAIN1:

	BTFSS	INT_OUT,A
	GOTO	MAIN0			; no anaolgue scan until
					; comms established with control processor
	CALL	SCAN_ANALOGUE
	BTFSS	ANALOG_FLG,A
	GOTO	MAIN0

	BCF	ANALOG_FLG,A

					; ignore channels 2 and 6
					; unused i/p and breath control
	MOVF	WK2,W,A
	XORLW	002H
	BTFSC	STATUS,Z,A
	GOTO	MAIN0
	
	MOVF	WK2,W,A
	XORLW	006H
	BTFSC	STATUS,Z,A
	GOTO	MAIN0
		
	MOVF	WK2,W,A			; get channel if
	IORLW	010H			; say analogue
	MOVWF	I2C_BUFFER,A
	
	BCF	STATUS,C,A		; shift data right 2 bits
	RRCF	WK1,F,A
	RRCF	WK0,F,A
	RRCF	WK1,F,A
	RRCF	WK0,W,A			; last shift to wreg

	MOVWF	I2C_BUFFER+1,A

	XORWF	I2C_BUFFER,W,A
	MOVWF	I2C_BUFFER+2,A		; csum

	BCF	INT_OUT,A
	BCF	LED_DEBUG
	GOTO	MAIN0


			; *****************
			; Keyboard Routines
			; *****************

SCAN_KEYS:				; called after timer0 interrupt

	CLRF	KEY_ID,A		; clear key id

	CALL	GET_NEW1

	MOVF	NEW1,W,A		; test for change
	XORWF	OLD1,W,A
	MOVWF	XOR1,A

	BZ	SCAN_KEY1		; no change

	MOVFF	NEW1,WK0		; 
	CALL	KEY_DELAY		; debounce
	CALL	GET_NEW1
	
	MOVF	NEW1,W,A
	XORWF	WK0,W,A
	
	BNZ	SCAN_KEY1		; bouncing

	MOVFF	NEW1,WK0		; copy to wk regs
	MOVFF	OLD1,WK1		; -> use common report routine
	MOVFF	XOR1,WK2
	CALL	REPORT_KEY

	MOVFF	NEW1,OLD1
	RETURN				; report only single key

SCAN_KEY1:

	MOVLW	008H			; start at if of 8 for byte 2
	MOVWF	KEY_ID,A

	CALL	GET_NEW2

	MOVF	NEW2,W,A
	XORWF	OLD2,W,A
	MOVWF	XOR2,A

	BZ	SCAN_KEY2

	MOVFF	NEW2,WK0		; 
	CALL	KEY_DELAY		; debounce
	CALL	GET_NEW2
	
	MOVF	NEW2,W,A
	XORWF	WK0,W,A
	
	BNZ	SCAN_KEY2		; bouncing

	MOVFF	NEW2,WK0
	MOVFF	OLD2,WK1
	MOVFF	XOR2,WK2
	CALL	REPORT_KEY

	MOVFF	NEW2,OLD2

SCAN_KEY2:
	RETURN


REPORT_KEY:				; wk0 - new, wk1 - old, wk2 - xor
					; new & xor -> key down change
					; old & xor -> key released
					; enter key_id -> lsb (0 or 8)
	

	MOVLW	0FFH			; say key down
	MOVWF	KEY_DIR,A

	MOVF	WK0,W,A			; new
	ANDWF	WK2,W,A			; xor of new and old
	MOVWF	WK3,A
	BZ	REPORT_KEY1		; no new key down

	GOTO	REPORT_KEY2		; go report key

REPORT_KEY1:

	CLRF	KEY_DIR,A		; say key released

	MOVF	WK1,W,A			; old
	ANDWF	WK2,W,A			; xor of new and old
	MOVWF	WK3,A
	BNZ	REPORT_KEY2		; no new key release
	RETURN				; should no occur
	
REPORT_KEY2:				; change in wk3

	MOVLW	008H
	MOVWF	BITPTR,A

REPORT_KEY2A:

	RRCF	WK3,F,A
	BC	REPORT_KEY2B

	INCF	KEY_ID,F,A
	DECFSZ	BITPTR,F,A
	GOTO	REPORT_KEY2A
	RETURN

REPORT_KEY2B:

	BSF	KEY_FLG,A		; say key change
	RETURN


GET_NEW1:
	COMF	PORTD,W,A		; 1 = key down
	MOVWF	NEW1,A
	RETURN

GET_NEW2:
	COMF	PORTB,W,A
	ANDLW	03FH			; mask bits 0-5
	MOVWF	NEW2,A

	COMF	PORTC,W,A
	ANDLW	0C0H			; mask bits 6-7
	IORWF	NEW2,F,A
	RETURN

KEY_DELAY:				; wair for next timer0 interrupt - 10 msec

	BTFSS	TMR0_FLG,A
	GOTO	KEY_DELAY
	BCF	TMR0_FLG,A
	RETURN


TRANSPOSE_KEY:				; enter key_id from scan

	MOVLW	LOW M_TRANSPOSE
	MOVWF	TBLPTRL,A
	MOVLW	HIGH M_TRANSPOSE
	MOVWF	TBLPTRH,A

	MOVF	KEY_ID,W,A
	ADDWF	TBLPTRL,F,A
	MOVLW	000H
	ADDWFC	TBLPTRH,F,A

	TBLRD*

	MOVFF	TABLAT,KEY_ID
	RETURN


			; ******************
			; I2C Error Routines
			; ******************


I2C_ERROR1:				; overlow on receive

	MOVF	SSPBUF,W,A		; reset BF in sspstat
	BCF	SSPCON1,SSPOV,A
	GOTO	INIT0

I2C_ERROR2:				; collision detect on transmit

	BCF	SSPCON1,WCOL,A
	GOTO	INIT0




			; *****************
			; Timer 0 Interrupt
			; *****************

			; every 10 msec

TIMER0_INTERRUPT:

	BCF	INTCON,TMR0IF,A		; reset interrupt flag

	MOVLW	03CH			; set for 1 msec interrupt
	MOVWF	TMR0H,A
	MOVLW	0B0H
	MOVWF	TMR0L,A

	BSF	TMR0_FLG,A		; say timer 0 interrupt occurred

	MOVF	COUNT,W,A		; test count register for timeout
	BTFSC	STATUS,Z,A
	RETURN

	DCFSNZ	COUNT,F,A
	BSF	TIMEOUT_FLG,A

	RETURN	


			; ***********************
			; Analogue Input Routines
			; ***********************


SCAN_ANALOGUE:				; ADCON1 already configured in init

	CLRF	CHANPTR,A		; reset channel pointer

	BCF	DEBUG_OUT,A
SCAN_A1:

	MOVFF	CHANPTR,WK0
	RLCF	WK0,F,A			; position address bits 3-5
	RLCF	WK0,F,A
	RLCF	WK0,W,A			; final result in wreg
	
	ANDLW	B'00111000'		; mask channel address
	IORLW	B'10000001'		; fosc/64, atod on
	MOVWF	ADCON0,A

	CALL	AQU_DELAY		; aquisition delay

	BSF	ADCON0,GO_DONE,A

SCAN_A2:
	BTFSC	ADCON0,GO_DONE,A	; wait on conversion
	GOTO	SCAN_A2

	LFSR	FSR0,0000H		; point to new analogue buffer
	
	RLNCF	CHANPTR,W,A		; * 2
	MOVWF	FSR0L,A

	MOVF	ADRESL,W,A		; store in analogue buffer
	MOVWF	POSTINC0,A
	MOVF	ADRESH,W,A
	MOVWF	POSTINC0,A

	INCF	CHANPTR,F,A		; -> next channel
	BTFSS	CHANPTR,3,A
	GOTO	SCAN_A1

	BSF	DEBUG_OUT,A

					; noise filter 
					; must change by count of 4 min to register
					; channel out pointer coutptr set on change
					; start next test after previous change to 
					; avoid a noisy channel blocking other changes


	LFSR	FSR0,000H		; new
	LFSR	FSR1,010H		; old

	RLNCF	COUTPTR,W,A		; * 2
	ADDWF	FSR0L,F,A
	ADDWF	FSR1L,F,A

	MOVLW	008H			
	MOVWF	BITPTR,A		; no of channels

	BCF	ANALOG_FLG,A

SCAN_A3:

	MOVF	POSTINC1,W,A		
	SUBWF	POSTINC0,W,A		; new - old 
	MOVWF	WK0,A

	MOVF	POSTDEC1,W,A		; high bytes
	SUBWFB	POSTDEC0,W,A
	MOVWF	WK1,A			; fsrs -> low bytes of current data

	BTFSC	WK1,7,A			; test polarity
	CALL	COMP_WK_REGS

	MOVF	WK1,W,A			; test high byte
	BNZ	SCAN_A6			; go report change

	MOVLW	004H
	CPFSLT	WK0,A			; skip if change < 4
	GOTO	SCAN_A6
	
	INCF	COUTPTR,F,A
	MOVLW	007H
	ANDWF	COUTPTR,F,A		; keep in range

	INCF	FSR0L,F,A		; point to next data
	INCF	FSR0L,F,A
	INCF	FSR1L,F,A
	INCF	FSR1L,F,A

	MOVLW	00FH			; ensure fsrs stay in range
	ANDWF	FSR0L,F,A
	ANDWF	FSR1L,F,A
	BSF	FSR1L,4,A			; old range 10-1fh

	DECFSZ	BITPTR,F,A
	GOTO	SCAN_A3
	RETURN


SCAN_A6:				

	MOVF	POSTINC0,W,A		; transfer new to old
	MOVWF	POSTINC1,A		; fsr inc by 2
	MOVWF	WK0,A

	MOVF	POSTINC0,W,A
	MOVWF	POSTINC1,A
	MOVWF	WK1,A

	MOVFF	COUTPTR,WK2		; return wk0,wk1 new analogue
					; wk2 -> channel 0-7
	INCF	COUTPTR,F,A
	MOVLW	007H
	ANDWF	COUTPTR,F,A		; keep in range

	BSF	ANALOG_FLG,A
	RETURN



COMP_WK_REGS:				; 2S complement od wk1,wk0


	COMF	WK0,F,A
	COMF	WK1,F,A
	MOVLW	001H
	ADDWF	WK0,F,A
	MOVLW	000H
	ADDWFC	WK1,F,A
	RETURN


AQU_DELAY:				; aquisitoin delay
					; set at 20 usec
	MOVLW	043H
	MOVWF	ADLY,A

AQU_DELAY_LOOP:

	DECFSZ	ADLY,F,A
	GOTO	AQU_DELAY_LOOP
	RETURN


			; *********************
			; I2C Interrupt routine
			; *********************

			; TX used when transmitting data to bus - READ FROM SLAVE
			; RX used when receiving data from bus - WRITE TO SLAVE
			; FSR2 used exclusively for this interrupt

I2C_INTERRUPT:

	BCF	PIR1,SSPIF,A		; clear ssp interrupt

	BSF	INT_OUT,A		; clear interrupt out
	BSF	LED_DEBUG,A

	BTFSS	SSPSTAT,D_A,A		; test address of data
	GOTO	I2C_ADDRESS_INT		

	BTFSS	I2C_RX_FLG,A
	GOTO	I2C_TX_DATA

I2C_RX_DATA:				; receive data from bus
					; not used
	MOVF	SSPBUF,W,A		; get data to save overflow

	LFSR	FSR2,IPBFR
	MOVFF	RXPTR,FSR2L
	MOVWF	INDF2,A

	INCF	RXPTR,F,A
	
	MOVF	RXPTR,W,A
	XORLW	004H
	BTFSC	STATUS,Z
	BSF	I2C_BFR_FLG
	RETURN

				
I2C_TX_DATA:

	MOVLW	I2C_BUFFER+3		; test data already sent rev 1_2
	XORWF	FSR2L,W,A
	BTFSC	STATUS,Z,A
	RETURN

	MOVF	POSTINC2,W,A		; get data
	MOVWF	SSPBUF,A

	BSF	SSPCON1,CKP,A		; enable serial clock
	RETURN

I2C_ADDRESS_INT:

	BTFSS	SSPSTAT,R_W,A		; test read/write
					; set=read -> tx to bus
	GOTO	I2C_RX_ADDRESS

I2C_TX_ADDRESS:				; transmit data to bus

	BCF	I2C_RX_FLG,A		; say tx data
	MOVF	SSPBUF,W,A		; clear buffer

	LFSR	FSR2,I2C_BUFFER		; point to start of data
	GOTO	I2C_TX_DATA


I2C_RX_ADDRESS:				; not used

	BSF	I2C_RX_FLG,A		; say receiving data
	MOVF	SSPBUF,W,A		; clear address
	RETURN


			; *******************
			; Handle i2c routines
			; *******************


HANDLE_I2C:

	LFSR	FSR0,IPBFR
	CLRF	WK0,A		; reset csum

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	XORLW	001H		; test header
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_I2C_EXIT

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	XORLW	004H		; confirm packet length
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_I2C_EXIT

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	MOVWF	WK1,A		; save id

	MOVF	POSTINC0,W,A
	XORWF	WK0,W,A
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_I2C_EXIT

				; packet ok - test pkt id
	MOVF	WK1,W,A
	ANDLW	00FH
	XORLW	00BH
	BTFSC	STATUS,Z,A
	BSF	BOOT_FLG,A

HANDLE_I2C_EXIT:

	CLRF	RXPTR,A
	RETURN


			; **************
			; Delay Routines
			; **************

			; nb beware watchdog timeout

DELAY:
	MOVWF	COUNT,A			; enter w regs holds delay/4 msec
	BCF	TIMEOUT_FLG,A

DELAY1:
	BTFSS	TIMEOUT_FLG,A
	GOTO	DELAY1
	RETURN


					; pack to 64 byte boundary
	
	NOP
	NOP
	NOP
	
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	




			; *****************
			; Maps and Messages
			; *****************

	ORG	2000H

			; nb. keep to even number of characters

M_REVISION:	dw "  Revision 3.1,00h

M_TRANSPOSE:	DB 005H,004H,000H,001H,003H,009H,008H,007H
		DB 00AH,00BH,00CH,00DH,00EH,00FH,002H,006H

				; pack to 64 byte boundary

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	

			; **************
			; Bootstrap Code
			; **************

	ORG	7000H

ERROR_TRAP:	
	GOTO	ERROR_TRAP	; trap any run away code

BOOT_INTERRUPT:

	BTFSC	INTCON,TMR0IF,A		; test timer0 interrupt
	BCF	INTCON,TMR0IF,A

	BTFSC	PIR1,SSPIF,A		; test i2c interrupt
	CALL	BOOT_I2C_INTERRUPT

	RETFIE	FAST			; restore w, bsr, status

BOOT_INIT:

	
	CALL	CLEAR_RAM		; clear ram 000h thro' 5ffh
	
	CLRF	EECON1,A		; ensure access to flash/eeprom
	BCF	RCON,IPEN,A		; single level priority on interrupts

					; set port i/o
					; port a
	MOVLW	B'11000000'
	MOVWF	ADCON1,A		; all analogue, right justify, adcs2 set
	MOVLW	010H
	MOVWF	LATA,A			; debug led off

	MOVLW	B'00101111'		; all input except debug led
	MOVWF	TRISA,A			
					; port b
	CLRF	LATB,A
	MOVLW	0FFH			; all input
	MOVWF	TRISB,A			
					; port c
	MOVLW	003H
	MOVWF	LATC,A
	MOVLW	0FCH			; all input except portc,0
	MOVWF	TRISC,A			; scl sda configured later

					; port d
	CLRF	LATD,A
	MOVLW	0FFH			; all input
	MOVWF	TRISD,A
					; port e
	CLRF	LATE,A
	MOVLW	007H			; all analogue input
	MOVWF	TRISE,A			; portd gpio

BOOT_INIT0:					
					; init i2c as slave
	MOVLW	FKADD			; set i2c slave adddress
	MOVWF	SSPADD,A

	MOVLW	080H			; slew rate disabled
	MOVWF	SSPSTAT,A

	MOVLW	B'00110110'		; i2c 7bit slave
	MOVWF	SSPCON1,A
	CLRF	SSPCON2,A
	
	
	BCF	T0CON,TMR0ON,A		; disable timer 0

					; enable interrupts

	CLRF	INTCON,A
	CLRF	INTCON2,A		; weak pull-up on port b
	CLRF	INTCON3,A

	BSF	PIE1,SSPIE,A		; enable i2c interrupt

	BSF	INTCON,PEIE,A		; enable peripheral interrupts
	BSF	INTCON,GIE,A		; set global interrupt enable


	CLRF	TBLPTRU,A		; assume all maps and messages
					; in range 0000h to ffffh

			; ************
			; Main Routine
			; ************

BOOT_MAIN:


BOOT_MAIN0:

	BTFSC	SSPCON1,SSPOV,A		; test overflow of i2c
	GOTO	BOOT_I2C_ERROR1

	BTFSC	SSPCON1,WCOL,A		; test write collision
	GOTO	BOOT_I2C_ERROR2

	CLRWDT
	BTFSC	I2C_BFR_FLG,A
	CALL	HANDLE_BOOT

	GOTO	BOOT_MAIN0


			; ******************
			; I2C Error Routines
			; ******************


BOOT_I2C_ERROR1:				; overlow on receive

	MOVF	SSPBUF,W,A		; reset BF in sspstat
	BCF	SSPCON1,SSPOV,A
	GOTO	BOOT_INIT0

BOOT_I2C_ERROR2:				; collision detect on transmit

	BCF	SSPCON1,WCOL,A
	GOTO	BOOT_INIT0





			; ***********
			; Handle Boot 
			; ***********

HANDLE_BOOT:

	LFSR	FSR0,IPBFR

	MOVF	POSTINC0,W,A
	XORLW	001H		; test header
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_BOOT_EXIT

	DECF	POSTINC0,W,A
	MOVWF	WK2,A		; save packet length -1

	LFSR	FSR0,IPBFR
	CLRF	WK0,A

HANDLE_BOOT1:

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	DECFSZ	WK2,F,A
	GOTO	HANDLE_BOOT1

	MOVLW	07FH
	ANDWF	WK0,F,A		; 7 data only

	MOVF	POSTINC0,W,A
	XORWF	WK0,W,A
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_BOOT_EXIT

				; valid packet

	MOVFF	IPBFR+2,WK0	; get id to wk0

	MOVLW	00FH
	ANDWF	WK0,F,A		; mask processor address
	MOVF	WK0,W,A
	XORLW	00AH
	BNZ	HANDLE_BOOT2

	CALL	RETURN_NORM
	RESET


HANDLE_BOOT2:


	MOVF	WK0,W,A
	BNZ	HANDLE_BOOT3	; test hex file data

	CALL	VERIFY_HEX
	BTFSS	STATUS,Z,A
	GOTO	BOOT_NAK

	BCF	LED_DEBUG,A
	CALL	PROGRAM_UPDATE
	BSF	LED_DEBUG,A

	GOTO	BOOT_ACK

HANDLE_BOOT3:

	MOVF	WK0,W,A
	XORLW	009H
	BNZ	HANDLE_BOOT_EXIT

	CALL	SEND_CRC_PKT

HANDLE_BOOT_EXIT:

	BCF	I2C_BFR_FLG,A
	CLRF	RXPTR,A
	RETURN

BOOT_ACK:
	MOVLW	03CH
	CALL	SEND_PKT
	GOTO	HANDLE_BOOT_EXIT

BOOT_NAK:

	MOVLW	03DH
	CALL	SEND_PKT
	GOTO	HANDLE_BOOT_EXIT

			; ****************
			; Programme Update
			; ****************

PROGRAM_UPDATE:		; parity check and csum on packet ok
		

	LFSR	FSR0,IPBFR+6
	CALL	GET_HEX_BYTE
	MOVWF	TBLPTRH,A

	CALL	GET_HEX_BYTE
	MOVWF	TBLPTRL,A

PU1:

	LFSR	FSR0,IPBFR+0CH

	LFSR	FSR2,MBFR	; output buffer

	MOVF	TBLPTRL,W,A
	ANDLW	030H
	MOVWF	FSR2L,A

	MOVLW	008H		; 8 words in total
	MOVWF	WK2,A

PU2:

	CALL	GET_HEX_BYTE
	MOVWF	LSB,A
	CALL	GET_HEX_BYTE
	MOVWF	MSB,A

	MOVFF	LSB,POSTINC2
	MOVFF	MSB,POSTINC2

	DECFSZ	WK2,F,A
	GOTO	PU2

	MOVF	TBLPTRL,W,A
	ANDLW	030H		; test 4th line
	XORLW	030H
	BTFSS	STATUS,Z,A
	RETURN			

	BCF	LED_DEBUG,A

	MOVLW	0C0H
	ANDWF	TBLPTRL,F,A	; tblptr -> start of 64 byte block

	MOVF	TBLPTRH,W,A	; test power on vector
	IORWF	TBLPTRL,W,A
	BNZ	PU3

	CALL	INIT_CRC		; init crc if first block

	DSTR	NORMAL_VECTOR

PU3:
	MOVFF	TBLPTRL,TPTRL		; save tblptr
	MOVFF	TBLPTRH,TPTRH

	CALL	WRITE_FLASH

	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH

	MOVLW	040H			; bytes to verify
	MOVWF	WK2,A
	LFSR	FSR0,MBFR
	
PU4:
	TBLRD*+

	MOVF	TABLAT,W,A
	XORWF	POSTINC0,W,A
	BTFSS	STATUS,Z,A
	GOTO	PU5			; fail - try write once more			
	DECFSZ	WK2,F,A
	GOTO	PU4
					; update crc after verify
	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH

	MOVLW	040H
	MOVWF	WK2,A

	
PU6:

	TBLRD*+

	CALL	CRC

	DECFSZ	WK2,F,A
	GOTO	PU6

	BSF	LED_DEBUG,A

	RETURN

PU5:
	CLRWDT
	
	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH
	GOTO	PU3			; go try again


			; ***********
			; CRC Routine
			; ***********
INIT_CRC:

	MOVLW	0FFH
	MOVWF	CRCL,A
	MOVWF	CRCH,A
	RETURN

CRC:			; enter - data in tablat
			; WK0,WK1,WK3 destroyed

	MOVFF	TABLAT,WK1

	MOVLW	008H
	MOVWF	WK3,A	; bit count


CRC1:

	MOVF	CRCL,W,A	; xor lsbits
	XORWF	WK1,W,A
	MOVWF	WK0,A		; wk0 stores result of xor

	BCF	STATUS,C,A	; clear c
	RRCF	CRCH,F,A
	RRCF	CRCL,F,A

	BTFSS	WK0,0,A		; test reult of xor
	GOTO	CRC2

	MOVLW	084H
	XORWF	CRCH,F,A
	MOVLW	008H
	XORWF	CRCL,F,A

CRC2:
	RRNCF	WK1,F,A
	DECFSZ	WK3,F,A
	GOTO	CRC1
	RETURN



			; ***************
			; Verify Hex File
			; ***************

VERIFY_HEX:

	LFSR	FSR0,IPBFR+3		; -> :
	MOVF	POSTINC0,W,A
	XORLW	03AH
	BTFSS	STATUS,Z,A
	RETURN				; return non-zero if fail

	CLRF	WK0,A	; reset csum
	
	MOVLW	015H			; line lenght in bytes - always full line
	MOVWF	WK2,A			; including csum

VERIFY_HEX1:	
	
	CALL	GET_HEX_BYTE
	ADDWF	WK0,F,A

	DECFSZ	WK2,F,A
	GOTO	VERIFY_HEX1
	MOVF	WK0,W,A			; return zero if ok

	RETURN


GET_HEX_BYTE:				; get hex byte @ fsr0

	CALL	GET_HEX_NIBBLE
	MOVWF	WK1,A
	SWAPF	WK1,F,A			; xfer to top nibble
	CALL	GET_HEX_NIBBLE
	ADDWF	WK1,W,A
	RETURN


GET_HEX_NIBBLE:				; get single nibble from hex file

	MOVLW	000H
	BTFSC	INDF0,6,A			; test a-f
	MOVLW	009H
	ADDWF	POSTINC0,W,A
	ANDLW	00FH
	RETURN


			; ******************
			; Boot i2c Interrupt
			; ******************

BOOT_I2C_INTERRUPT:


			; TX used when transmitting data to bus - READ FROM SLAVE
			; RX used when receiving data from bus - WRITE TO SLAVE
			; FSR2 used exclusively for this interrupt


	BCF	PIR1,SSPIF,A		; clear ssp interrupt

	BSF	INT_OUT,A		; clear interrupt out
	BSF	LED_DEBUG,A

	BTFSS	SSPSTAT,D_A,A		; test address of data
	GOTO	BOOT_I2C_ADDRESS_INT		

	BTFSS	I2C_RX_FLG,A
	GOTO	BOOT_I2C_TX_DATA

BOOT_I2C_RX_DATA:			; receive data from bus
					; not used
	MOVF	SSPBUF,W,A		; get data to save overflow

	LFSR	FSR2,IPBFR
	MOVFF	RXPTR,FSR2L
	MOVWF	INDF2,A

	MOVF	RXPTR,W,A		; test -> pkt length
	XORLW	001H
	BTFSC	STATUS,Z,A
	MOVFF	INDF2,RXLEN

	INCF	RXPTR,F,A
	
	MOVLW	003H
	CPFSGT	RXPTR,A
	RETURN				; minimum packet not yet recd

	MOVF	RXPTR,W,A
	XORWF	RXLEN,W,A
	BTFSC	STATUS,Z,A
	BSF	I2C_BFR_FLG
	RETURN
				
BOOT_I2C_TX_DATA:

	MOVF	I2C_BUFFER+1,W,A		; get pkt length
	ADDLW	I2C_BUFFER
	XORWF	FSR2L,W,A		; test all data sent
	BTFSC	STATUS,Z,A		; don't load for ack interrupt
	RETURN

	MOVF	POSTINC2,W,A		; get data
	MOVWF	SSPBUF,A

	BSF	SSPCON1,CKP,A		; enable serial clock
	RETURN

BOOT_I2C_ADDRESS_INT:

	BTFSS	SSPSTAT,R_W,A		; test read/write
					; set=read -> tx to bus
	GOTO	BOOT_I2C_RX_ADDRESS

BOOT_I2C_TX_ADDRESS:				; transmit data to bus

	BCF	I2C_RX_FLG,A		; say tx data
	MOVF	SSPBUF,W,A		; clear buffer

	LFSR	FSR2,I2C_BUFFER		; point to start of data
	GOTO	BOOT_I2C_TX_DATA


BOOT_I2C_RX_ADDRESS:				; not used

	BSF	I2C_RX_FLG,A		; say receiving data
	MOVF	SSPBUF,W,A		; clear address
	RETURN


			; *************************
			; Boot send packet routines
			; *************************


SEND_PKT:

	MOVWF	I2C_BUFFER+2,A		; enter w = pkt id
	MOVWF	TXCSUM,A
	MOVLW	001H			; header
	MOVWF	I2C_BUFFER,A
	ADDWF	TXCSUM,F,A

	MOVLW	004H			; length
	MOVWF	I2C_BUFFER+1,A
	ADDWF	TXCSUM,F,A

	MOVFF	TXCSUM,I2C_BUFFER+3	; put csum
	BCF	INT_OUT,A
	RETURN

SEND_CRC_PKT:

	LFSR	FSR0,I2C_BUFFER

	MOVLW	001H			; header
	MOVWF	POSTINC0,A
	MOVWF	TXCSUM,A

	MOVLW	008H			; length
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A

	MOVLW	034H
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A


	MOVFF	CRCH,WK0		; place crc in ascii hex format
	CALL	PUT_HEX
	MOVFF	CRCL,WK0
	CALL	PUT_HEX

	MOVFF	TXCSUM,POSTINC0
	BCF	INT_OUT,A
	RETURN

PUT_HEX:				; enter data in wk0

	SWAPF	WK0,W,A
	ANDLW	00FH
	MOVWF	WK1,A

	CALL	PUT_H1

	MOVF	WK0,W,A
	ANDLW	00FH
	MOVWF	WK1,A			; fall thro for low nibble

PUT_H1:

	MOVLW	00AH
	CPFSLT	WK1,A
	GOTO	PUT_H2

	MOVF	WK1,W,A			; 0-9
	IORLW	030H
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A
	RETURN
PUT_H2:				; A-F

	MOVF	WK1,W,A
	ADDLW	'A'-00AH
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A
	RETURN



			; **************
			; Flash Routines
			; **************
			;  all flash routines based on 64 byte transfer
			; mbfr used for ram
			; fsr0 as indirect pointer

READ_FLASH:		; read 64 bytes flash @ tblptr to mbfr
			; use dstr to set tblptr

	BCF	EECON1,CFGS,A	; data/ flash
	BSF	EECON1,EEPGD,A	; flash
	LFSR	FSR0,MBFR
	MOVLW	040H		; 64 bytes
	MOVWF	WK2,A

READ_FLASH1:

	TBLRD*+	
	MOVFF	TABLAT,POSTINC0	; xfer data
	DECFSZ	WK2,F,A
	GOTO	READ_FLASH1
	RETURN


ERASE_FLASH:

	BCF	INTCON,GIE,A	; disable all interrupts during write

	BCF	EECON1,CFGS,A	; flash/eeprom
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A	; enable write
	BSF	EECON1,FREE,A	; line erase (64 bytes)

	MOVLW	055H		; timed access
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A

	BSF	EECON1,WR,A
	NOP
	BSF	INTCON,GIE,A
	RETURN



WRITE_FLASH:

	BCF	INTCON,GIE,A	; disable all interrupts during write

	BCF	EECON1,CFGS,A	; flash/eeprom
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A	; enable write
	BSF	EECON1,FREE,A	; line erase (64 bytes)

	MOVLW	055H		; timed access
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A

	BSF	EECON1,WR,A
	NOP
		
				; write data in mbfr @ tblptr
				; write 8 blocks of 8 bytes @ tblptr


	LFSR	FSR0,MBFR	; point to ram
	MOVLW	008H
	MOVWF	WK3,A		; 8 loops of 8 bytes each

WRITE_FLASH1:

	MOVLW	008H		; byte count
	MOVWF	WK2,A

WRITE_FLASH2:

	MOVF	POSTINC0,W,A	; get data from buffer	
	MOVWF	TABLAT,A	; 8 bytes to temp registers
	TBLWT*+

	DECFSZ	WK2,F,A
	GOTO	WRITE_FLASH2

	TBLRD*-			; dec tblptr to ensure written 8 byte earlier
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A
	
	MOVLW	055H
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A
	BSF	EECON1,WR,A
	NOP
	
	TBLRD*+			; restore tablptr
	DECFSZ	WK3,F,A
	GOTO	WRITE_FLASH1
	BCF	EECON1,WREN,A	; disable further writes
	BSF	INTCON,GIE,A
	RETURN




			; *********
			; Clear Ram
			; *********

CLEAR_RAM:

	LFSR	FSR0,0000H		; clear ram 000h thro 5ffh

CLR_LOOP:

	CLRF	POSTINC0,A
	
	MOVF	FSR0H,W,A		; high pointer to w reg
	XORLW	006H
	BTFSS	STATUS,Z,A
	GOTO	CLR_LOOP
	RETURN	



			; ********************
			; Read / Write Vectors
			; ********************

COPY_NORM_VECTOR:	; copy normal vetor to normal_vector

	DSTR	PON_RESET
	CALL	READ_FLASH

	DSTR	NORMAL_VECTOR
	CALL	WRITE_FLASH
	RETURN
	
WRITE_BOOT_VECTOR:	; replace normal vectors withboot vectors

	DSTR	BOOT_VECTOR
	CALL	READ_FLASH

	DSTR	PON_RESET
	CALL	WRITE_FLASH

	RETURN

RETURN_NORM:

	CLRF	INTCON,A

	DSTR	NORMAL_VECTOR
	CALL	READ_FLASH	; get normal vector

	DSTR	PON_RESET
	CALL	WRITE_FLASH
	RETURN



			; *************
			; Store Vectors
			; *************


	ORG	7780H

BOOT_VECTOR:

	GOTO	BOOT_INIT
	
	ORG	7788H
	GOTO	BOOT_INTERRUPT




	ORG	7800H

NORMAL_VECTOR:				; normal vector copied here
					; on entering pgm mode


	END
