

	LIST P=PIC18F452

			; ************************
			; MIDI Keyboard Controller
			; Revision 3_3
			; September 2006
			; ************************



#INCLUDE <C:\Program Files\MICROCHIP\MPASM Suite\p18f452.inc>

; conditional compilation
; =======================

; set i2c address

UNIT	EQU	000H
DEBUG	EQU	080H	; debug mode
ICD	EQU	020H


; development
; ===========

; MPLAB IDE 7.00
; MPLAB ICD2
; MPLAB MASM


; firmware
; ========

; packet format
; byte 0	id 		00h to 77h for key
;				MIDI note number
; byte 1	velocity	00h - off
;				other - key down velocity (00h - 64h)
; byte 2	csum		byte0 xor byte1

; all code protected

; single level interrupt

; revision 1_1
; ============

; ignore final i2c interrupt generated after csum transmitted
;	loading extra byte causes collision detect error

; clear intcon2 prior to getting i2c address
;	this ensures weak pull-up applied

; rev 3_0
; =======

; support for serial line programme update
; all code blocks filled to 64 byte boundaries
; to enable crc calculation

; revision 3_1
; ============

; if second key closed then assume 1st key close and send keydown
; and set status to 2
; switch release on release of second switch

; reset count on closure of sw1 if status 0 or 1

; revision 3_2
; ============

; status 3 aand 4 added to key status
; 	-> monitor release of keys
;	stat 3  k1 down k2 up (normal)
;	stat 4	k1 up k2 down (bouncing k1)
; sweep routine added after scan
; if inner key up and stat3 then send key off and goto stat0

; revision 3_3
; ============

; counts reset to zero in key_up routine
; counts also reset after inner switch only closure and open
; counts reset when outer switch only closure


; hardware
; ========

; 10 MHz xtal with PLL enabled (40 MHz effective)
; i2c address 10h + 2*(rb7, rb6)

#DEFINE	INT_OUT		PORTD,7
#DEFINE	LED_OUT		PORTA,4

; configuration
; =============

;Program Configuration Register 1H
		__CONFIG    _CONFIG1H, _OSCS_OFF_1H & _HSPLL_OSC_1H

;Program Configuration Register 2L
		__CONFIG    _CONFIG2L, _BOR_ON_2L & _BORV_27_2L & _PWRT_ON_2L

;Program Configuration Register 3H
		__CONFIG    _CONFIG3H, _CCP2MX_ON_3H

;Program Configuration Register 6L
		__CONFIG    _CONFIG6L, _WRT0_OFF_6L & _WRT1_OFF_6L & _WRT2_OFF_6L & _WRT3_OFF_6L 

;Program Configuration Register 6H
		__CONFIG    _CONFIG6H, _WRTC_OFF_6H & _WRTB_OFF_6H & _WRTD_OFF_6H

;Program Configuration Register 7L
		__CONFIG    _CONFIG7L, _EBTR0_OFF_7L & _EBTR1_OFF_7L & _EBTR2_OFF_7L & _EBTR3_OFF_7L

;Program Configuration Register 7H
		__CONFIG    _CONFIG7H, _EBTRB_OFF_7H


.IF (UNIT & DEBUG)

;Program Configuration Register 2H
		__CONFIG    _CONFIG2H, _WDT_OFF_2H & _WDTPS_128_2H

;Program Configuration Register 5L
		__CONFIG    _CONFIG5L, _CP0_OFF_5L & _CP1_OFF_5L & _CP2_OFF_5L & _CP3_OFF_5L 

;Program Configuration Register 5H
		__CONFIG    _CONFIG5H, _CPB_OFF_5H & _CPD_OFF_5H

.ELSE

;Program Configuration Register 2H
		__CONFIG    _CONFIG2H, _WDT_ON_2H & _WDTPS_128_2H

;Program Configuration Register 5L
		__CONFIG    _CONFIG5L, _CP0_ON_5L & _CP1_ON_5L & _CP2_ON_5L & _CP3_ON_5L 

;Program Configuration Register 5H
		__CONFIG    _CONFIG5H, _CPB_ON_5H & _CPD_OFF_5H

.ENDIF


.IF (UNIT & ICD)

;Program Configuration Register 4L
		__CONFIG    _CONFIG4L, _STVR_ON_4L & _LVP_OFF_4L & _DEBUG_ON_4L

.ELSE

;Program Configuration Register 4L
		__CONFIG    _CONFIG4L, _STVR_ON_4L & _LVP_OFF_4L & _DEBUG_OFF_4L

.ENDIF

; Constants
; =========


; Register Map
; ============

; BANK 0
; ======

SCAN_BFR	EQU	00H		; 40 byte Scan buffer
					; byte pairs new then old

SCAN_PTR	EQU	40H		; pointer to scanned key 0 - 140 decimal
NEW		EQU	41H
OLD		EQU	42H

KEYCODE		EQU	43H		; MIDI key code

COUNT		EQU	48H		; dec on 1 msec basis to zero
					; timeout flag set
BIT_PTR		EQU	49H		; gp bit pointer
BFR_PTR		EQU	4AH		; gp buffer pointer

RXPTR		EQU	50H		; i2c input pointer
RXLEN		EQU	51H		; i2c rx pkt len
TXLEN		EQU	52H		; i2c tx pkt len
TXCSUM		EQU	53H

MSB		EQU	54H
LSB		EQU	55H

CRCH		EQU	56H
CRCL		EQU	57H

TPTRH		EQU	58H
TPTRL		EQU	59H

I2C_ADD		EQU	5AH


I2C_BUFFER	EQU	60H		; i2c output buffer
I2C_DATA	EQU	61H		; 2 bytes only plus csum
I2C_CSUM	EQU	62H

OBFR_IN		EQU	64H		; 256 byte fifo buffer pointers
OBFR_OUT	EQU	65H
OBFR_SIZE	EQU	66H

FSR0_SAVL	EQU	68H		; save fsr0 during key scan
FSR0_SAVH	EQU	69H



WK0		EQU	70H		; wk regs
WK1		EQU	71H
WK2		EQU	72H
WK3		EQU	73H

DLY_REG		EQU	74H


DEBUG0		EQU	78H		; debug
DEBUG1		EQU	79H
DEBUG2		EQU	7AH
DEBUG3		EQU	7BH


FLAG0		EQU	7CH		; general purpose flag registers
FLAG1		EQU	7DH
FLAG2		EQU	7EH
FLAG3		EQU	7FH



#DEFINE	TMR0_FLG	FLAG0,0		; set during timer int 0
#DEFINE	I2CERR_FLG	FLAG0,1		; set for i2c error
#DEFINE	TIMEOUT_FLG	FLAG0,2		; set on timeout of count register
#DEFINE	I2C_RX_FLG	FLAG0,3		; set in i2c receive mode (write to slave)
#DEFINE	KEY_FLG		FLAG0,4		; set on key change
#DEFINE	I2C_BFR_FLG	FLAG0,6		; set if i2c i/p buffer full
#DEFINE	BOOT_FLG	FLAG0,7		; set to enter/exit boot

#DEFINE	DEBUG_FLG	FLAG3,7		; used for debug

; BANK1
; =====


STATUS_BFR	EQU	100H
MBFR		EQU	100H		; used only in programme update

; BANK2
; =====

IPBFR		EQU	200H

; BANK 3
; ======

COUNT_BFR	EQU	300H


; BANK 4
; ======

OUTPUT_BFR:	EQU	400H


			; ******
			; Macros
			; ******

DSTR	MACRO STRINGPTR
					; String display
					; tblptru assumed zero
	MOVLW	HIGH STRINGPTR
	MOVWF	TBLPTRH,A
	MOVLW	LOW STRINGPTR
	MOVWF	TBLPTRL,A
	ENDM


; Code
; ====
			; **************
			; Power On Reset
			; **************

	ORG	0000H

PON_RESET:
	GOTO	INIT			; jump to initialisation routine
	NOP
	NOP

	ORG	0008H

INT_HIGH:

	BTFSC	INTCON,TMR0IF,A		; test timer0 interrupt
	CALL	TIMER0_INTERRUPT

	BTFSC	PIR1,SSPIF,A		; test i2c interrupt
	CALL	I2C_INTERRUPT

	RETFIE	FAST			; restore w, bsr, status

	NOP
	NOP
	NOP
	NOP
	NOP				; pack hex file
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP


	ORG	040H

INIT:
	
	CALL	CLEAR_RAM		; clear ram 000h thro' 5ffh
	
	BCF	RCON,IPEN,A		; single level priority on interrupts

					; set port i/o
					; port a

	MOVLW	B'00000110'		; all digital
	MOVWF	ADCON1,A		
	MOVLW	010H			; ra4 high
	MOVWF	LATA,A
	CLRF	TRISA,A			; all output

					; port b
	CLRF	LATB,A
	MOVLW	0C0H			; rb6,rb7 input - reset output
	MOVWF	TRISB,A			
					; port c

	CLRF	LATC,A
	MOVLW	B'00011000'		; all input except i2c bus
	MOVWF	TRISC,A			; scl sda configured later

					; port d
	MOVLW	80H			; int out set high
	MOVWF	LATD,A
	MOVLW	07FH			; all input except rd7
	MOVWF	TRISD,A
					; port e
	CLRF	LATE,A
	CLRF	TRISE,A			; portd gpio / all output

	CLRF	INTCON2			; ensure weak pull-up on port b
	CALL	DELAY_50USEC
	CALL	DELAY_50USEC

INIT0:					
					; init i2c as slave

	SWAPF	PORTB,W,A		; rb7, rb6 to bits 3,2
	MOVWF	WK0,A
	RRNCF	WK0,W,A
	XORLW	0FFH			; jumper = high
	ANDLW	B'00000110'		; i2c address 10h , 12h or 14h
	IORLW	B'00010000'

.IF (UNIT & DEBUG)		
	MOVLW	010H			; use LEFT block for debug
.ENDIF

	MOVWF	SSPADD,A

	MOVLW	080H			; slew rate disabled
	MOVWF	SSPSTAT,A

	MOVLW	B'00110110'		; i2c 7bit slave
	MOVWF	SSPCON1,A
	CLRF	SSPCON2,A
	
					; init timer 0

	MOVLW	B'00001000'		; 16 bit, no prescale
	MOVWF	T0CON,A

	MOVLW	0D8H			; set for 1 msec interrupt
	MOVWF	TMR0H,A
	MOVLW	0F0H
	MOVWF	TMR0L,A
	
	BSF	T0CON,TMR0ON,A		; start timer 0

					; enable interrupts

	BSF	INTCON,TMR0IE,A		; timer 0
	CLRF	INTCON2,A		; weak pull-up on port b
	CLRF	INTCON3,A

	BSF	PIE1,SSPIE,A		; enable i2c interrupt

	BSF	INTCON,PEIE,A		; enable peripheral interrupts
	BSF	INTCON,GIE,A		; set global interrupt enable


	CLRF	TBLPTRU,A		; assume all maps and messages
					; in range 0000h to ffffh


			; ************
			; Main Routine
			; ************

MAIN:

	BSF	INT_OUT,A		; ensure no interrupt request

MAIN0:

	BTFSC	SSPCON1,SSPOV,A		; test overflow of i2c
	GOTO	I2C_ERROR1

	BTFSC	SSPCON1,WCOL,A		; test write collision
	GOTO	I2C_ERROR2

	BTFSS	I2C_BFR_FLG,A		; test data in i2c rec buffer
	GOTO	MAIN01

	BCF	I2C_BFR_FLG,A
	CALL	HANDLE_I2C
	BTFSS	BOOT_FLG,A
	GOTO	MAIN01

	BCF	BOOT_FLG,A

	CALL	COPY_NORM_VECTOR
	CLRWDT
	CALL	WRITE_BOOT_VECTOR	; write boot vector at start of flash

	BCF	RCON,IPEN,A		; for icd - ensure no unused interrupts
	BCF	INTCON,TMR0IE,A
	BCF	PIE1,SSPIE,A		; i2c interrupt

	RESET

MAIN01:

	BTFSS	LED_OUT,A
	GOTO	MAIN1			; i2c busy

	MOVF	OBFR_SIZE,W,A
	BTFSC	STATUS,Z
	GOTO	MAIN1

	CALL	DELAY_50USEC		; ensure previous transmission complete

	CALL	SEND_I2C		; transfer data from output bfr
					; to i2c bfr and interrupt control
					; processor
MAIN1:

	BTFSS	TMR0_FLG,A
	GOTO	MAIN0

	BCF	TMR0_FLG,A		; 1 msec interval
	CLRWDT

	CALL	UPDATE_COUNT		; update velocity counts
	CALL	SCAN_KBD

	GOTO	MAIN0


			; **********************
			; Update Velocity Counts
			; **********************
UPDATE_COUNT:

	LFSR	FSR0,STATUS_BFR+21H	; first key 21h
	MOVLW	040H			; 64 keys
	MOVWF	BFR_PTR,A

UPDATE1:

	BTFSS	INDF0,0,A		; test status=1
	GOTO	UPDATE2

	BSF	FSR0H,1,A		; -> count bfr
	BTFSS	INDF0,7,A		; test count = 80h
	INCF	INDF0,F,A			; no inc if so
	BCF	FSR0H,1,A

UPDATE2:

	INCF	FSR0L,F,A
	DECFSZ	BFR_PTR,F,A
	GOTO	UPDATE1
	RETURN


			; *****************
			; Keyboard Routines
			; *****************

SCAN_KBD:				; called after timer0 interrupt

.IF (UNIT & DEBUG)			; measure scan period

	MOVLW	011H
	MOVWF	PORTA,A
	CALL	DELAY_5USEC
	MOVLW	010H
	MOVWF	PORTA,A
.ENDIF


	MOVLW	LOW SCAN_MAP		; point to scan map
	MOVWF	TBLPTRL,A
	MOVLW	HIGH SCAN_MAP
	MOVWF	TBLPTRH,A

	LFSR	FSR0,SCAN_BFR		; FSR0 -> new buff	
	MOVLW	014H			; 20 lines to scan
	MOVWF	BFR_PTR,A

SCAN1:

	TBLRD*+				; set up output
	MOVF	TABLAT,W,A		; porta map has led drive low = on
	MOVWF	PORTA,A
	TBLRD*+
	MOVFF	TABLAT,PORTB
	TBLRD*+
	MOVFF	TABLAT,PORTC
	TBLRD*+
	MOVFF	TABLAT,PORTE

	CALL	DELAY_5USEC	

	MOVF	PORTD,W,A		; read input

	ANDLW	07FH			; mask rd7
	MOVWF	POSTINC0,A
	INCF	FSR0L,F,A		; skip old scan	

	DECFSZ	BFR_PTR,F,A
	GOTO	SCAN1
					; test for key change

	LFSR	FSR0,SCAN_BFR		; FSR0 -> scan buff	
	LFSR	FSR1,STATUS_BFR		; FSR1 -> status

	MOVLW	LOW KEY_MAP		; point to key map
	MOVWF	TBLPTRL,A
	MOVLW	HIGH KEY_MAP
	MOVWF	TBLPTRH,A

	MOVLW	014H			; 20 lines to scan
	MOVWF	BFR_PTR,A

SCAN2:
	MOVLW	007H			; 7 bits only
	MOVWF	BIT_PTR,A

	MOVFF	POSTINC0,NEW		; get new and old scans
	MOVFF	POSTINC0,OLD

	MOVF	NEW,W,A			; create change registers
	XORWF	OLD,W,A
					; w reg holds change
	ANDWF	NEW,F,A			; new - bit set on closure
	ANDWF	OLD,F,A			; old - bit set n release

SCAN3:
	BTFSC	NEW,0,A
	CALL	SW_CLOSE
	BTFSC	OLD,0,A
	CALL	SW_RELEASE
	
	RRNCF	NEW,F,A			; get next key
	RRNCF	OLD,F,A

	TBLRD*+				; increment table pointer

	DECFSZ	BIT_PTR,F,A
	GOTO	SCAN3

	DECFSZ	BFR_PTR,F,A
	GOTO	SCAN2
	
	LFSR	FSR0,SCAN_BFR		; copy new to old
	MOVLW	014H			; 20 lines
	MOVWF	BFR_PTR,A

SCAN4:

	MOVF	POSTINC0,W,A
	MOVWF	POSTINC0,A
	DECFSZ	BFR_PTR,F,A
	GOTO	SCAN4


					; sweep up residual stat3
					; test inner key clear and stat3
					; send key off and set stat0

	LFSR	FSR0,SCAN_BFR		; FSR0 -> scan buff	
	LFSR	FSR1,STATUS_BFR		; FSR1 -> status

	MOVLW	LOW KEY_MAP		; point to key map
	MOVWF	TBLPTRL,A
	MOVLW	HIGH KEY_MAP
	MOVWF	TBLPTRH,A

	MOVLW	014H			; 20 lines to scan
	MOVWF	BFR_PTR,A

SWEEP2:
	MOVLW	007H			; 7 bits only
	MOVWF	BIT_PTR,A

	MOVFF	POSTINC0,NEW		; get new and old scans
	MOVFF	POSTINC0,OLD

SWEEP3:
	BTFSS	NEW,0,A			; skip if key down
	CALL	SWEEP
	
	RRNCF	NEW,F,A			; get next key

	TBLRD*+				; increment table pointer

	DECFSZ	BIT_PTR,F,A
	GOTO	SWEEP3

	DECFSZ	BFR_PTR,F,A
	GOTO	SWEEP2

SWEEP_RET:	
.IF (UNIT & DEBUG)			; measure scan period

	MOVLW	011H
	MOVWF	PORTA,A
	CALL	DELAY_50USEC
	MOVLW	010H
	MOVWF	PORTA,A
.ENDIF

	RETURN

SWEEP:					; enter if key up

	TBLRD*				; read keycode
	MOVF	TABLAT,W,A
	BTFSC	STATUS,Z,A
	RETURN				; no key
	
	
	MOVFF	TABLAT,KEYCODE
	BTFSC	KEYCODE,7,A		; only test inner keys
	RETURN
	
	MOVFF	KEYCODE,FSR1L		; point to status register
	MOVF	INDF1,W,A
	XORLW	003H			; test stat3 and open switch
	BTFSS	STATUS,Z,A
	RETURN
	
	CALL	KEY_UP
	RETURN
	
					; enter TBLPTR -> scan code
					; FSR1 -> status buffer
SW_CLOSE:

	TBLRD*				; no increment
	MOVFF	TABLAT,KEYCODE

	BTFSC	KEYCODE,7,A		; test 8th bit for second key
	GOTO	SW2_CLOSE


SW1_CLOSE:				; first (inner) switch closure

	MOVFF	KEYCODE,FSR1L		; point to status register

					; stat0 -> stat1
					; stat1, stat2, stat3 exit
					; stat4 -> stat2

	MOVF	INDF1,W,A		; get status
	BTFSS	STATUS,Z,A
	GOTO	SW1_CLOSE4

SW1_CLOSE0:				; currently status zero
					; set to status 1
	MOVLW	001H			; status = 1 -> counting
	MOVWF	INDF1,A

	BSF	FSR1H,1,A		; point to count register
	CLRF	INDF1,A			; clear count
	BCF	FSR1H,1,A		; back to status buffer
	RETURN

SW1_CLOSE4:				; status 1-4
	XORLW	004H
	BTFSS	STATUS,Z,A
	RETURN				; status 1,2,3 - stay at same status
					; sw1 should be down
	
	MOVLW	002H			; status 4 - return to status 2
	MOVWF	INDF1,A			; 
	RETURN
	
SW2_CLOSE:				; second (outer) switch closure

	BCF	KEYCODE,7,A		; mask 8th bit
	MOVFF	KEYCODE,FSR1L

	MOVF	INDF1,W,A		; get status
	BNZ	SW2_CLOSE1

					; sw2 close at status zero
	BSF	FSR1H,1,A		; point to count
	CLRF	INDF1,A			; 
	BCF	FSR1H,1,A		; back tio status

	GOTO	KEY_DOWN		; sw2 close at status zero
					; -> status 2 and key down to midi
SW2_CLOSE1:

	MOVF	INDF1,W,A		; get status again
	XORLW	001H
	BTFSC	STATUS,Z,A
	GOTO	KEY_DOWN		; sw2 close at status 1
					; -> stat 2 and key down
	
	MOVF	INDF1,W,A		; status
	XORLW	003H
	BTFSS	STATUS,Z,A
	RETURN				; stat 2, 4 remain unchanged
					; sw2 should be down
	MOVLW	002H			; stat 3 - return to stat 2
	MOVWF	INDF1,A			; 
	RETURN
	
KEY_DOWN:				; set status 2 and send key down

	MOVLW	002H
	MOVWF	INDF1,A			; status to 2 and send packet

	BSF	FSR1H,1,A		; -> count
	MOVFF	INDF1,WK1		; wk1 - velocity couunt
	BCF	FSR1H,1,A		; back to status buffer

					; test velocity
	MOVLW	001H			; minimum of 1 msec
	CPFSGT	WK1,A
	MOVWF	WK1,A

	MOVLW	064H			; maximum 100msec
	CPFSLT	WK1,A
	MOVWF	WK1,A
	
	CALL	TO_OUTPUT_BFR		; place in output buffer

	RETURN




SW_RELEASE:

	TBLRD*				; no increment
	MOVFF	TABLAT,KEYCODE
	
	BTFSC	KEYCODE,7,A
	GOTO	SW2_RELEASE

SW1_RELEASE:
	
	MOVFF	KEYCODE,FSR1L		; point to status register
	
	MOVF	INDF1,W,A		; get status
	BTFSC	STATUS,Z,A	
	RETURN				; stat zero -> stat zero
	
	XORLW	001H
	BTFSS	STATUS,Z,A
	GOTO	SW1_RELEASE2
	
	CLRF	INDF1,A			; stat1 -> return to stat zero

	BSF	FSR1H,1,A		; -> count
	CLRF	INDF1,A			; clear counr
	BCF	FSR1H,1,A

	RETURN

SW1_RELEASE2:

	MOVF	INDF1,W,A		; get status again
	XORLW	002H
	BTFSS	STATUS,Z,A
	GOTO	SW1_RELEASE3
	
	MOVLW	004H			; stat 2 - out key released first
	MOVWF	INDF1,A			; return stat 4
	RETURN
	
SW1_RELEASE3:

	MOVF	INDF1,W,A		; status
	XORLW	003H
	BTFSC	STATUS,Z,A
	GOTO	KEY_UP			; sw1 release in stat3
					; send key up -> stat0
	RETURN				; stat 4 -> return still stat 4
	
	
SW2_RELEASE:

	BCF	KEYCODE,7,A		; mask 8th bit
	MOVFF	KEYCODE,FSR1L

	MOVF	INDF1,W,A		; get status
	BTFSC	STATUS,Z,A
	RETURN				; stat 0 -> return stat 0

	XORLW	001H
	BTFSC	STATUS,Z,A
	RETURN				; stat1 -> return still stat 1
	
	MOVF	INDF1,W,A
	XORLW	003H
	BTFSC	STATUS,Z,A
	RETURN				; stat3 - return still stat3
	
	MOVF	INDF1,W,A
	XORLW	004H
	BTFSC	STATUS,Z,A
	GOTO	KEY_UP			; sw release in stat4 - send key up
			
					; must be stat2
	MOVLW	003H
	MOVWF	INDF1,A			; return stat3
	RETURN
	

KEY_UP:					; status zero and key off to midi

	CLRF	INDF1,A			; status zero

	BSF	FSR1H,1,A		; -> count
	CLRF	INDF1,A			; clear counr
	BCF	FSR1H,1,A
	
	CLRF	WK1,A			; zero velocity
	CALL	TO_OUTPUT_BFR
	RETURN
	


TO_OUTPUT_BFR:					; place pkt in output buffer
						; 2 bytes only - no csum
	MOVFF	FSR0L,FSR0_SAVL			; save fsr0
	MOVFF	FSR0H,FSR0_SAVH
		
	LFSR	FSR0,OUTPUT_BFR
	MOVFF	OBFR_IN,FSR0L			; set input pointer

	MOVFF	KEYCODE,POSTINC0		; keycode
	MOVFF	WK1,POSTINC0			; vel count
	
	MOVFF	FSR0L,OBFR_IN			; update input pointer
	MOVLW	002H
	ADDWF	OBFR_SIZE,F,A			; update buffer size

	MOVFF	FSR0_SAVL,FSR0L			; restore fsr0
	MOVFF	FSR0_SAVH,FSR0H

	RETURN

			; *************
			; send i2c data
			; *************

SEND_I2C:

	LFSR	FSR0,OUTPUT_BFR
	MOVFF	OBFR_OUT,FSR0L			; point to output buffer

	MOVF	INDF0,W,A
	MOVFF	POSTINC0,I2C_BUFFER
	XORWF	INDF0,W,A
	MOVFF	POSTINC0,I2C_BUFFER+1

	MOVWF	I2C_BUFFER+2,A
	BCF	INT_OUT,A
	BCF	LED_OUT,A
	
	MOVFF	FSR0L,OBFR_OUT			; set out pointer and size
	DECF	OBFR_SIZE,F,A
	DECF	OBFR_SIZE,F,A
	RETURN


			; ******************
			; I2C Error Routines
			; ******************


I2C_ERROR1:				; overlow on receive

	MOVF	SSPBUF,W,A		; reset BF in sspstat
	BCF	SSPCON1,SSPOV,A
	GOTO	INIT0

I2C_ERROR2:				; collision detect on transmit

	BCF	SSPCON1,WCOL,A
	GOTO	INIT0




			; *****************
			; Timer 0 Interrupt
			; *****************

			; every 1 msec

TIMER0_INTERRUPT:

	BCF	INTCON,TMR0IF,A		; reset interrupt flag

	MOVLW	0D8H			; set for 1 msec interrupt
	MOVWF	TMR0H,A
	MOVLW	0F0H
	MOVWF	TMR0L,A

	BSF	TMR0_FLG,A		; say timer 0 interrupt occurred

	MOVF	COUNT,W,A		; test count register for timeout
	BTFSC	STATUS,Z,A
	RETURN

	DCFSNZ	COUNT,F,A
	BSF	TIMEOUT_FLG,A

	RETURN	




			; *********************
			; I2C Interrupt routine
			; *********************

			; TX used when transmitting data to bus - READ FROM SLAVE
			; RX used when receiving data from bus - WRITE TO SLAVE
			; FSR2 used exclusively in i2c interrupt	

I2C_INTERRUPT:

	BCF	PIR1,SSPIF,A		; clear ssp interrupt

	BTFSS	SSPSTAT,D_A,A		; test address of data
	GOTO	I2C_ADDRESS_INT		

	BTFSS	I2C_RX_FLG,A
	GOTO	I2C_TX_DATA

I2C_RX_DATA:				; receive data from bus
					; not used
	MOVF	SSPBUF,W,A		; get data to save overflow

	LFSR	FSR2,IPBFR
	MOVFF	RXPTR,FSR2L
	MOVWF	INDF2,A

	INCF	RXPTR,F,A
	
	MOVF	RXPTR,W,A
	XORLW	004H
	BTFSC	STATUS,Z
	BSF	I2C_BFR_FLG
	RETURN



I2C_TX_DATA:

	MOVLW	I2C_BUFFER+3		; test all three byte already sent
	XORWF	FSR2L,W,A		; i.e. this is interrupt after not ack recd
	BTFSC	STATUS,Z,A
	RETURN				; don't load data if so

	MOVF	POSTINC2,W,A		; get data
	MOVWF	SSPBUF,A

	BSF	SSPCON1,CKP,A		; enable serial clock

	MOVLW	I2C_BUFFER+3		;  test last byte sent
	CPFSLT	FSR2L,A			; 
	BSF	LED_OUT,A		; 

	RETURN

I2C_ADDRESS_INT:

	BTFSS	SSPSTAT,R_W,A		; test read/write
					; set=read -> tx to bus
	GOTO	I2C_RX_ADDRESS

I2C_TX_ADDRESS:				; transmit data to bus

	BCF	I2C_RX_FLG,A		; say tx data
	MOVF	SSPBUF,W,A		; clear buffer

	BSF	INT_OUT			; 

	LFSR	FSR2,I2C_BUFFER		; point to start of data
	GOTO	I2C_TX_DATA


I2C_RX_ADDRESS:				; not used

	BSF	I2C_RX_FLG,A		; say receiving data
	MOVF	SSPBUF,W,A		; clear data from sspdat
	RETURN




			; *******************
			; Handle i2c routines
			; *******************


HANDLE_I2C:

	LFSR	FSR0,IPBFR
	CLRF	WK0,A		; reset csum

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	XORLW	001H		; test header
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_I2C_EXIT

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	XORLW	004H		; confirm packet length
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_I2C_EXIT

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	MOVWF	WK1,A		; save id

	MOVF	POSTINC0,W,A
	XORWF	WK0,W,A
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_I2C_EXIT

				; packet ok - test pkt id
	MOVF	WK1,W,A
	ANDLW	00FH
	XORLW	00BH
	BTFSC	STATUS,Z,A
	BSF	BOOT_FLG,A

HANDLE_I2C_EXIT:

	CLRF	RXPTR,A
	RETURN



			; **************
			; Delay Routines
			; **************

			; nb beware watchdog timeout

DELAY:
	MOVWF	COUNT,A			; enter w regs holds delay in msec
	BCF	TIMEOUT_FLG,A

DELAY1:
	BTFSS	TIMEOUT_FLG,A
	GOTO	DELAY1
	RETURN


DELAY_50USEC:

	MOVLW	0A0H
	MOVWF	DLY_REG,A
	GOTO	DLY1


DELAY_5USEC:

	MOVLW	010H			
	MOVWF	DLY_REG,A

DLY1:
	DECFSZ	DLY_REG,F,A
	GOTO	DLY1
	RETURN


			; pack to 64 byte boundary

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	




			; *****************
			; Maps and Messages
			; *****************

	ORG	2000H

			; nb. keep to even number of characters

			; port a, b, c, e 
SCAN_MAP:

	DB	B'00110000',B'00000000',B'00000000',B'00000000'
	DB	B'00011000',B'00000000',B'00000000',B'00000000'
	DB	B'00010100',B'00000000',B'00000000',B'00000000'
	DB	B'00010010',B'00000000',B'00000000',B'00000000'
	DB	B'00010001',B'00000000',B'00000000',B'00000000'

	DB	B'00010000',B'00100000',B'00000000',B'00000000'
	DB	B'00010000',B'00010000',B'00000000',B'00000000'
	DB	B'00010000',B'00001000',B'00000000',B'00000000'
	DB	B'00010000',B'00000100',B'00000000',B'00000000'
	DB	B'00010000',B'00000010',B'00000000',B'00000000'
	DB	B'00010000',B'00000001',B'00000000',B'00000000'

	DB	B'00010000',B'00000000',B'10000000',B'00000000'
	DB	B'00010000',B'00000000',B'01000000',B'00000000'
	DB	B'00010000',B'00000000',B'00100000',B'00000000'
	DB	B'00010000',B'00000000',B'00000100',B'00000000'
	DB	B'00010000',B'00000000',B'00000010',B'00000000'
	DB	B'00010000',B'00000000',B'00000001',B'00000000'

	DB	B'00010000',B'00000000',B'00000000',B'00000100'
	DB	B'00010000',B'00000000',B'00000000',B'00000010'
	DB	B'00010000',B'00000000',B'00000000',B'00000001'


			; position in table = scan code
			; value in table = MIDI key code
			; MIDI key code used as address for status and count
			; buffers (range 21h to 60h inc)
			; bit 7 set for 2nd switch

KEY_MAP:		

	DB	037H,03AH,039H,036H,03BH,038H,03CH,030H
	DB	033H,032H,02FH,034H,031H,035H,0B0H,0B3H
	DB	0B2H,0AFH,0B4H,0B1H,0B5H,029H,02CH,02BH
	DB	028H,02DH,02AH,02EH,022H,025H,024H,021H
	DB	026H,023H,027H,0A2H,0A5H,0A4H,0A1H,0A6H
	DB	0A3H,0A7H,0A9H,0ACH,0ABH,0A8H,0ADH,0AAH
	DB	0AEH,0B7H,0BAH,0B9H,0B6H,0BBH,0B8H,0BCH
	DB	0BEH,0C1H,0C0H,0BDH,0C2H,0BFH,0C3H,0C5H
	DB	0C8H,0C7H,0C4H,0C9H,0C6H,0CAH

	DB	0CCH,0CFH,0CEH,0CBH,0D0H,0CDH,0D1H,000H
	DB	000H,000H,0E0H,000H,000H,000H,0DAH,0DDH
	DB	0DCH,0D9H,0DEH,0DBH,0DFH,05AH,05DH,05CH
	DB	059H,05EH,05BH,05FH,053H,056H,055H,052H
	DB	057H,054H,058H,000H,000H,000H,060H,000H
	DB	000H,000H,0D3H,0D6H,0D5H,0D2H,0D7H,0D4H
	DB	0D8H,04CH,04FH,04EH,04BH,050H,04DH,051H
	DB	045H,048H,047H,044H,049H,046H,04AH,03EH
	DB	041H,040H,03DH,042H,03FH,043H


M_REVISION:	dw "  Revision 3.3",00h



	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	
	NOP
	NOP

			; **************
			; Bootstrap Code
			; **************

	ORG	7000H

ERROR_TRAP:

	GOTO	ERROR_TRAP	; trap any run away code

BOOT_INTERRUPT:

	BTFSC	INTCON,TMR0IF,A		; test timer0 interrupt
	BCF	INTCON,TMR0IF,A

	BTFSC	PIR1,SSPIF,A		; test i2c interrupt
	CALL	BOOT_I2C_INTERRUPT

	RETFIE	FAST			; restore w, bsr, status

BOOT_INIT:

	
	CALL	CLEAR_RAM		; clear ram 000h thro' 5ffh
	
	CLRF	EECON1,A		; ensure access to flash/eeprom
	BCF	RCON,IPEN,A		; single level priority on interrupts

					; set port i/o
					; port a

	MOVLW	B'00000110'		; all digital
	MOVWF	ADCON1,A		
	MOVLW	010H			; ra4 high
	MOVWF	LATA,A
	CLRF	TRISA,A			; all output

					; port b
	CLRF	LATB,A
	MOVLW	0C0H			; rb6,rb7 input - reset output
	MOVWF	TRISB,A			
					; port c

	CLRF	LATC,A
	MOVLW	B'00011000'		; all input except i2c bus
	MOVWF	TRISC,A			; scl sda configured later

					; port d
	MOVLW	80H			; int out set high
	MOVWF	LATD,A
	MOVLW	07FH			; all input except rd7
	MOVWF	TRISD,A
					; port e
	CLRF	LATE,A
	CLRF	TRISE,A			; portd gpio / all output

	CLRF	INTCON2,A		; ensure weak pull-up on port b
	CALL	DELAY_50USEC
	CALL	DELAY_50USEC

BOOT_INIT0:					
					; init i2c as slave

	SWAPF	PORTB,W,A		; rb7, rb6 to bits 3,2
	MOVWF	WK0,A
	RRNCF	WK0,W,A
	XORLW	0FFH			; jumper = high
	ANDLW	B'00000110'		; i2c address 10h , 12h or 14h
	IORLW	B'00010000'

.IF (UNIT & DEBUG)		
	MOVLW	010H			; use LEFT block for debug
.ENDIF

	MOVWF	SSPADD,A
	
	ANDLW	00FH
	MOVWF	I2C_ADD,A		; save i2c address
	RRNCF	I2C_ADD,F,A
	SWAPF	I2C_ADD,F,A		; 00, 10 or 20h

	MOVLW	080H			; slew rate disabled
	MOVWF	SSPSTAT,A

	MOVLW	B'00110110'		; i2c 7bit slave
	MOVWF	SSPCON1,A
	CLRF	SSPCON2,A
	
	
	BCF	T0CON,TMR0ON,A		; disable timer0


				; init timer 0

	MOVLW	B'00001000'		; 16 bit, no prescale
	MOVWF	T0CON,A

	MOVLW	0D8H			; set for 1 msec interrupt
	MOVWF	TMR0H,A
	MOVLW	0F0H
	MOVWF	TMR0L,A
	
	BSF	T0CON,TMR0ON,A		; start timer 0

	CLRF	INTCON2,A		; weak pull-up on port b
	CLRF	INTCON3,A

	BSF	PIE1,SSPIE,A		; enable i2c interrupt

	BSF	INTCON,PEIE,A		; enable peripheral interrupts
	BSF	INTCON,GIE,A		; set global interrupt enable


	CLRF	TBLPTRU,A		; assume all maps and messages
					; in range 0000h to ffffh


			; ************
			; Main Routine
			; ************

BOOT_MAIN:


BOOT_MAIN0:

	BTFSC	SSPCON1,SSPOV,A		; test overflow of i2c
	GOTO	BOOT_I2C_ERROR1

	BTFSC	SSPCON1,WCOL,A		; test write collision
	GOTO	BOOT_I2C_ERROR2


	CLRWDT
	BTFSC	I2C_BFR_FLG,A
	CALL	HANDLE_BOOT

	GOTO	BOOT_MAIN0


			; ******************
			; I2C Error Routines
			; ******************


BOOT_I2C_ERROR1:				; overlow on receive

	MOVF	SSPBUF,W,A		; reset BF in sspstat
	BCF	SSPCON1,SSPOV,A
	GOTO	BOOT_INIT0

BOOT_I2C_ERROR2:				; collision detect on transmit

	BCF	SSPCON1,WCOL,A
	GOTO	BOOT_INIT0


			; ***********
			; Handle Boot 
			; ***********

HANDLE_BOOT:

	LFSR	FSR0,IPBFR

	MOVF	POSTINC0,W,A
	XORLW	001H		; test header
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_BOOT_EXIT

	DECF	POSTINC0,W,A
	MOVWF	WK2,A		; save packet length -1

	LFSR	FSR0,IPBFR
	CLRF	WK0,A

HANDLE_BOOT1:

	MOVF	POSTINC0,W,A
	ADDWF	WK0,F,A
	DECFSZ	WK2,F,A
	GOTO	HANDLE_BOOT1

	MOVLW	07FH
	ANDWF	WK0,F,A		; 7 data only

	MOVF	POSTINC0,W,A
	XORWF	WK0,W,A
	BTFSS	STATUS,Z,A
	GOTO	HANDLE_BOOT_EXIT

				; valid packet

	MOVFF	IPBFR+2,WK0	; get id to wk0

	MOVLW	00FH
	ANDWF	WK0,F,A		; mask processor address
	MOVF	WK0,W,A
	XORLW	00AH
	BNZ	HANDLE_BOOT2

	CALL	RETURN_NORM
	RESET


HANDLE_BOOT2:


	MOVF	WK0,W,A
	BNZ	HANDLE_BOOT3	; test hex file data

	CALL	VERIFY_HEX
	BTFSS	STATUS,Z,A
	GOTO	BOOT_NAK

	BCF	LED_OUT,A
	CALL	PROGRAM_UPDATE
	BSF	LED_OUT,A

	GOTO	BOOT_ACK

HANDLE_BOOT3:

	MOVF	WK0,W,A
	XORLW	009H
	BNZ	HANDLE_BOOT_EXIT

	CALL	SEND_CRC_PKT

HANDLE_BOOT_EXIT:

	BCF	I2C_BFR_FLG,A
	CLRF	RXPTR,A
	RETURN

BOOT_ACK:
	MOVF	I2C_ADD,W,A
	ADDLW	00CH
	CALL	SEND_PKT
	GOTO	HANDLE_BOOT_EXIT

BOOT_NAK:

	MOVF	I2C_ADD,W,A
	ADDLW	00DH
	CALL	SEND_PKT
	GOTO	HANDLE_BOOT_EXIT



			; ****************
			; Programme Update
			; ****************

PROGRAM_UPDATE:		; parity check and csum on packet ok
		

	LFSR	FSR0,IPBFR+6
	CALL	GET_HEX_BYTE
	MOVWF	TBLPTRH,A

	CALL	GET_HEX_BYTE
	MOVWF	TBLPTRL,A

PU1:

	LFSR	FSR0,IPBFR+0CH

	LFSR	FSR2,MBFR	; output buffer

	MOVF	TBLPTRL,W,A
	ANDLW	030H
	MOVWF	FSR2L,A

	MOVLW	008H		; 8 words in total
	MOVWF	WK2,A

PU2:

	CALL	GET_HEX_BYTE
	MOVWF	LSB,A
	CALL	GET_HEX_BYTE
	MOVWF	MSB,A

	MOVFF	LSB,POSTINC2
	MOVFF	MSB,POSTINC2

	DECFSZ	WK2,F,A
	GOTO	PU2

	MOVF	TBLPTRL,W,A
	ANDLW	030H		; test 4th line
	XORLW	030H
	BTFSS	STATUS,Z,A
	RETURN			

	BCF	LED_OUT,A

	MOVLW	0C0H
	ANDWF	TBLPTRL,F,A	; tblptr -> start of 64 byte block

	MOVF	TBLPTRH,W,A	; test power on vector
	IORWF	TBLPTRL,W,A
	BNZ	PU3

	CALL	INIT_CRC		; init crc if first block

	DSTR	NORMAL_VECTOR

PU3:
	MOVFF	TBLPTRL,TPTRL		; save tblptr
	MOVFF	TBLPTRH,TPTRH

	CALL	WRITE_FLASH

	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH

	MOVLW	040H			; bytes to verify
	MOVWF	WK2,A
	LFSR	FSR0,MBFR
	
PU4:
	TBLRD*+

	MOVF	TABLAT,W,A
	XORWF	POSTINC0,W,A
	BTFSS	STATUS,Z,A
	GOTO	PU5			; fail - try write once more			
	DECFSZ	WK2,F,A
	GOTO	PU4
					; update crc after verify
	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH

	MOVLW	040H
	MOVWF	WK2,A

	
PU6:

	TBLRD*+

	CALL	CRC

	DECFSZ	WK2,F,A
	GOTO	PU6

	BSF	LED_OUT,A

	RETURN

PU5:
	CLRWDT
	
	MOVFF	TPTRL,TBLPTRL		; restore tablptr
	MOVFF	TPTRH,TBLPTRH
	GOTO	PU3			; go try again


			; ***********
			; CRC Routine
			; ***********
INIT_CRC:

	MOVLW	0FFH
	MOVWF	CRCL,A
	MOVWF	CRCH,A
	RETURN

CRC:			; enter - data in tablat
			; WK0,WK1,WK3 destroyed

	MOVFF	TABLAT,WK1

	MOVLW	008H
	MOVWF	WK3,A	; bit count


CRC1:

	MOVF	CRCL,W,A	; xor lsbits
	XORWF	WK1,W,A
	MOVWF	WK0,A		; wk0 stores result of xor

	BCF	STATUS,C,A	; clear c
	RRCF	CRCH,F,A
	RRCF	CRCL,F,A

	BTFSS	WK0,0,A		; test reult of xor
	GOTO	CRC2

	MOVLW	084H
	XORWF	CRCH,F,A
	MOVLW	008H
	XORWF	CRCL,F,A

CRC2:
	RRNCF	WK1,F,A
	DECFSZ	WK3,F,A
	GOTO	CRC1
	RETURN



			; ***************
			; Verify Hex File
			; ***************

VERIFY_HEX:

	LFSR	FSR0,IPBFR+3		; -> :
	MOVF	POSTINC0,W,A
	XORLW	03AH
	BTFSS	STATUS,Z,A
	RETURN				; return non-zero if fail

	CLRF	WK0,A	; reset csum
	
	MOVLW	015H			; line lenght in bytes - always full line
	MOVWF	WK2,A			; including csum

VERIFY_HEX1:	
	
	CALL	GET_HEX_BYTE
	ADDWF	WK0,F,A

	DECFSZ	WK2,F,A
	GOTO	VERIFY_HEX1
	MOVF	WK0,W,A			; return zero if ok

	RETURN


GET_HEX_BYTE:				; get hex byte @ fsr0

	CALL	GET_HEX_NIBBLE
	MOVWF	WK1,A
	SWAPF	WK1,F,A			; xfer to top nibble
	CALL	GET_HEX_NIBBLE
	ADDWF	WK1,W,A
	RETURN


GET_HEX_NIBBLE:				; get single nibble from hex file

	MOVLW	000H
	BTFSC	INDF0,6,A			; test a-f
	MOVLW	009H
	ADDWF	POSTINC0,W,A
	ANDLW	00FH
	RETURN



			; ******************
			; Boot i2c Interrupt
			; ******************

BOOT_I2C_INTERRUPT:


			; TX used when transmitting data to bus - READ FROM SLAVE
			; RX used when receiving data from bus - WRITE TO SLAVE
			; FSR2 used exclusively for this interrupt


	BCF	PIR1,SSPIF,A		; clear ssp interrupt

	BSF	INT_OUT,A		; clear interrupt out
	BSF	LED_OUT,A

	BTFSS	SSPSTAT,D_A,A		; test address of data
	GOTO	BOOT_I2C_ADDRESS_INT		

	BTFSS	I2C_RX_FLG,A
	GOTO	BOOT_I2C_TX_DATA

BOOT_I2C_RX_DATA:			; receive data from bus
					; not used
	MOVF	SSPBUF,W,A		; get data to save overflow

	LFSR	FSR2,IPBFR
	MOVFF	RXPTR,FSR2L
	MOVWF	INDF2,A

	MOVF	RXPTR,W,A		; test -> pkt length
	XORLW	001H
	BTFSC	STATUS,Z,A
	MOVFF	INDF2,RXLEN

	INCF	RXPTR,F,A
	
	MOVLW	003H
	CPFSGT	RXPTR,A
	RETURN				; minimum packet not yet recd

	MOVF	RXPTR,W,A
	XORWF	RXLEN,W,A
	BTFSC	STATUS,Z
	BSF	I2C_BFR_FLG,A
	RETURN
				
BOOT_I2C_TX_DATA:

	MOVF	I2C_BUFFER+1,W,A	; get pkt length
	ADDLW	I2C_BUFFER
	XORWF	FSR2L,W,A		; test all data sent
	BTFSC	STATUS,Z,A		; don't load for ack interrupt
	RETURN

	MOVF	POSTINC2,W,A		; get data
	MOVWF	SSPBUF,A

	BSF	SSPCON1,CKP,A		; enable serial clock
	RETURN

BOOT_I2C_ADDRESS_INT:

	BTFSS	SSPSTAT,R_W,A		; test read/write
					; set=read -> tx to bus
	GOTO	BOOT_I2C_RX_ADDRESS

BOOT_I2C_TX_ADDRESS:				; transmit data to bus

	BCF	I2C_RX_FLG,A		; say tx data
	MOVF	SSPBUF,W,A		; clear buffer

	LFSR	FSR2,I2C_BUFFER		; point to start of data
	GOTO	BOOT_I2C_TX_DATA


BOOT_I2C_RX_ADDRESS:				; not used

	BSF	I2C_RX_FLG,A		; say receiving data
	MOVF	SSPBUF,W,A		; clear address
	RETURN



			; *************************
			; Boot send packet routines
			; *************************


SEND_PKT:

	MOVWF	I2C_BUFFER+2,A		; enter w = pkt id
	MOVWF	TXCSUM,A
	MOVLW	001H			; header
	MOVWF	I2C_BUFFER,A
	ADDWF	TXCSUM,F,A

	MOVLW	004H			; length
	MOVWF	I2C_BUFFER+1,A
	ADDWF	TXCSUM,F,A

	MOVFF	TXCSUM,I2C_BUFFER+3	; put csum
	BCF	INT_OUT,A
	RETURN

SEND_CRC_PKT:

	LFSR	FSR0,I2C_BUFFER

	MOVLW	001H			; header
	MOVWF	POSTINC0,A
	MOVWF	TXCSUM,A

	MOVLW	008H			; length
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A

	MOVF	I2C_ADD,W,A
	ADDLW	004H
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A


	MOVFF	CRCH,WK0		; place crc in ascii hex format
	CALL	PUT_HEX
	MOVFF	CRCL,WK0
	CALL	PUT_HEX

	MOVFF	TXCSUM,POSTINC0
	BCF	INT_OUT,A
	RETURN

PUT_HEX:				; enter data in wk0

	SWAPF	WK0,W,A
	ANDLW	00FH
	MOVWF	WK1,A

	CALL	PUT_H1

	MOVF	WK0,W,A
	ANDLW	00FH
	MOVWF	WK1,A			; fall thro for low nibble

PUT_H1:

	MOVLW	00AH
	CPFSLT	WK1,A
	GOTO	PUT_H2

	MOVF	WK1,W,A			; 0-9
	IORLW	030H
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A
	RETURN
PUT_H2:				; A-F

	MOVF	WK1,W,A
	ADDLW	'A'-00AH
	MOVWF	POSTINC0,A
	ADDWF	TXCSUM,F,A
	RETURN





			; **************
			; Flash Routines
			; **************
			;  all flash routines based on 64 byte transfer
			; mbfr used for ram
			; fsr0 as indirect pointer

READ_FLASH:		; read 64 bytes flash @ tblptr to mbfr
			; use dstr to set tblptr

	BCF	EECON1,CFGS,A	; data/ flash
	BSF	EECON1,EEPGD,A	; flash
	LFSR	FSR0,MBFR
	MOVLW	040H		; 64 bytes
	MOVWF	WK2,A

READ_FLASH1:

	TBLRD*+	
	MOVFF	TABLAT,POSTINC0	; xfer data
	DECFSZ	WK2,F,A
	GOTO	READ_FLASH1
	RETURN


ERASE_FLASH:

	BCF	INTCON,GIE,A	; disable all interrupts during write

	BCF	EECON1,CFGS,A	; flash/eeprom
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A	; enable write
	BSF	EECON1,FREE,A	; line erase (64 bytes)

	MOVLW	055H		; timed access
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A

	BSF	EECON1,WR,A
	NOP
	BSF	INTCON,GIE,A
	RETURN



WRITE_FLASH:

	BCF	INTCON,GIE,A	; disable all interrupts during write

	BCF	EECON1,CFGS,A	; flash/eeprom
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A	; enable write
	BSF	EECON1,FREE,A	; line erase (64 bytes)

	MOVLW	055H		; timed access
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A

	BSF	EECON1,WR,A
	NOP
		
				; write data in mbfr @ tblptr
				; write 8 blocks of 8 bytes @ tblptr


	LFSR	FSR0,MBFR	; point to ram
	MOVLW	008H
	MOVWF	WK3,A		; 8 loops of 8 bytes each

WRITE_FLASH1:

	MOVLW	008H		; byte count
	MOVWF	WK2,A

WRITE_FLASH2:

	MOVF	POSTINC0,W,A	; get data from buffer	
	MOVWF	TABLAT,A	; 8 bytes to temp registers
	TBLWT*+

	DECFSZ	WK2,F,A
	GOTO	WRITE_FLASH2

	TBLRD*-			; dec tblptr to ensure written 8 byte earlier
	BSF	EECON1,EEPGD,A	; -> flash
	BSF	EECON1,WREN,A
	
	MOVLW	055H
	MOVWF	EECON2,A
	MOVLW	0AAH
	MOVWF	EECON2,A
	BSF	EECON1,WR,A
	NOP
	
	TBLRD*+			; restore tablptr
	DECFSZ	WK3,F,A
	GOTO	WRITE_FLASH1
	BCF	EECON1,WREN,A	; disable further writes
	BSF	INTCON,GIE,A
	RETURN




			; *********
			; Clear Ram
			; *********

CLEAR_RAM:

	LFSR	FSR0,0000H		; clear ram 000h thro 5ffh

CLR_LOOP:

	CLRF	POSTINC0,A
	
	MOVF	FSR0H,W,A		; high pointer to w reg
	XORLW	006H
	BTFSS	STATUS,Z,A
	GOTO	CLR_LOOP
	RETURN	




			; ********************
			; Read / Write Vectors
			; ********************

COPY_NORM_VECTOR:	; copy normal vetor to normal_vector

	DSTR	PON_RESET
	CALL	READ_FLASH

	DSTR	NORMAL_VECTOR
	CALL	WRITE_FLASH
	RETURN
	
WRITE_BOOT_VECTOR:	; replace normal vectors withboot vectors

	DSTR	BOOT_VECTOR
	CALL	READ_FLASH

	DSTR	PON_RESET
	CALL	WRITE_FLASH

	RETURN

RETURN_NORM:

	CLRF	INTCON,A

	DSTR	NORMAL_VECTOR
	CALL	READ_FLASH	; get normal vector

	DSTR	PON_RESET
	CALL	WRITE_FLASH
	RETURN



			; *************
			; Store Vectors
			; *************


	ORG	7780H

BOOT_VECTOR:

	GOTO	BOOT_INIT
	
	ORG	7788H
	GOTO	BOOT_INTERRUPT




	ORG	7800H

NORMAL_VECTOR:				; normal vector copied here
					; on entering pgm mode






	END
