/* MainFrame.java
 *
 * Created on 29 September 2006, 09:37
 *
 * Alan Bradbury - AND Technology Research
 * Steven Kear - AND Technology Research
 *
 * AXiS 64 programming utility.
 *
 * This class contains the main function.
 *
 * Version History
 * ---------------
 *
 * Vn       Date            Author
 * ----     ----------      ---------
 * 1.0      21/09/2006      AB & SK - ANDTR
 * 1.2      17/01/2007      PS - ANDTR
 *
 * TODO
 *
 */

package AXISProgrammer;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;
import gnu.io.*;	// the serial comms functions
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.JRootPane;
import javax.swing.JOptionPane;
import java.awt.*;

import javax.swing.*;
import java.nio.*;
import java.nio.channels.*;
import java.io.*;

import com.apple.eawt.ApplicationAdapter;
import com.apple.eawt.ApplicationEvent;
import com.apple.eawt.Application;

/** Retrieves all the user properties.  The properties are retrieved from
 * a modal dialog box.  The properties that are currently selectable are:
 * <p>
 * comm port
 *
 * @author Alan Bradbury - AND Technology Research
 * @version 1.0
 */
public final class MainFrame extends javax.swing.JFrame {
	// classes
	private final class MacOSAboutHandler extends Application {
		// classes
		private final class AboutBoxHandler extends ApplicationAdapter {
			public final void handleAbout(ApplicationEvent event) {
				AboutDialog dlg = new AboutDialog(owner, true);
				dlg.setLocationRelativeTo(owner);
				dlg.setVisible(true);
				event.setHandled(true);		// don't want the default about box popping up!!!
			}
			
			public final void handleQuit(ApplicationEvent event) {
		//		boolean closeApplication = owner.isCloseApplication();
		//		event.setHandled(closeApplication);
				event.setHandled(true);		// seems to indicate that want to close application!!!
			}
		}
		
		// variables
		private MainFrame owner;

		// constructor
		private MacOSAboutHandler(MainFrame owner) {
			this.owner = owner;
			addApplicationListener(new AboutBoxHandler());
		}
	}
	
	/** Defines a filter for a file chooser
	*  Filters '.zip' files
	*/
	private final class ZipFileFilter extends javax.swing.filechooser.FileFilter {
		public final boolean accept(File file) {
			String name = file.getName();
			if (name.endsWith(".zip") || file.isDirectory()) {
				return true;
			}
			return false;
		} 

		public final String getDescription() {
			return "ZIP Files (*.zip)";
		}
	}
	
	// port identifier item
	private final class PortIdentifierItem {
		// variables
		private CommPortIdentifier portIdentifier;
		
		// constructor
		private PortIdentifierItem(CommPortIdentifier portIdentifier) {
			this.portIdentifier = portIdentifier;
		}
		
		public final String toString() {
			return portIdentifier.getName();
		}
	}
	
	// constants
	protected static final String APP_NAME = "AXiS 64 Firmware Updater";
	
	private static final String UPDATEURLDIRECTORYSTRING = "http://www.c-thru-music.com/support/AXiS-64/Firmware/";   // URL to get the update zip file from the internet
	private static final String UPDATEZIPFILEPREFIX = "AXiS-64_";   // prefix of the update zip file name
	private static final int VERSIONNUMBERSIZE = 3; // size of the version number in the update zip file name (eg 1.2)

	// variables
	private boolean getUpdateZipFromInternet;		// flag for whether to get the update zip from the internet (initialised to get it from the internet)

	/** Creates a new form UserProperties.  Any serial comm ports on the system
	 * are enumerated and added to the serial port combo box.
	 *
	 * @param parent The parent window
	 * @param modal Whether the dialog box is modal
	 * @param textArea Text area to output error messages to
	 */
	protected MainFrame() {
		getUpdateZipFromInternet = true;
		
		initComponents();
		
		// add the menu bar if not using a mac
		String version = System.getProperty("mrj.version");
		if (version == null) {
			setJMenuBar(menuBar);
			pack();		// need to re-pack the layout
		}
		else {
			// create the mac handler
			new MacOSAboutHandler(this);
		}
		
	//	JRootPane rootPane = getRootPane();
	//	rootPane.setDefaultButton(updateButton);

		// enumerates the serial ports and adds them to the combo box
		Enumeration ports = CommPortIdentifier.getPortIdentifiers();

		if (ports != null) {    // are there any comm ports
			while (ports.hasMoreElements()) {   // loops through the comm ports
				CommPortIdentifier port = (CommPortIdentifier)ports.nextElement();  // get next port

				int portType = port.getPortType();
				if (portType == port.PORT_SERIAL) {   // is the port serial
					// add the serial port to the combo box
					PortIdentifierItem item = new PortIdentifierItem(port);
					serialPort.addItem(item);
				}
			}
		}
		
		// select the first port that is available
		int count = serialPort.getItemCount();
		if (count > 0) {
			serialPort.setSelectedIndex(0);
		}
		
		internetOpt.setSelected(getUpdateZipFromInternet);
		hardDriveOpt.setSelected(!getUpdateZipFromInternet);
		
		fileNameLabel.setEnabled(!getUpdateZipFromInternet);
		fileNameTextField.setEnabled(!getUpdateZipFromInternet);
		fileNameTextField.setText("");
		browseButton.setEnabled(!getUpdateZipFromInternet);
		
		// Retrieves the application's icon from the lib directory, allowing
		// the customer to indepdantly update the icon if they wish to.
		Class c = getClass();
		URL url = c.getResource("AXiS 64 Logo.gif");
		if (url != null) {
			ImageIcon imageIcon = new ImageIcon(url);

			// set the application's icon
			Image image = imageIcon.getImage();
			setIconImage(image);
		}
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        fileLocation = new javax.swing.ButtonGroup();
        menuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();
        exitMenuItem = new javax.swing.JMenuItem();
        serialPortLabel = new javax.swing.JLabel();
        serialPort = new javax.swing.JComboBox();
        updateButton = new javax.swing.JButton();
        refreshButton = new javax.swing.JButton();
        fileLocationPanel = new javax.swing.JPanel();
        internetOpt = new javax.swing.JRadioButton();
        hardDriveOpt = new javax.swing.JRadioButton();
        fileNameLabel = new javax.swing.JLabel();
        fileNameTextField = new javax.swing.JTextField();
        browseButton = new javax.swing.JButton();

        fileMenu.setMnemonic('F');
        fileMenu.setText("File");
        aboutMenuItem.setMnemonic('A');
        aboutMenuItem.setText("About...");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(aboutMenuItem);

        exitMenuItem.setMnemonic('E');
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });

        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("AXiS 64 Firmware Updater");
        setResizable(false);
        serialPortLabel.setDisplayedMnemonic('S');
        serialPortLabel.setLabelFor(serialPort);
        serialPortLabel.setText("Serial port:");

        updateButton.setMnemonic('U');
        updateButton.setText("Update...");
        updateButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateButtonActionPerformed(evt);
            }
        });

        refreshButton.setMnemonic('R');
        refreshButton.setText("Refresh");
        refreshButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                refreshButtonActionPerformed(evt);
            }
        });

        fileLocationPanel.setBorder(javax.swing.BorderFactory.createTitledBorder("File location"));
        fileLocation.add(internetOpt);
        internetOpt.setMnemonic('I');
        internetOpt.setSelected(true);
        internetOpt.setText("Internet (recommended)");
        internetOpt.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        internetOpt.setMargin(new java.awt.Insets(0, 0, 0, 0));
        internetOpt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                internetOptActionPerformed(evt);
            }
        });

        fileLocation.add(hardDriveOpt);
        hardDriveOpt.setMnemonic('H');
        hardDriveOpt.setText("Hard drive");
        hardDriveOpt.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        hardDriveOpt.setMargin(new java.awt.Insets(0, 0, 0, 0));
        hardDriveOpt.setName("");
        hardDriveOpt.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                hardDriveOptActionPerformed(evt);
            }
        });

        fileNameLabel.setDisplayedMnemonic('n');
        fileNameLabel.setLabelFor(fileNameTextField);
        fileNameLabel.setText("File name:");

        browseButton.setText("...");
        browseButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                browseButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout fileLocationPanelLayout = new org.jdesktop.layout.GroupLayout(fileLocationPanel);
        fileLocationPanel.setLayout(fileLocationPanelLayout);
        fileLocationPanelLayout.setHorizontalGroup(
            fileLocationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, fileLocationPanelLayout.createSequentialGroup()
                .addContainerGap()
                .add(fileLocationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, fileLocationPanelLayout.createSequentialGroup()
                        .add(fileNameTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 239, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(browseButton))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, hardDriveOpt, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 290, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, internetOpt, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 290, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, fileNameLabel))
                .addContainerGap())
        );
        fileLocationPanelLayout.setVerticalGroup(
            fileLocationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(fileLocationPanelLayout.createSequentialGroup()
                .add(internetOpt, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 18, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(hardDriveOpt, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 18, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(fileNameLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(fileLocationPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(fileNameTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(browseButton))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(fileLocationPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(serialPort, 0, 249, Short.MAX_VALUE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(refreshButton))
                    .add(serialPortLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 326, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, updateButton))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(serialPortLabel)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(serialPort, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(refreshButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(fileLocationPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(updateButton)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void hardDriveOptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_hardDriveOptActionPerformed
		// We're going to be getting the file from the hard drive/filesystem so 
		// enable the browse ComboBox.
		getUpdateZipFromInternet = false;
		fileNameLabel.setEnabled(true);
		fileNameTextField.setEnabled(true);
		browseButton.setEnabled(true);
	}//GEN-LAST:event_hardDriveOptActionPerformed

	private void internetOptActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_internetOptActionPerformed
		// We're going to be getting the file from the Web so no need to browse
		// hard drive/filesystem.
		getUpdateZipFromInternet = true;
		fileNameLabel.setEnabled(false);
		fileNameTextField.setEnabled(false);
		browseButton.setEnabled(false);
	}//GEN-LAST:event_internetOptActionPerformed

	private void refreshButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_refreshButtonActionPerformed
		try {
			// show a busy cursor whilst updating the serial ports
			setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

			// get the currently selected port name
			String portName = null;
			PortIdentifierItem item = (PortIdentifierItem)serialPort.getSelectedItem();
			if (item != null) {
				portName = item.portIdentifier.getName();
			}

			// update the list of serial ports
			serialPort.removeAllItems();
			Enumeration ports = CommPortIdentifier.getPortIdentifiers();
			if (ports != null) {    // are there any comm ports
				while (ports.hasMoreElements()) {   // loops through the comm ports
					CommPortIdentifier port = (CommPortIdentifier)ports.nextElement();  // get next port

					int portType = port.getPortType();
					if (portType == port.PORT_SERIAL) {   // is the port serial
						// add the serial port to the combo box
						item = new PortIdentifierItem(port);
						serialPort.addItem(item);
					}
				}
			}

			// select the previous port (if possible)
			int count = serialPort.getItemCount();
			if (portName != null) {
				int i;
				for (i = 0; i < count; i++) {
					item = (PortIdentifierItem)serialPort.getItemAt(i);
					String listPortName = item.portIdentifier.getName();
					if (listPortName.compareTo(portName) == 0) {
						serialPort.setSelectedIndex(i);
						break;		// escape from loop
					}
				}
				if (i >= count) {
					serialPort.setSelectedIndex(-1);
				}
			}
			else {
				// select the first port (if possible)
				if (count > 0) {
					serialPort.setSelectedIndex(0);
				}
			}
		}
		finally {
			setCursor(Cursor.getDefaultCursor());
		}
	}//GEN-LAST:event_refreshButtonActionPerformed

	private void updateButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateButtonActionPerformed

		// store the comms settings and close the properties window
		String portName = null;
		String fileName = "";
		URL updateURL = null;
		
		// serial port
		PortIdentifierItem item = (PortIdentifierItem)serialPort.getSelectedItem();
		if (item != null) {
			portName = item.portIdentifier.getName();
		}
		else {
			Toolkit.getDefaultToolkit().beep();
			JOptionPane.showMessageDialog(this, "Please select a valid serial port.",
				APP_NAME, JOptionPane.WARNING_MESSAGE);
			serialPort.requestFocusInWindow();
			return;		// go no further
		}
		
		// file name
		fileName = fileNameTextField.getText();
		if (!getUpdateZipFromInternet) {
			File file = new File(fileName);
			if (!file.isFile()) {
				Toolkit.getDefaultToolkit().beep();
				JOptionPane.showMessageDialog(this, "Please enter a valid ZIP file name.",
					APP_NAME, JOptionPane.WARNING_MESSAGE);
				String text = fileNameTextField.getText();
				int length = text.length();
				fileNameTextField.select(0, length);
				fileNameTextField.requestFocusInWindow();
				return;		// go no further
			}
		}
		
		// get update url first
		if (getUpdateZipFromInternet) {
			String updateURLString = null;
			try {
				URL url = new URL(UPDATEURLDIRECTORYSTRING);
				InputStream inputStream = url.openStream();
				InputStreamReader reader = new InputStreamReader(inputStream);
				BufferedReader updateDirectoryHTML = new BufferedReader(reader);   // get the html that contains the update zip file names

				String updateDirectoryHTMLLine;
				String updateZipFileVersion = "0";

				while ((updateDirectoryHTMLLine = updateDirectoryHTML.readLine()) != null) {    // itterates through the lines of the update directory html
					int updateZipFileIndex = updateDirectoryHTMLLine.indexOf(UPDATEZIPFILEPREFIX);     // stores the index of an update zip file in the update directory HTML line
					if (updateZipFileIndex != -1) {    // is there an update zip file specified in this line
						String possibleUpdateZipFileVersion = updateDirectoryHTMLLine.substring(updateZipFileIndex + UPDATEZIPFILEPREFIX.length(), updateZipFileIndex + UPDATEZIPFILEPREFIX.length() + VERSIONNUMBERSIZE);
						if (Float.parseFloat(possibleUpdateZipFileVersion) > Float.parseFloat(updateZipFileVersion)) {
							int updateZipFileDotIndex = updateDirectoryHTMLLine.indexOf(".zip", updateZipFileIndex);    // index of the . in the update zip file
							if (updateZipFileDotIndex != -1) {  // have we found a zip file
								String updateZipFileSufix = updateDirectoryHTMLLine.substring(updateZipFileIndex + UPDATEZIPFILEPREFIX.length() + VERSIONNUMBERSIZE, updateZipFileDotIndex + VERSIONNUMBERSIZE + 1);
								updateZipFileVersion = possibleUpdateZipFileVersion;
								updateURLString = UPDATEURLDIRECTORYSTRING + UPDATEZIPFILEPREFIX + possibleUpdateZipFileVersion + updateZipFileSufix;
							}
						}
					}
				}
			}
			catch (IOException e) {
				Toolkit.getDefaultToolkit().beep();
				JOptionPane.showMessageDialog(this, "Unable to access firmware update page (" + UPDATEURLDIRECTORYSTRING + ").",
					APP_NAME, JOptionPane.WARNING_MESSAGE);
				return;		// go no further
			}

			if (updateURLString == null) {
				Toolkit.getDefaultToolkit().beep();
				JOptionPane.showMessageDialog(this, "Unable to extract ZIP file from firmware update page (" + UPDATEURLDIRECTORYSTRING + ").",
					APP_NAME, JOptionPane.WARNING_MESSAGE);
				return;		// go no further
			}

			try {
				updateURL = new URL(updateURLString);
			}
			catch (MalformedURLException e) {
				Toolkit.getDefaultToolkit().beep();
				JOptionPane.showMessageDialog(this, "Unable to access ZIP file.",
					APP_NAME, JOptionPane.WARNING_MESSAGE);
				return;		// go no further
			}
		}
		else {
			File file = new File(fileName);
			try {
				updateURL = file.toURL();
			}
			catch (MalformedURLException e) {
				Toolkit.getDefaultToolkit().beep();
				JOptionPane.showMessageDialog(this, "Unable to access ZIP file.",
					APP_NAME, JOptionPane.WARNING_MESSAGE);
				return;		// go no further
			}
		}
		
		UpdateDialog updateDialog = new UpdateDialog(this, portName, updateURL);
		updateDialog.setLocationRelativeTo(this);
		updateDialog.setVisible(true);
	}//GEN-LAST:event_updateButtonActionPerformed

	private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
	//	if (isCloseApplication()) {
			//	dispose();
			System.exit(0);
	//	}
	}//GEN-LAST:event_exitMenuItemActionPerformed

	private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
		AboutDialog dlg = new AboutDialog(this, true);
		dlg.setLocationRelativeTo(this);
		dlg.setVisible(true);
	}//GEN-LAST:event_aboutMenuItemActionPerformed

	private void browseButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_browseButtonActionPerformed
		// Clicking on the combo box launches the file browser (jFileChooser)
		// Create a new instance of the JFileChooser 
		JFileChooser fileChooser = new JFileChooser();

		fileChooser.setAcceptAllFileFilterUsed(false);
		fileChooser.setFileFilter(new ZipFileFilter()); // must set file filter after removing other file filters to ensure
							// this file filter is selected when the file chooser is opened

		// Set the current directory to the application's current directory
		try {
			// Create a File object containing the canonical path of the
			// desired file
			File file = null;
			String fileName = this.fileNameTextField.getText();
			int length = fileName.length();
			if (length > 0) {
				file = new File(fileName);
				if (file.isFile()) {
					file = file.getParentFile();
				}
				else if (!file.isDirectory()) {
					file = null;
				}
			}
			if (file == null) {
				file = new File("");
				file = file.getCanonicalFile();
			}

			// Set the selected file
			fileChooser.setSelectedFile(file);
			fileChooser.setDialogTitle("Browse");

			// Show the dialog; wait until dialog is closed
			int result = fileChooser.showOpenDialog(this);

			if (result == JFileChooser.APPROVE_OPTION) {
				// Only get the name of the file if Aprove Option selected (ignore 
				// the file if cancel is pressed)
				// Get the currently selected file
				File updateZipFile = fileChooser.getSelectedFile();

				// Display the select file in the combo box
				String name;
				if (updateZipFile != null) {
					name = updateZipFile.getAbsolutePath();
				}
				else {
					name = "";
				}
				fileNameTextField.setText(name);
			}
		}
		catch (IOException e) {
			// do nothing
		}
	}//GEN-LAST:event_browseButtonActionPerformed

	/** Updates the AXIS 64 with the files contained within the zip file */                
	/** Makes visible the properties window as the properties button has been clicked */    
	/** The main function.
	 *
	 * @param args the command line arguments
	 */
	public static void main(String[] args) {
		// Hides consoles that the RXTX JAR file writes to when 
		// it's initalised.
		System.err.close(); 
		System.out.close();
 		
		java.awt.EventQueue.invokeLater(new Runnable() {
			public final void run() {
				// Sets the UI's look & feel to the system's UI style.
				// (Automatically happens with Mac, in Windows the default UI 
				// style is Java, this will set it to Windows style when run on 
				// Windows machine).
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				}
				catch (UnsupportedLookAndFeelException ex) {
					ex.printStackTrace();
				}
				catch (ClassNotFoundException ex) {
					ex.printStackTrace();
				}
				catch (IllegalAccessException ex) {
					ex.printStackTrace();
				}
				catch (InstantiationException ex) {
					ex.printStackTrace();
				}

				MainFrame frame = new MainFrame();	// creates and sets visible the main window
				frame.setLocationRelativeTo(null);
				frame.setVisible(true);
			}
		});
	}
	
	/** Whenever the properties window is shown the selected comm port
	 * (if there is one) is selected in the comm port combo box
	 */	
	/** Stores the properties and closes the properties window */
	/** Closes the properties window */    

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JButton browseButton;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.ButtonGroup fileLocation;
    private javax.swing.JPanel fileLocationPanel;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JLabel fileNameLabel;
    private javax.swing.JTextField fileNameTextField;
    private javax.swing.JRadioButton hardDriveOpt;
    private javax.swing.JRadioButton internetOpt;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JButton refreshButton;
    private javax.swing.JComboBox serialPort;
    private javax.swing.JLabel serialPortLabel;
    private javax.swing.JButton updateButton;
    // End of variables declaration//GEN-END:variables
}
