/* UpdateDialog.java
 *
 * Created on 29 September 2006, 09:37
 *
 * Alan Bradbury - AND Technology Research
 * Steven Kear - AND Technology Research
 *
 * AXiS 64 programming utility.
 *
 */

package AXISProgrammer;

import gnu.io.*;    // the serial comms functions
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.io.*;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.util.*;
import java.util.zip.*;
import java.net.*;
import javax.swing.*;
import java.awt.*;

// extra runtime option for mac osx -Xdock:name="AXiS 64 Firmware Updater"

/** Main application window.  This class contains the main function.
 *
 * @author Alan Bradbury - AND Technology Research
 * @version 1.0
 */
public final class UpdateDialog extends javax.swing.JDialog {
	// classes
	/** Thread to update the AXIS 64 
	 * Ensures that the UI updates while programming.
	 */

	private final class UpdateThread extends Thread {
		// variables
		private LabelUpdater statusUpdater;
		
		// constructor
		private UpdateThread() {
			statusUpdater = null;
		}
		
		// methods
		private final void setStatusText(String text) {
			statusUpdater.setText(text);
		}

		public void run() {
			statusUpdater = new LabelUpdater(status);
			
			ProgressBarUpdater progressBarUpdater = new ProgressBarUpdater(programmingProgressBar);
			Programmer AXIS64Programmer = new Programmer(portName, output, progressBarUpdater);		// creates the AXIS64 programmer
			int totalNumberOfLines = 0;

			if (AXIS64Programmer.connect(true)) {  // connect to the programming comm port and check for success
				// we have connected to the programming comm port successfully

				if (updateURL != null) {   // check a update zip file has been selected
					try {
						ZipInputStream updateZipStream;

						// Update the status text above the Progress Bar
						setStatusText("Entering programming mode...");
						if (AXIS64Programmer.enterProgramMode()) {  // enter program mode and check for success
							// we have entered program mode successfully

							// get total number of lines to download
							InputStream inputStream = updateURL.openStream();
							updateZipStream = new ZipInputStream(inputStream);

							ZipEntry updateFile;
							while ((updateFile = updateZipStream.getNextEntry()) != null) {   // loops through the entries in the zip file
								if (!updateFile.isDirectory()) {    // checks that the zip file entry is not a directory
									String name = updateFile.getName();
									File file = new File(name);
									name = file.getName();
									if (startsWithIgnoreCase(name, KEY_PROCESSOR_FILE_NAME)) { // is this the key procesor file
										// this is the key processor file

										totalNumberOfLines += 3 * AXIS64Programmer.countLinesInFile(new BufferedReader(new InputStreamReader(updateZipStream)));
									}
									else if (startsWithIgnoreCase(name, FUNCTION_KEY_PROCESSOR_FILE_NAME)) {   // is this the function key processor file
										// this is the function key process file

										totalNumberOfLines += AXIS64Programmer.countLinesInFile(new BufferedReader(new InputStreamReader(updateZipStream)));
									}
									else if (startsWithIgnoreCase(name, MAIN_PROCESSOR_FILE_NAME)) {   // is this the main processor file
										// this is the main processor file

										totalNumberOfLines += AXIS64Programmer.countLinesInFile(new BufferedReader(new InputStreamReader(updateZipStream)));
									}
									else if (startsWithIgnoreCase(name, VELOCITY_TABLE_FILE_NAME)) {   // is this the velocity table file
										// this is the velocity table file

										totalNumberOfLines += AXIS64Programmer.countLinesInFile(new BufferedReader(new InputStreamReader(updateZipStream)));
									}
								}

								updateZipStream.closeEntry();
							}

							// program the processors

							updateZipStream = new ZipInputStream(updateURL.openStream());

							while ((updateFile = updateZipStream.getNextEntry()) != null) {   // loops through the entries in the zip file
								if (!updateFile.isDirectory()) {    // checks that the zip file entry is not a directory
									String name = updateFile.getName();
									File file = new File(name);
									name = file.getName();
									if (startsWithIgnoreCase(name, KEY_PROCESSOR_FILE_NAME)) { // is this the key procesor file
										// this is the key processor file

										writeOutput("> Programming key processor 1...\r\n");
										setStatusText("Programming key processor 1...");
										if (AXIS64Programmer.downloadFile(new BufferedReader(new InputStreamReader(updateZipStream)), Programmer.KEY_PROCESSOR_1, totalNumberOfLines) == false) {
											writeOutput("> Error: Failed to program key processor 1\r\n");
											setStatusText("Error: Failed to program key processor 1");
										}
										else {
											writeOutput("> Successfully programmed key processor 1\r\n");
										}

										// as we have now used the key processor zip stream we need to get it again
										ZipInputStream updateKeyProcessorZipStream = new ZipInputStream(updateURL.openStream());
										while ((updateFile = updateKeyProcessorZipStream.getNextEntry()) != null) {
											if (!updateFile.isDirectory()) {    // checks that the zip file entry is not a directory
												name = updateFile.getName();
												file = new File(name);
												name = file.getName();
												if (startsWithIgnoreCase(name, KEY_PROCESSOR_FILE_NAME)) {
													break;		// escape from loop
												}
											}
										}

										writeOutput("> Programming key processor 2...\r\n");
										setStatusText("Programming key processor 2...");
										if (AXIS64Programmer.downloadFile(new BufferedReader(new InputStreamReader(updateKeyProcessorZipStream)), Programmer.KEY_PROCESSOR_2, totalNumberOfLines) == false) {
											writeOutput("> Error: Failed to program key processor 2\r\n");
											setStatusText("Error: Failed to program key processor 2");
										}
										else {
											writeOutput("> Successfully programmed key processor 2\r\n");
										}

										// as we have now used the key processor zip stream we need to get it again
										updateKeyProcessorZipStream = new ZipInputStream(updateURL.openStream());
										while ((updateFile = updateKeyProcessorZipStream.getNextEntry()) != null) {
											if (!updateFile.isDirectory()) {    // checks that the zip file entry is not a directory
												name = updateFile.getName();
												file = new File(name);
												name = file.getName();
												if (startsWithIgnoreCase(name, KEY_PROCESSOR_FILE_NAME)) {
													break;		// escape from loop
												}
											}
										}

										writeOutput("> Programming key processor 3...\r\n");
										setStatusText("Programming key processor 3...");
										if (AXIS64Programmer.downloadFile(new BufferedReader(new InputStreamReader(updateKeyProcessorZipStream)), Programmer.KEY_PROCESSOR_3, totalNumberOfLines) == false) {
											writeOutput("> Error: Failed to program key processor 3\r\n");
											setStatusText("Error: Failed to program key processor 3");
										}
										else {
											writeOutput("> Successfully programmed key processor 3\r\n");
										}
									}
									else if (startsWithIgnoreCase(name, FUNCTION_KEY_PROCESSOR_FILE_NAME)) {   // is this the function key processor file
										// this is the function key process file

										writeOutput("> Programming function key processor...\r\n");
										setStatusText("Programming function key processor...");
										if (AXIS64Programmer.downloadFile(new BufferedReader(new InputStreamReader(updateZipStream)), Programmer.FUNCTION_KEY_PROCESSOR, totalNumberOfLines) == false) {
											writeOutput("> Error: Failed to program the function key processor\r\n");
											setStatusText("Error: Failed to program the function key processor");
										}
										else {
											writeOutput("> Successfully programmed the function key processor\r\n");
										}
									}
									else if (startsWithIgnoreCase(name, MAIN_PROCESSOR_FILE_NAME)) {   // is this the main processor file
										// this is the main processor file

										writeOutput("> Programming main processor...\r\n");
										setStatusText("Programming main processor...");
										if (AXIS64Programmer.downloadFile(new BufferedReader(new InputStreamReader(updateZipStream)), Programmer.MAIN_PROCESSOR, totalNumberOfLines) == false) {
											writeOutput("> Error: Failed to program the main processor\r\n");
											setStatusText("Error: Failed to program the main processor");
										}
										else {
											writeOutput("> Successfully programmed the main processor\r\n");
										}
									}
									else if (startsWithIgnoreCase(name, VELOCITY_TABLE_FILE_NAME)) {   // is this the velocity table file
										// this is the velocity table file

										writeOutput("> Programming velocity table...\r\n");
										setStatusText("Programming velocity table...");
										if (AXIS64Programmer.downloadFile(new BufferedReader(new InputStreamReader(updateZipStream)), Programmer.VELOCITY_TABLE, totalNumberOfLines) == false) {
											writeOutput("> Error: Failed to program the velocity table\r\n");
											setStatusText("Error: Failed to program the velocity table");
										}
										else {
											writeOutput("> Successfully programmed the velocity table\r\n");
										}
									}
								}

								updateZipStream.closeEntry();
							}

							setStatusText("Programmed sucessfully");
						}
						else {
							writeOutput("> Error: Entering program mode\r\n");
							setStatusText("Error: Entering program mode");
						}
					}
					catch (IOException ex) {
						writeOutput("> Error: Accessing update ZIP file\r\n");
						setStatusText("Error: Accessing update ZIP file");
					}

					AXIS64Programmer.exitProgramMode(); // exit program mode
				}
				else {
					writeOutput("> Error: No update zip file located\r\n");
					setStatusText("Error: No update zip file located");
				}

				AXIS64Programmer.connect(false);    // disconnect from the programming comm port
			}
			else {
				writeOutput("> Error: Unable to connect to the programming comm port\r\n");
				setStatusText("Error: Unable to connect to the programming comm port");
			}
			
			// this needs to move out of the worker thread!!!
			SwingUtilities.invokeLater(new Runnable() {
				public final void run() {
					Toolkit.getDefaultToolkit().beep();
					closeButton.setEnabled(true);
					JRootPane rootPane = getRootPane();
					rootPane.setDefaultButton(closeButton);
					closeButton.requestFocusInWindow();
				}
			});
		}
	}
	
	// constants
	private static final String KEY_PROCESSOR_FILE_NAME = "Mkey";   // beginning of the key processor file name
	private static final String FUNCTION_KEY_PROCESSOR_FILE_NAME = "Mfunc";   // beginning of the function key processor file name
	private static final String MAIN_PROCESSOR_FILE_NAME = "Mcon";   // beginning of the main processor file name
	private static final String VELOCITY_TABLE_FILE_NAME = "Mvel";   // beginning of the velocity table file name
	
	// variables
	private boolean basicView;					// flag to denote which view mode we are in, basic or advanced. Set to basic.
	private TextAreaUpdater output;				// output
	private String portName;					// port name
	private URL updateURL;						// update url

	/** Creates a new form MainWindow */
	protected UpdateDialog(Frame owner, String portName, URL updateURL) {
		super(owner, true);

		basicView = true;
		
		this.portName = portName;
		this.updateURL = updateURL;

		initComponents();
		
		closeButton.setEnabled(false);		// just to be on the safe side
    
		output = new TextAreaUpdater(serialTextPane);
		
		/** The main window has two states a basic view & an advanced view.
			On startup the basic view should be seen by the user */
		setSize((int)getSize().getWidth(), (int)serialTextScrollPane.getLocation().getY() + (int)getSize().getHeight() - (int)getContentPane().getSize().getHeight());	// set the height of the window to the top of the serial text pane with an adjustment for the 
																																										// height of the title bar which is the difference between the height of the frame and the content pane
		status.setText("");
		programmingProgressBar.setValue(0);
	}
	
	private static final boolean startsWithIgnoreCase(String string, String prefix) {
		int stringLength = string.length();
		int prefixLength = prefix.length();
		if (stringLength > prefixLength) {
			string = string.substring(0, prefixLength);
		}
		boolean startsWith = string.equalsIgnoreCase(prefix);
		return startsWith;
	}
	
	private final void writeOutput(String text) {
		output.append(text);
	}
	
	private final void closeDialog() {
		if (closeButton.isEnabled()) {
			setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		}
		else {
			Toolkit.getDefaultToolkit().beep();
			JOptionPane.showMessageDialog(this, "Cannot close the dialog box while firmware update is in progress.",
				MainFrame.APP_NAME, JOptionPane.WARNING_MESSAGE);
			setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		}
	}
	
	/*
	private final void closeDialog() {
		boolean close = true;
		if (updateThread != null) {
			if (updateThread.isAlive()) {
				Toolkit.getDefaultToolkit().beep();
				JOptionPane.showMessageDialog(this, "Cannot close dialog box while firmware update is in progress.",
					MainFrame.APP_NAME, JOptionPane.WARNING_MESSAGE);
				close = false;
			}
		}
		if (close) {
			setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		}
		else {
			setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		}
	}
	*/

	/** This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        serialTextScrollPane = new javax.swing.JScrollPane();
        serialTextPane = new javax.swing.JTextArea();
        basicAdvancedView = new javax.swing.JButton();
        programmingProgressBar = new javax.swing.JProgressBar();
        statusLabel = new javax.swing.JLabel();
        closeButton = new javax.swing.JButton();
        status = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Update");
        setModal(true);
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
            public void windowOpened(java.awt.event.WindowEvent evt) {
                formWindowOpened(evt);
            }
        });

        getAccessibleContext().setAccessibleParent(null);
        serialTextPane.setColumns(20);
        serialTextPane.setEditable(false);
        serialTextPane.setLineWrap(true);
        serialTextPane.setRows(5);
        serialTextPane.setEnabled(false);
        serialTextScrollPane.setViewportView(serialTextPane);

        basicAdvancedView.setMnemonic('A');
        basicAdvancedView.setText("Advanced >>");
        basicAdvancedView.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                basicAdvancedViewActionPerformed(evt);
            }
        });

        statusLabel.setText("Status:");

        closeButton.setMnemonic('o');
        closeButton.setText("Close");
        closeButton.setEnabled(false);
        closeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                closeButtonActionPerformed(evt);
            }
        });

        status.setText("<status>");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, serialTextScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 390, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createSequentialGroup()
                        .add(statusLabel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(status, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 349, Short.MAX_VALUE))
                    .add(layout.createSequentialGroup()
                        .add(basicAdvancedView)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 232, Short.MAX_VALUE)
                        .add(closeButton))
                    .add(programmingProgressBar, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 390, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(statusLabel)
                    .add(status))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(programmingProgressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(closeButton)
                    .add(basicAdvancedView))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(serialTextScrollPane, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 200, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void formWindowOpened(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowOpened
		UpdateThread updateThread = new UpdateThread();
		updateThread.start();
	}//GEN-LAST:event_formWindowOpened

	private void closeButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeButtonActionPerformed
		dispose();
	}//GEN-LAST:event_closeButtonActionPerformed
	
	private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
		closeDialog();
	}//GEN-LAST:event_formWindowClosing

	private void basicAdvancedViewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_basicAdvancedViewActionPerformed
		// The Basic/Advanced view button has been pressed
		if (basicView) {
			// We're are already in basic view, so display the advanced
			int gapBelowSerialTextPane = (((int)getSize().getWidth() - (int)serialTextScrollPane.getSize().getWidth())/2);    // gets the gap between the side of the serial text pane and the side of the main window
			// sets the height of the window to the bottom of the serial text pane plus the  
			// gap between the side of the serial text pane and the side of the main window 
			// with an adjustment for the height of title bar which is the difference
			// between the height of the frame and the content pane
			serialTextPane.setEnabled(true);
			setSize((int)getSize().getWidth(),(int)serialTextScrollPane.getLocation().getY() + (int)serialTextScrollPane.getSize().getHeight() + gapBelowSerialTextPane + (int)getSize().getHeight() - (int)getContentPane().getSize().getHeight());
			
			basicAdvancedView.setText("Basic <<");	// Set the Basic/Advanced
			// view button text to basic
			basicView = false;	// We're no longer in basic view mode
		}
		else {
			// We're in advanced view, so display the basic
			setSize((int)getSize().getWidth(),(int)serialTextScrollPane.getLocation().getY() + (int)getSize().getHeight() - (int)this.getContentPane().getSize().getHeight());    // set the height of the window to the top of the serial text pane with an adjustment for the 
			serialTextPane.setEnabled(false);
																							// height of the title bar which is the difference between the height of the frame and the content pane
			basicAdvancedView.setText("Advanced >>");	// Set the Basic/Advanced
			// view button text to advanced
			basicView = true;	// We're in basic view mode
		}

		validate();		// Update visbility of components on the main window
	}//GEN-LAST:event_basicAdvancedViewActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton basicAdvancedView;
    private javax.swing.JButton closeButton;
    private javax.swing.JProgressBar programmingProgressBar;
    private javax.swing.JTextArea serialTextPane;
    private javax.swing.JScrollPane serialTextScrollPane;
    private javax.swing.JLabel status;
    private javax.swing.JLabel statusLabel;
    // End of variables declaration//GEN-END:variables
}
